A - HyperScore ToolKit Example Script - V5.2

Outline

	A - HyperScore ToolKit Example Script - V5.2
	B - Introduction
	C - Basic Event Creation
	D - EventList Creation and Interaction
	E - Simple EventList Processing
	F - EventGenerators
	G - EventModifiers
	H - Scheduling and I/O
	I - Event- and EventListEditors
	J - Extended Examples
	K - Making Examples
	L - Status, Known Bugs


B - Introduction

This file is a script for demonstraing the various representations and operations of the HyperScore ToolKit, version 5.0
It progresses from very simple examples of creating musical objects to more complex generative event stream modeling classes.
Each single example is written as a Smalltalk-80 block, enclosed in square brackets, so that selection in a text editor is simpler.  
It is intended that this list be evaluated step-by-step in a Smalltalk-80 workspace to achieve the full effect.
For the user who is interested in more detail, we can suggest inspecting the results of some of these expressions (e.g., the event list creation examples) or running MessageTallies on the expressions that play or do interesting processing.

If you're running the demo for the first time, skip ahead to the section on Scheduling and look at the initialization messages for the three scheduling schemes described there.
Depending upon your platform and virtual machine, you'll choose one of these and set up (initialize) one of these schedulers.

Now look at the method EventList>>play (in a Browser) to make sure that  is in sync with the scheduler and/or primitives you're using.

You can now progress to the simple eventList examples and try (for example) the scale playing messages--i.e., select and 'do' the first '['-ed expression, and then the second.
You should hear a C-major scale.
If there's a problem, check the scheduler class's test messages, the play method and the scale example before proceeding.


C - Basic Event Creation

"create and inspect a simple note event"
[(NoteEvent dur: 1/4  pitch: 'c3'  ampl: 'mf') inspect]

"make an interesting one with some new properties"
[((NoteEvent dur: 1/4  pitch: 'c3'  ampl: 'mf') 
		color: #green; accent: #sforzando) inspect]

"Demonstrate Abstract Music Magnitudes"
(Duration value: 1/16) asMS	"returns 62" 
(Pitch value: 36) asHertz 		"returns 261.623" 
(Amplitude value: 'ff') asMidi	"returns 106" 

"create a Midi-specific Note"
[(NoteElement new: 48 at: 0 dur: 100 vel: 100 voice: 1) inspect]

"Make a note with default parameters and turn it on."
NoteElement new perform.


D - EventList Creation and Interaction

"define a simple scale"
[(EventList newNamed: #scale1)
	add: (NoteEvent dur: 1/4  pitch: 'c3'  ampl: 'f'); 
	add: (NoteEvent dur: 1/4  pitch: 'd3'  ampl: 'f'); 
	add: (NoteEvent dur: 1/4  pitch: 'e3'  ampl: 'f'); 
	add: (NoteEvent dur: 1/4  pitch: 'f3'  ampl: 'f'); 
	add: (NoteEvent dur: 1/4  pitch: 'g3'  ampl: 'f'); 
	add: (NoteEvent dur: 1/4  pitch: 'a4'  ampl: 'f'); 
	add: (NoteEvent dur: 1/4  pitch: 'b4'  ampl: 'f'); 
	add: (NoteEvent dur: 1/4  pitch: 'c4'  ampl: 'f')]

"Edit or play the scale"
(EventList named: #scale1) edit
[(EventList named: #scale1) play]

MessageTally spyOn: [(EventList named: #scale1) play]

"make a list called random1 with 20 random notes"
[ | pattern rand |
pattern _ EventList newNamed: #random1.
rand _ Random new.
20 timesRepeat:
	[pattern add: 
		(NoteEvent 
					"50-150 ms duration"
			dur: 50 + ((rand next * 100) truncated)	
					"pitch MIDI 30-60"
			pitch: 30 + ((rand next * 30) truncated)	
					"ampl. 50-110"
			ampl: 50 + ((rand next * 60) truncated))]]	

[(EventList named: #random1) play]
(EventList named: #random1) edit

"Simple heirarchical EventList declaration"
"Arpeggiated chord followed by block chord"
[(EventList newNamed: #demo1)
	add: (NoteEvent dur: 500  pitch: 48  ampl: 100); 
	add: (NoteEvent dur: 500  pitch: 52  ampl: 100); 
	add: (NoteEvent dur: 500  pitch: 55  ampl: 100); 
	add: (EventList new 
		add: (NoteEvent  dur: 500  pitch: 48  ampl: 100)  at: 0; 
		add: (NoteEvent  dur: 500  pitch: 52  ampl: 100)  at: 0; 
		add: (NoteEvent  dur: 500  pitch: 55  ampl: 100)  at: 0) ]

[(EventList named: #demo1) play]

"the same with staccato"
[(EventList newNamed: #demo1)
	add: (NoteEvent dur: 400  pitch: 36  ampl: 100); 
	add: (NoteEvent dur: 400  pitch: 40  ampl: 100) at: 1000; 
	add: (NoteEvent dur: 400  pitch: 43  ampl: 100) at: 2000; 
	add: (EventList new 
		add: (NoteEvent  dur: 400  pitch: 36  ampl: 100)  at: 0; 
		add: (NoteEvent  dur: 400  pitch: 40  ampl: 100)  at: 0; 
		add: (NoteEvent  dur: 400  pitch: 43  ampl: 100)  at: 0) 
			at: 3000 ]

[(EventList named: #demo1) edit]

"a deeper hierarchical EL'
[(EventList newNamed: #demo2)
							"add demo1"
	add: (EventList named: #demo1);
							"and a delayed copy"
	add: ((EventList named: #demo1) copy) at: 1300;
							"and a delayed and transposed one"
	add: ((EventList named: #demo1) copy pitch: 12) at: 2500 ]	

[(EventList named: #demo2) play]

"expand it out to a flat list"
(EventList named: #demo2) expanded inspect

"repetitions of the same note; a roll"
[ | temp |
temp _ EventList newNamed: #roll1.
(0 to: 4000 by: 50) do: 	"4 seconds, 20 per second"
	[ :index |
	temp add: (NoteEvent dur: 200 pitch: 36 ampl: 100) at: index ] ]

[(EventList named: #roll1) play]

"Midi-score declaration example"
[	| score |
	score _ Score new.
	score add: (NoteElement new: 60 at: 0 dur: 40).
	score add: (NoteElement new: 62 at: 50 dur: 40).
	score add: (NoteElement new: 64 at: 100 dur: 40).
	score add: (NoteElement new: 65 at: 150 dur: 40).
	score add: (NoteElement new: 67 at: 200 dur: 25).
	score add: (NoteElement new: 65 at: 225 dur: 25).
	score add: (NoteElement new: 64 at: 250 dur: 25).
	score add: (NoteElement new: 62 at: 275 dur: 25).
	score add: (NoteElement new: 60 at: 300 dur: 100).
"	score storeAdagioOnFile: 'test1'."
	score asEventList edit.
"	score play"]

"create a trivial score"
AdagioParser parse: (ReadStream on: 'a;b;c').

"more creation examples and reading/writing files later"


E - Simple EventList Processing

"inspect the dictionary of all global eventLists"
[EventListDictionaryInspectorView openWithButtons: 
			(EventListDictionaryInspector inspect: EventLists)]

"make eventList random1 legato (duty factor = 2)"
[(EventList named: #random1) notesDo: 
	[ :anEvent |
	anEvent duration: (anEvent duration * 2)]]

[(EventList named: #random1) play]
(EventList named: #random1) edit

"setting the amplitude of the notes of random1"
(EventList named: #random1) notesDo: 
	[ :anEvent |
	anEvent ampl: (Amplitude value: 120)]

"adding some jitter to the start times"
[ | rand |
rand _ Random new.
(EventList named: #roll1) do: 
	[ :anAssoc |
	anAssoc key: (anAssoc key + 
		((rand next - 0.5) * 0.2 * (anAssoc key)) rounded)]]

[(EventList named: #roll1) play]

"Examples of EventList processing and filter messages"
(EventList named: #le1) setVoice: 1  "set properties"
(EventList named: #n17sc) addVoice: 1  "add constant"
(EventList named: #re3) chooseVoice: #(1 2)  "select among"
(EventList named: #scale1) spreadLoudness: 10 "% random smear"
(EventList named: #scale1) scaleDuration: 0.1 "scale factor"
	
	"many more messages of this type available, system easily extensible"

"Processing/Mixing example"
[(EventList newNamed: #mix3)
	add: (EventList named: #le1);
	add: ((EventList named: #le2) copy) at: 4300;
	add: ((EventList named: #re3) copy) at: 5500;
	add: ((EventList named: #re3) copy) at: 7500;
	add: ((EventList named: #re4) copy) at: 9500;
	add: ((EventList named: #le1) copy) at: 18000;
	add: ((EventList named: #le2) copy) at: 23000;
	add: ((EventList named: #re3) copy) at: 26000;
	add: ((EventList named: #re3) copy) at: 30000;
	add: ((EventList named: #re4) copy) at: 33000;
	add: ((EventList named: #le1) copy) at: 42000;
	add: ((EventList named: #le2) copy) at: 47000]


F - EventGenerators

"A Simple Progression - C Major cadence"
[(EventList newNamed: #progression1)
	add: ((Chord majorTetradOn: 'c4' inversion: 0) duration: 1/2);
	add: ((Chord majorTetradOn: 'f3' inversion: 2) duration: 1/2);
	add: ((Chord majorTetradOn: 'g3' inversion: 1) duration: 1/2);
	add: ((Chord majorTetradOn: 'c4' inversion: 0) duration: 1/2)]

[(EventList named: #progression1) play]
[(EventList named: #progression1) edit]
(EventList named: #progression1) expanded

"basic event generator types"
"arpeggiate a Chord"
[(Arpeggio on: 			"Arp on some given eventList"
		((Chord majorTetradOn: 'c3' inversion: 0) duration: 1.0) 
		delay: 100) ampl: 'f'; edit]	"given a delay for arpeggiation"

"set up an Ostinato on the scale"
(Ostinato newNamed: #scaleO onList: (EventList named: #scale1))
(Ostinato named: #scaleO) play
(Ostinato named: #scaleO) stop

"simple rolls/trills"
[((Roll length: 2.0 			"duration of roll"
		rhythm: 60 			"delay between notes"
		note: 'c4') 			"pitch"
		ampl: 'f') play]		"amplitude"

[((Trill length: 2.0
		rhythm: 40 
		notes: #(48 50)) 	"given array of pitches"
		ampl: 100) "edit" play]

"static stochastic wave/cloud"
[((Cloud dur: 2.0  				"duration"
		pitch: (48 to: 64)  		"pitch range--an interval"
		ampl: (80 to: 120) 		"amplitude range--an interval"
		voice: (1 to: 8)) 		"voice range--an interval"
		density: 15) 			"density per second"
		"edit" play ]

"static selection cloud"
[(SelectionCloud dur: 2  
		pitch: #('c3' 'f3' 'g3' 'a4' 'c4')	"select from this pitch array"
		ampl: #('f' 'f' 'mf')					"and this array of amplitude values"
		voice: #(0 1 2 6)					"and these voices"
		density: 16) 						"rate per second"
		 "edit" play]

"a low/rising crescendo cloud"
[(DynamicCloud dur: 6  
		pitch: #((30 to: 54)  (72 to: 72))		"starting and ending pitch ranges"
		ampl: #((40 to: 60) (80 to: 120))		"and amplitude ranges"
		voice: (1 to: 3) 								"static voice selection"
		density: 20) edit" play]

"textured roll"
[(DynamicCloud dur: 12
	pitch: (20 to: 20)
	ampl: #((20 to: 30) (100 to: 120))
	voice: (1 to: 1)
	density: 20) "edit" play ]

"pentatonic selection that focuses onto a trill"
[(DynamicSelectionCloud dur: 9
		pitch: #( #(40 43 45) #(53 57 60))	"starting and ending pitch sets"
		ampl: #(80 80 120)				"static amplitude set"
		voice: #(1 3 5 7)					"and voice set"
		density: 20) "edit"  play]

"create a pentatonic cluster/chord"
[EventList named: #clus1 
		fromPitches: #(36 38 40 43 45)]

"make a cluster on the given list's pitches"
[(Cluster dur: 2.0
		list: (EventList named: #clus1)
		ampl: 100
		voice: 6) "edit " play ]

"nicer cluster creation messages"
[(Cluster dur: 2.0
		pitchSet: #(48 49 50 51 52 53 54 55) 
		ampl: 100 voice: 6) play]

[(Cluster dur: 2.0 pitchSet: #(24 30 35 41 42 54 61) 
		ampl: 100 voice: 6) "edit" play]

"Bell Peals"
[	| p1 |
	p1 _ Peal upon: #(60 64 65).
	p1 playOn: Midi durations: 800 meter: 400 at: Midi getClockTime]

[	| p1 p2 t |
	t _ Midi getClockTime.
	p1 _ Peal upon: #(82 79 77 74 72).
	p2 _ Peal upon: #(50 47 45 43).
	p2 playOn: Midi durations: 1400 meter: 700 at: t.
	p1 playOn: Midi durations: 400 meter: 90 at: t + 100]

[	| p1 |
	p1 _ Peal upon: #(60 64 65).
	p1 playOn: (EventList newNamed: #peal1) durations: 400 meter: 200 at: 0]

[	| p1 p2 |
	p1 _ Peal upon: #(82 79 77 74 72).
	p2 _ Peal upon: #(50 47 45 43).
	p2 playOn: (EventList newNamed: #peal2) 
			durations: 600 meter: 400 at: 0.
	p1 playOn: (EventList newNamed: #peal3) 
			durations: 200 meter: 90 at: 0]

[	(Peal upon: #(60 64 65)) 
		playOn: (EventList newNamed: #peal1b) 
		durations: 400 meter: 400 
		at: 0]

[	(Peal upon: #(50 47 45 43)) 
		playOn: (EventList newNamed: #peal3b) 
		durations: 1400 meter: 700 at: 0.
	(Peal upon: #(82 79 77 74 72)) 
		playOn: (EventList named: #peal3b) 
		durations: 200 meter: 90 at: 0]


G - EventModifiers

"Applying a Crescendo to an EventList using a LinSeg function"
[ | temp fcn |
	EventLists at: #roll2 put: 
		((Roll length: 4000 rhythm: 50 note: 36) ampl: 100) eventList.
	temp _ EventList named: #roll2.
	temp play.
	(Delay forMilliseconds: temp duration + 500) wait.
	fcn _ LinSeg from: #((0 @ 0) (1 @ 1)).
	temp apply: fcn to: #loudness.
	temp play]

"a cresc./decresc. example"
[ | score |
	score _ EventList newAnonymous.
	score addAll: (EventList named: #le1).
	score apply: 
		(LinSeg from: #((0 @ 0.1) (0.5 @ 1) (1 @ 0.1))) 
		to: #loudness.
	score play].

"Playing 'molto rubato'"
	| temp fcn |
	EventLists at: #roll2 put: 
		((Roll length: 4000 rhythm: 80 note: 36) ampl: 100) eventList.
	temp _ EventList named: #roll2.
	temp apply: 
			(LinSeg from: #((0@1) (0.35@1.3) (0.65@0.7) (1@1))) 
			to: #start.
	temp play.


H - Scheduling and I/O

"Reading/Writing external Files and Formats"
"standard dump format"
[(EventList named: #le1) storeOnFile: 'le1.el']

"dump all event lists into the scores directory"
EventLists associationsDo: [ :anAss |
		anAss value storeOnFile: 'Scores:', anAss key, '.el'] 

"Load EventLists fromFiles"
"File in EventLists"
[(FileDirectory filesMatching: 'SCORES:*.el') 
	do: [ :name | (FileStream oldFileNamed: name) fileIn]]

"Load Adagio Files"
[(FileDirectory filesMatching: 'SCORES:*.gio') 
	do: [ :name |
	EventLists at: 
			(name copyFrom: 8 to: name size - 4) asSymbol
		put: (AdagioParser parse: 
			(FileStream oldFileNamed: name)) asEventList]]

"read in OpCode format Midi Dumps"
(OpcodeVoice1 newOn: 'SCORES:n15.sc') 
	loadOnto: (EventList newNamed: #n15.sc)

"change delta-time encoding to duration"
(EventList named: #n15sc) setDurations

"filter non-notes"
(EventList named: #n15.sc) events: 
	((EventList named: #n15.sc) events removeAllSuchThat: 
			[ :item |
			item value duration == nil])

"scale tempo if desired"
(EventList named: #n18sc) scaleTempo: 0.8

"Create a new CMusic voice"
CmusicVoice newNamed: #violin
			onStream: (FileStream newFileNamed: 'rand.cm')

"play the random notes onto a cmusic formatted stream"
(EventList named: #random1) playOn: (Voice named: #violin)

(Voice named: #violin) close
(FileStream oldFileNamed: 'rand.cm') edit

"create a trivial score using Adagio Format"
AdagioParser parse: (ReadStream on: 'a;b;c')

(AdagioParser parse: 
		(FileStream oldFileNamed: 'SCORES:Allemande.gio'))
	 edit "eventList play" 

"SCHEDULERS"
"to change the scheduler used, look at the method EventList>>play.  
It's normally one of the following expressions:
	^self playOn: self voice at: (Time millisecondClockValue)
	^self asScore play
	^self playPrim"

"SCHEDULER 0--Voice/Device/Port approach in Smalltalk
	-most general, slowest and least useful"
"set up the Internally-scheduled SerialPort Midi world"
	MidiPort flush.
	MidiDevice initialize.
	EventScheduler initialize.
	Delay postSnapshot.

"create a MidiVoice on the MidiDevice on port 1 channel 1 and play onto it"
(EventList named: #scale1) voice: 
		(MidiVoice onDevice: (MidiDevice newOn: 1) 
				channel: 1)

"look at the global schedule"
EventScheduler schedule inspect

"SCHEDULER 1--John Maloney's MdiInterface Classes
	-uses Midi-specific Note and Score classes, so it's faster;
	-runs with or without special primitives"
"World 1: Set up MIDIInterface--with primitives"
[	MidiInterface initialize; debugOff; primOn.
	MidiInterface openPort: 0]

"--without primitives"
[	MidiInterface initialize; debugOff; primOff.
	MidiInterface openPort]

"now play the scale as a score object"
[	(EventList named: #scale1) asScore play.]

"flush the interface"
	MidiInterface allNotesOff

'Record and Edit an EventList'
	"record until mouse button pressed"
| recorder score |
recorder _ (MidiRecorder new) record.
score _ recorder.
score storeAdagioOnFile: 'bachTest3'

"will record until a mouse click"
((MidiRecorder new) record) asEventList edit 

MidiInterface debugOn; primOff.
MidiInterface debugOff; primOn.
(EventList named: #scale1) asScore play

"Play the score from the beginning, normal speed."
(EventList named: #Song1455) asScore playFrom: 0 rate: 1.

"Play it from the 10 second point at 1.5 times normal speed."
(EventList named: #Song1455) asScore playFrom: 100 rate: 1.5.

"Play the score as fast as possible (used for testing and benchmarks)."
(EventList named: #Song1455) asScore testPlay.

"SCHEDULER 3-Guy and Lee's C-language scheduler with user primitive interface
	-the fastest;
	-primitives exist for Mac and Atari"
"World 3: MIDI interface using a primitive scheduler"
Midi open
Midi openAndInitialize
Midi fromMouse
Midi getClockTime
Midi getTickSize
Midi setClockTime: 0
Midi setTickSize: 10 "in MS"

(EventList named: #le2) voice: PrimMidiVoice new; play

"roll at 20 notes/second"
(EventList named: #test2) playPrim"


I - Event- and EventListEditors

"open one with an instrument selector sub view"
[HSEventListView openWithSelectorOn: 
		EventListEditor defaultEventList]

"open an editor with loudness slider and instr. selector inset"
[HSEventListView openWithInsetSelectorAndSliderOn: 
		EventListEditor defaultEventList]

"now one with a button row; the buttons mean: 
	scroll, play, select, group, delete, inspect, redraw and erase"
[HSEventListView openWith8ButtonsOn: 
		EventListEditor defaultEventList]

"with dual button rows and spline tempo editor"
[HSEventListView openWith7ButtonsAndSplineOn: 
		EventListEditor defaultEventList]

"And now with fancy scroll bars"
[HSEventListView openWith8ButtonsAndHAndVScrollingOn: 
		EventListEditor defaultEventList]

"Examples of Composite EventListEditors on two parallel EventLists"
[HSEventListView openDualSimple]
[HSEventListView openDualFancy]
[HSEventListView openDualFancy]

"Cheap Common-Music Notation Editor (Warning, this ain't meant to be 'real' CMN)"
[PianoView open2StaffOn: (EventList named: #Allemande)]
[PianoView open2StaffOn: (EventList named: #Song1455)]
[PianoView open2StaffOn: (EventList named: #scale1)]

[StringQView open4StaffOn: (EventListEditor defaultEL)]

PianoRollView openOn: (Score new)
PianoRollView openOn: (Score example)


J - Extended Examples

"Cloud Mixes"
[#(	test1
	test2
	cMix1
	cFade1
	le1
	le2
	re2
	mix1
	mix3
) do: [ :name | (EventList named: name) play ]]


K - Making Examples

[	| list |
	list _ (EventList newNamed: #scaleLong).
	0 to: 48 do: [ :count |
		list add: (NoteEvent dur: 1/2 pitch: 24 + count ampl: 'f')]]

[	| list |
	list _ (EventList newNamed: #scale2).
	0 to: 48 do: [ :count |
		list add: (NoteEvent dur: 1/4 pitch: 24 + count ampl: 'f')]]

[	| list |
	list _ (EventList newNamed: #scale3).
	0 to: 24 do: [ :count |
		list add: (NoteEvent dur: 1/4 pitch: 24 + count ampl: 'f') 
				at: (count * 250)].
	0 to: 24 do: [ :count |
		list add: (NoteEvent dur: 1/4 pitch: 48 + count ampl: 'f') 
				at: (count * 250)]]

[	| p1 |
	p1 _ Peal upon: #(60 64 65).
	p1 playOn: (EventList newNamed: #peal1) durations: 400 meter: 200 at: 0]

[	| p1 p2 |
	p1 _ Peal upon: #(82 79 77 74 72).
	p2 _ Peal upon: #(50 47 45 43).
	p2 playOn: (EventList newNamed: #peal2) 
			durations: 600 meter: 400 at: 0.
	p1 playOn: (EventList newNamed: #peal3) 
			durations: 200 meter: 90 at: 0]

[	(Peal upon: #(60 64 65)) 
		playOn: (EventList newNamed: #peal1b) 
		durations: 400 meter: 400 
		at: 0]

[	(Peal upon: #(50 47 45 43)) 
		playOn: (EventList newNamed: #peal3b) 
		durations: 1400 meter: 700 at: 0.
	(Peal upon: #(82 79 77 74 72)) 
		playOn: (EventList named: #peal3b) 
		durations: 200 meter: 90 at: 0]

[	(Peal upon: #(36 41 43)) 
		playOn: (EventList newNamed: #peal4A) durations: 250 meter: 250 
				at: 0.
	(Peal upon: #(36 41 43)) 
		playOn: (EventList named: #peal4A) durations: 250 meter: 250 
				at: 6000.
	(Peal upon: #(60 65 67 69)) 
		playOn: (EventList named: #peal4A) durations: 125 meter: 125 
				at: 0]

"read in all Adagio files--example"
"see the method EventList class loadAllFrom: directoryName"
"N.B. This is done by the build script"

[(FileDirectory filesMatching: 'Scores:*.gio') do: [ :name |
	EventLists at: (name copyFrom: 8 to: name size - 4) asSymbol
		put: (AdagioParser parse: (FileStream oldFileNamed: name)) asEventList]]

"transpose the song lower"
[	| new |
	new _ EventList newNamed: #Song1455b.
	(EventList named: #Song1455) do:
		[ :anA |
			new add: (anA value deepCopy pitch: 
				 (Pitch value: anA value pitch - 12)) 
				at: anA key]


L - Status, Known Bugs

Still to implement:
	tempo spline scrolling 
	better handling of large lists - cache multiple forms
	EventModifier processing in event mappers
	StringQView is not implemented 

After opening an editor on a non-empty list, one must say redraw before it displays the notes.

Vertical scrollers with bead gauges are upside-down (according to the normal kinaesthetics of scroll bars) (I prefer it this way...)

The PianoRollEditor's scroll bar is way off.

The schedulers are ranked according to their real-time performance as follows:
	STP's simple voices and devices- mediocre
	JHM's more sophisticated MidiInterface - better
	LB/GEG's primitive output driver - good

The primitive scheduler is about 10% fast

LastEditDate: 2 April 1989


