EventList variableSubclass: #EventGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
EventGenerator comment:
'This class is the parent of most of the simple motive classes in this category.
An EventGenerator can create an EventList using its given parameters and a function (block) to use to create Event sequences.  See the subclasses for examples.'!


!EventGenerator methodsFor: 'playing'!

edit
	"edit me"

	self eventList edit!

eventList
	"return an eventList for me"

	self subclassResponsibility!

play
	"play me"

	self eventList play
"	self asScore play"! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EventGenerator class
	instanceVariableNames: ''!


!EventGenerator class methodsFor: 'accessing instances'!

all
	"Find the and return instances of me or any subclasses of me."

	| coll |
	coll _ Set new.
	self allSubInstancesDo: [ :anI | coll add: anI ].
	^coll!

allPlaying
	"Find all playing instances of me or my subclasses."

	| coll |
	coll _ Set new.
	self allSubInstancesDo: [ :anI | anI playing ifTrue: [coll add: anI]].
	^coll!

named: aName 
	"Find the named instance of me or a subclass of me."

	^EventLists at: aName ifAbsent: 
		[Events at: aName ifAbsent: [nil]]!

stopAll
	"Find all playing instances of me or my subclasses and terminate them."

	self allSubInstancesDo: [ :anI | anI playing ifTrue: [anI stop]].! !

!EventGenerator class methodsFor: 'examples'!

exampleRand
	"play a random example from the given set of examples in my subclasses"
	"[Sensor anyButtonPressed] whileFalse: [EventGenerator exampleRand]"

	(Smalltalk at: 
			(#(Cloud DynamicCloud SelectionCloud DynamicSelectionCloud Peal) atRandom)) 
		exampleRand! !

EventGenerator variableSubclass: #Ostinato
	instanceVariableNames: 'list playing process '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
Ostinato comment:
'Ostinati can repeat their basic eventList without end, if necessary.

Instance variables:
	list <EventList> my riff
	playing <Boolean> am i on?
	process <Process> the process that''s playing me
'!


!Ostinato methodsFor: 'initializing'!

initializeAnonymous
	"set up a new Ostinato."

	super initializeAnonymous.
	playing _ false.!

initializeNamed: aName
	"set up a new Ostinato."

	super initializeNamed: aName.
	playing _ false.! !

!Ostinato methodsFor: 'accessing'!

events
	^list events!

list
	^list!

list: anEL
	list _ anEL! !

!Ostinato methodsFor: 'playing'!

play
	"use the prims by default"

	self playPrimAt: 0!

play: numberOfTimes
	"return an event list from the given number of times playing"

	| time pattern |
	time _ 0.
	pattern _ EventList newAnonymous.
	numberOfTimes timesRepeat:
		[pattern add: self events at: time.
		time _ time + self duration ].
	pattern play!

playOn: aVoice at: firstTime
	"play an ostinato until turned off."

	| startTime realTime | 
	startTime _ firstTime.
	playing _ true.
	process _ [[playing] whileTrue: 
			[list playOn: aVoice at: startTime.
			startTime _ startTime + list duration.
			(Delay untilMilliseconds: startTime - 500) wait]] forkAt: 5!

playPrimAt: firstTime
	"play an ostinato until turned off."

	| startTime realTime | 
	startTime _ firstTime.
	realTime _ Time millisecondClockValue.
	playing _ true.
	process _ [[playing] whileTrue: 
			[list playPrimAt: startTime.
			startTime _ startTime + list duration.
			(Delay untilMilliseconds: realTime + startTime - 500) wait.
		Transcript show: 'x '. ]] fork"At: 5"!

stop
	"stop a playing Ostinato."
		
	playing _ false.
"	self list stop"! !

!Ostinato methodsFor: 'printing'!

printOn: aStream

	self name isNil ifFalse: [
		aStream nextPutAll: 'Ostinato named: ', self name asString, ' on EventList: (']
	ifTrue: [
		aStream nextPutAll: 'Ostinato on EventList: ('].
	aStream cr.
	list printOn: aStream.
	aStream nextPutAll: ') playing= ', playing printString; cr! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Ostinato class
	instanceVariableNames: ''!


!Ostinato class methodsFor: 'instance creation'!

named: aSymbol onList: anEL 
	"create an Ostinato on the given EventList."

	| me |
	me _ self new initializeNamed: aSymbol.
	me list: anEL.
	^me!

named: aSymbol onList: anEL dur: theDur
	"create an Ostinato on the given EventList."

	| me |
	me _ self new initializeNamed: aSymbol.
	me list: anEL.
	me dur: theDur.
	^me!

newNamed: aSymbol onList: anEL 
	"create an Ostinato on the given EventList."

	| me |
	me _ self new initializeNamed: aSymbol.
	me list: anEL.
	^me!

onList: anEL 
	"create an Ostinato on the given EventList."

	| me |
	me _ self new initializeAnonymous.
	me list: anEL.
	^me!

onList: anEL dur: theDur
	"create an Ostinato on the given EventList."

	| me |
	me _ self new initializeAnonymous.
	me list: anEL.
	me dur: theDur.
	^me! !

!Ostinato class methodsFor: 'examples'!

ostinatoExamples
	"create and edit some ostinati"

	"Ostinato onList: (EventList named: #scale1)"
	"Ostinato onList: (EventList named: #scale1) edit"
	"Ostinato newNamed: #scaleO onList: (EventList named: #scale1)"
	"(Ostinato named: #scaleO) play"
	"(Ostinato named: #scaleO) playOn: 1 at: 0"
	"(Ostinato named: #scaleO) stop"! !

EventGenerator variableSubclass: #Cluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
Cluster comment:
'Cluster is an abstraction of a set of simultaneous events.
A cluster need only have a set of pitches.
'!


!Cluster methodsFor: 'playing'!

eventList
	"make the cluster's events"
	"assume that the receiver's pitch, loudness and durations are actually intervals"

	| pattern rand number length start density |
	pattern _ EventList newAnonymous.
	self events do:
		[ :anEvent |
		pattern add: (NoteEvent 
			dur: self duration
			pitch: ((anEvent isKindOf: Association) ifTrue: [anEvent value pitch] ifFalse: [anEvent])
			ampl: self ampl
			voice: self voice)
			at: 0 ].
	^pattern! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Cluster class
	instanceVariableNames: ''!


!Cluster class methodsFor: 'instance creation'!

dur: aD list: anEL ampl: anA
	"return an initialized Cluster with the given list as pitches"

	^self new duration: aD; 
		events: anEL; 
		loudness: anA!

dur: aD list: anEL ampl: anA voice: aV
	"return an initialized Cluster with the given list as pitches"

	^self new duration: aD; 
		events: anEL; 
		loudness: anA; 
		voice: aV!

dur: aD pitchSet: aColl ampl: anA voice: aV
	"return an initialized Cluster with the given list as pitches"

	^self new duration: aD; 
		events: aColl; 
		loudness: anA; 
		voice: aV! !

!Cluster class methodsFor: 'examples'!

example1
	"Cluster example1"
	"play a low 2 second cluster"

	(Cluster  dur: 2.0
		list: (EventList named: #clus1)
		ampl: (80 to: 100) 
		voice: 6) eventList "edit" "asScore" "playPrim"!

example2
	"Cluster example2"
	"play a low 2 second cluster"

	(Cluster  dur: 2.0
		pitchSet: #(48 49 50 51 52 53 54 55)
		ampl: 100
		voice: 6) edit play! !

Cluster variableSubclass: #Chord
	instanceVariableNames: 'root inversion '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
Chord comment:
'Instances of Chord are eventLists that can be created by giving them a root and inversion.
They can return eventLists.

Instance variables:
	root <Pitch> the root of the chord
	inversion <Integer> the inversion level (unused at present)'!


!Chord methodsFor: 'accessing'!

duration: aDuration
	"set the durations of my notes"
	
	| realDuration |
	aDuration class == Duration 
		ifTrue: [ realDuration _ aDuration ]
		ifFalse: [ realDuration _ Duration value: aDuration ].
	events do: [ :anAssociation |
		anAssociation value duration: realDuration ].
	duration _ realDuration asMS!

inversion: number
	"set the inversion of the receiver chord"

	inversion _ number!

root: tonic
	"set the root of the receiver chord"

	root _ tonic! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Chord class
	instanceVariableNames: ''!


!Chord class methodsFor: 'instance creation'!

majorTetradOn: tonic inversion: inversion
	"return a three-note major chord on the given tonic in the given inversion"
	"Chord majorTetradOn: 'c3' inversion: 0"

	| me rootNote |
	me _ self newAnonymous.
	me root: tonic;
		inversion: inversion.
	rootNote _ me defaultEventClass dur: 0 pitch: tonic ampl: 100.
	inversion = 0 ifTrue:
		[ me add: rootNote at: 0;
			add: (rootNote copy transposeBy: 4) at: 0;
			add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0 ].
	inversion = 1 ifTrue:
		[ me add: (rootNote copy transposeBy: 4) at: 0;
			add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0;
			add: (rootNote copy transposeBy: 16) at: 0 ].
	inversion = 2 ifTrue:
		[ me add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0;
			add: (rootNote copy transposeBy: 16) at: 0;
			add: (rootNote copy transposeBy: 19) at: 0 ].
	^me!

majorTriadOn: tonic inversion: inversion
	"return a three-note major chord on the given tonic in the given inversion"
	"Chord majorTriadOn: 'c3' inversion: 0"

	| me rootNote |
	me _ self newAnonymous.
	me root: tonic;
		inversion: inversion.
	rootNote _ me defaultEventClass dur: 0 pitch: tonic ampl: 100.
	inversion = 0 ifTrue:
		[ me add: rootNote at: 0;
			add: (rootNote copy transposeBy: 4) at: 0;
			add: (rootNote copy transposeBy: 7) at: 0 ].
	inversion = 1 ifTrue:
		[ me add: (rootNote copy transposeBy: 4) at: 0;
			add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0 ].
	inversion = 2 ifTrue:
		[ me add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0;
			add: (rootNote copy transposeBy: 16) at: 0 ].
	^me!

minorTetradOn: tonic inversion: inversion
	"return a three-note minor chord on the given tonic in the given inversion"
	"Chord minorTetradOn: 'c3' inversion: 0"

	| me rootNote |
	me _ self newAnonymous.
	me root: tonic;
		inversion: inversion.
	rootNote _ me defaultEventClass dur: 0 pitch: tonic ampl: 100.
	inversion = 0 ifTrue:
		[ me add: rootNote at: 0;
			add: (rootNote copy transposeBy: 3) at: 0;
			add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0 ].
	inversion = 1 ifTrue:
		[ me add: (rootNote copy transposeBy: 3) at: 0;
			add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0;
			add: (rootNote copy transposeBy: 15) at: 0 ].
	inversion = 2 ifTrue:
		[ me add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0;
			add: (rootNote copy transposeBy: 15) at: 0;
			add: (rootNote copy transposeBy: 19) at: 0 ].
	^me!

minorTriadOn: tonic inversion: inversion
	"return a three-note minor chord on the given tonic in the given inversion"
	"Chord minorTriadOn: 'c3' inversion: 0"

	| me rootNote |
	me _ self newAnonymous.
	me root: tonic;
		inversion: inversion.
	rootNote _ me defaultEventClass dur: 0 pitch: tonic ampl: 100.
	inversion = 0 ifTrue:
		[ me add: rootNote at: 0;
			add: (rootNote copy transposeBy: 3) at: 0;
			add: (rootNote copy transposeBy: 7) at: 0 ].
	inversion = 1 ifTrue:
		[ me add: (rootNote copy transposeBy: 3) at: 0;
			add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0 ].
	inversion = 2 ifTrue:
		[ me add: (rootNote copy transposeBy: 7) at: 0;
			add: (rootNote copy transposeBy: 12) at: 0;
			add: (rootNote copy transposeBy: 15) at: 0 ].
	^me! !

!Chord class methodsFor: 'examples'!

ChordExample
	"Create and edit some chords"

	"Chord majorTriadOn: 'c3' inversion: 0"
	"((Chord majorTetradOn: 'f2' inversion: 2) duration: 1.0) edit"
	"((Chord majorTetradOn: 'g2' inversion: 1) duration: 1.0) play"! !

Chord variableSubclass: #Arpeggio
	instanceVariableNames: 'delay '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
Arpeggio comment:
'Arpeggii can be created on Chords or other event lists and can step through their events (assumed to be simultaneous at the start) with the given delay time.

Instance variables:
	delay <Duration or Number> the delay betyween the onsets of my events'!


!Arpeggio methodsFor: 'accessing'!

delay: aValue
	"set the delay between the onset of my notes"

	delay _ aValue.
	self setDelays.! !

!Arpeggio methodsFor: 'playing'!

edit
	"edit me"

	ThreePartHSEventListView openWith8ButtonsAndHAndVScrollingOn: self!

eventList
	"return my event list"

	^self!

play
	"don't expand me"

	^self playAt: 0! !

!Arpeggio methodsFor: 'private'!

setDelays
	"set the start times of my notes"

	| counter |
	counter _ 0.
	events do: [ :anAssociation |
		anAssociation key: (anAssociation key + (counter * delay)).
		counter _ counter + 1 ]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Arpeggio class
	instanceVariableNames: ''!


!Arpeggio class methodsFor: 'instance creation'!

on: aChordOrList delay: aDelay
	"create an Arpeggio on the given chord or event list"

	| me |
	me _ self newAnonymous.
	me events: aChordOrList events copy.
	me delay: aDelay.
	^me! !

EventGenerator variableSubclass: #Roll
	instanceVariableNames: 'number delta noteDuration '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
Roll comment:
'Rolls repeat their single events.
It will eventually be possible to apply pitch, amplitude or duration envelopes to them as well.

Instance variables:
	number			<Integer> how many notes to play
	delta			<Number> delta time between events
	noteDuration	<Number> duration of events'!


!Roll methodsFor: 'initialize'!

length: aLength rhythm: aDuration note: aNote
	"make a new roll of the specified length..."

	delta _ aDuration.
	noteDuration _ aDuration.
	aLength class == Float "it's seconds"
		ifTrue: [number _ aLength * 1000 / aDuration]
		ifFalse: [number _ aLength / aDuration].
	pitch _ Pitch value: aNote.
	^self!

number: aNumber rhythm: aDuration note: aNote
	"make a new roll with the specified number of notes..."

	delta _ aDuration.
	number _ aNumber.
	noteDuration _ aDuration.
	pitch _ Pitch value: aNote.
	^self! !

!Roll methodsFor: 'accessing'!

duration
	"computer the duration"

	^duration == nil 
		ifFalse: [ duration ]
		ifTrue: [ number * noteDuration ]! !

!Roll methodsFor: 'playing'!

eventList
	"return an eventList for me"

	| newList |
	newList _ EventList newAnonymous.
	0 to: number do:
		[ :count |
			newList add: (self defaultEventClass 
					dur: noteDuration pitch: pitch ampl: loudness) 
				at: (delta * count) ].
	^newList! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Roll class
	instanceVariableNames: ''!


!Roll class methodsFor: 'instance creation'!

length: aNumber rhythm: aDuration note: aNote
	"return a new roll of the specified length..."

	^self new length: aNumber rhythm: aDuration note: aNote!

number: aNumber rhythm: aDuration note: aNote
	"return a new roll with the specified number of events..."

	^self new number: aNumber rhythm: aDuration note: aNote! !

!Roll class methodsFor: 'examples'!

rollExample
	"create and edit some rolls"

	"((Roll length: 2000 rhythm: 100 note: 48) ampl: 100) eventList"
	"((Roll length: 2000 rhythm: 100 note: 48) ampl: 100) eventList edit"
	"((Roll length: 2000 rhythm: 50 note: 60) ampl: 100) eventList playPrim"! !

EventGenerator variableSubclass: #Cloud
	instanceVariableNames: 'density '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
Cloud comment:
'Clouds are eventLists that are describe in terms of their contours or selection criteria.
This can be (e.g.,) POD-style specification of ranges for random selection, or selection from a given set of pitches.
See the class examples of my subclasses for description examples.

Instance variables:
	density		<Number> number of notes per second'!


!Cloud methodsFor: 'accessing'!

density: theDensity
	"set the density instance variable"

	density _ theDensity!

loudness: aValue
	"set my loudness - overrride setting it to be an Amplitude type"

	loudness _ aValue! !

!Cloud methodsFor: 'playing'!

eventList
	"make the cloud's events"
	"assume that the receiver's pitch, loudness and durations are actually intervals"

	| pattern rand number length start |
	pattern _ EventList new.
	rand _ Random new.
	number _ duration asFloat * density.
	length _ (1000 / density) rounded.
	start _ 0.
	number truncated timesRepeat:
		[pattern add: (NoteEvent 
			dur: (length * 2 "+ (2000 * rand next) rounded")
			pitch: ((pitch first) + 
					((rand next * (pitch last - pitch first)) rounded))
			ampl: ((loudness first) + 
					((rand next * (loudness last - loudness first)) rounded))
			voice: ((voice == nil 
						ifTrue: [nil] 
						ifFalse: [(voice first) + 
					((rand next * (voice last - voice first)) rounded)])))
			at: start.
		start _ start + length].
	^pattern!

eventList1
	"make the cloud's events"
	"assume that the receiver's pitch, loudness and durations are actually intervals"

	| pattern number length start rand |
	rand _ Random new.
	pattern _ EventList new.
	number _ duration asFloat * density.
	length _ (1000 / density) rounded.
	start _ 0.
	number truncated timesRepeat:
		[ pattern add: (NoteEvent 
			dur: (length * 2 + (2000 * rand next) rounded)
			pitch: ((pitch first) + 
					((rand next * (pitch last - pitch first)) rounded))
			ampl: ((loudness first) + 
					((rand next * (loudness last - loudness first)) rounded))
			voice: ((voice == nil 
						ifTrue: [nil] 
						ifFalse: [(voice first) + 
					((rand next * (voice last - voice first)) rounded)])))
			at: start.
		start _ start + length].
	^pattern!

score
	"make the cloud's events as a score object"
	"assume that the receiver's pitch, loudness and durations are actually intervals"

	| pattern number length start rand |
	rand _ Random new.
	pattern _ Score new.
	number _ duration asFloat * density.
	length _ (100 / density) rounded.
	start _ 0.
	number truncated timesRepeat:
		[ pattern add: ((NoteElement new: ((pitch first) + 
					((rand next * (pitch last - pitch first)) rounded)) 
				at: start
				dur: (length * (1 + (rand next * 2)) + (200 * rand next) rounded) 
				vel: ((loudness first) + 
					((rand next * (loudness last - loudness first)) rounded))) 
				chan: (voice == nil ifTrue: [nil] 
					ifFalse: [(voice first) + 
						((rand next * (voice last - voice first)) rounded)])).
		start _ start + length].
	^pattern! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Cloud class
	instanceVariableNames: ''!


!Cloud class methodsFor: 'instance creation'!

dur: aD pitch: aP ampl: anA
	"return an initialized Cloud"

	^self new duration: aD; 
		pitch: aP; 
		loudness: anA!

dur: aD pitch: aP ampl: anA voice: aVoice
	"return an initialized Cloud"

	^self new duration: aD; 
		pitch: aP; 
		loudness: anA; 
		voice: aVoice!

dur: aD pitch: aP ampl: anA voice: aVoice density: aDe
	"return an initialized Cloud"

	^self new duration: aD; 
		pitch: aP; 
		loudness: anA; 
		voice: aVoice; 
		density: aDe! !

!Cloud class methodsFor: 'examples'!

example1
	"Cloud example1"
	"play a low 2 second stochastic cloud"

	(Cloud  dur: 2  
		pitch: (48 to: 54)  
		ampl: (80 to: 120) 
		voice: (1 to: 8)
		density: 16) eventList "edit" "asScore" playPrim!

example2
	"Cloud example2"

	| rand pBase score |
	rand _ Random new.
	pBase _ 20 + (rand next * 60) rounded.
	score _ (Cloud  dur: (2 + (rand next * 6))
		pitch: (pBase to: (pBase + (rand next * 40)) rounded)  
		ampl: (80 to: 120) 
		voice: (1 to: 8)
		density: (rand next * 20)) eventList.
	Processor yield.
	score playPrim.
	(Delay forMilliseconds: (score duration + 1000)) wait!

example2a
	"Cloud example2"
	"play completely random clouds for a while"
		"[Sensor keyboardPressed] whileFalse:
			[Display black.
			Cloud example2].
		Sensor keyboard.
		ScheduledControllers restore"

	| rand pBase score |
	rand _ Random new.
	pBase _ 20 + (rand next * 60) rounded.
	score _ (Cloud  dur: (2 + (rand next * 6))
		pitch: (pBase to: (pBase + (rand next * 40)) rounded)  
		ampl: (80 to: 120) 
		voice: (1 to: 8)
		density: (rand next * 20)) eventList asScore.
	Processor yield.
	score play!

example3
	"Cloud example3"
	"play completely random clouds for a while"
		"[Sensor keyboardPressed] whileFalse:
			[Display black.
			Cloud example3].
		Sensor keyboard.
		ScheduledControllers restore"

	| rand pBase score |
	rand _ Random new.
	pBase _ 20 + (rand next * 40) rounded.
	score _ (Cloud  dur: (3 + (rand next * 9))
		pitch: (pBase to: (pBase + (rand next * 40)) rounded)  
		ampl: (40 to: 120) 
		voice: (1 to: 8)
		density: (rand next * 20)) score.
	Processor yield.
	score play!

exampleRand
	"Cloud exampleRand"

	| rand pBase score |
	rand _ Random new.
	pBase _ 20 + (rand next * 60) rounded.
	score _ (Cloud  dur: (2 + (rand next * 6))
		pitch: (pBase to: (pBase + (rand next * 40)) rounded)  
		ampl: (80 to: 120) 
		voice: (1 to: 8)
		density: (rand next * 20)) eventList.
	Processor yield.
	score playPrim.
	(Delay forMilliseconds: (score duration + 1000)) wait!

exampleX
	"Cloud exampleX"

	| rand pBase score |
	rand _ Random new.
	pBase _ 20 + (rand next * 60) rounded.
	score _ (Cloud  dur: (2 + (rand next * 6))
		pitch: (pBase to: (pBase + (rand next * 40)) rounded)  
		ampl: (80 to: 120) 
		voice: (1 to: 8)
		density: (rand next * 20)) eventList.
	Processor yield.
	score playPrim.
	(Delay forMilliseconds: (score duration + 1000)) wait! !

SequenceableCollection variableSubclass: #Peal
	instanceVariableNames: 'base directions position finished '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
Peal comment:
'Peals are repetitive note-generating sequences.  This implementation was written by Mark Lentczner (mark@Apple.com) in 1986.

Instance variables:
	base <Collection of Numbers> the pitches i use
	directions <Array of +-1> the direction of the current sub-sequence
	position <Number> the current index
	finished <Boolean> an i done?'!


!Peal methodsFor: 'access'!

at: index
	"Answer the element in the base collection currently mapped into the index position."

	^ base at: (self basicAt: index)!

at: index put: object

	^ self error: 'Can''t modify a peal by at:put:'!

currentChange
	"Answer the base as mapped by the current change."

	^(1 to: self size) collect: [ :i  | self at: i ]!

finished
	"Answer is the peal has completed it's last change"

	^ finished!

size
	"Redone here because SequencableCollection overrides it."

	^ self basicSize! !

!Peal methodsFor: 'changing'!

change
	"Produce the next change in the peal"

	self changeOrder: 1 bounds: (1 to: self size)! !

!Peal methodsFor: 'playing'!

playOn: out durations: dur meter: mtr at: start
	"Play the receiver"

	| time siz |
	time _ start.
	siz _ self size.
	[finished] whileFalse:
		[1 to: siz do: 
			[ :i | out play: (self at: i) at: time dur: dur amp: 100. 
				time _ time + mtr ].
		self change.
		1 to: siz - 1 do: 
			[ :i | out play: (self at: i) at: time dur: dur amp: 100. 
				time _ time + mtr ].
		out play: (self at: siz ) at: time dur: dur + mtr amp: 100.  
				time _ time + mtr + mtr.
		self change].
	1 to: siz do: 
			[ :i | out play: (self at: i) at: time dur: dur amp: 100. 
				time _ time + mtr ].! !

!Peal methodsFor: 'private'!

changeOrder: n bounds: range
	"Produce the next change in the peal by moving the element n within the range.  If the element wants to move outside the range, alter it's direction and move the next element (recursively call this)."

	| neighbor location |
	location _ self indexOfElement: n.
	neighbor _ location + (directions at: n).

	(n = self size) ifTrue:
		[(n = 1) ifFalse:
			[self swap: location with: neighbor.
			directions at: n-1 put: (directions at: n-1) negated].
		finished _ true.
		^ self].

	(range includes: neighbor) ifTrue:
		[self swap: location with: neighbor]
	ifFalse:
		[directions at: n put: (directions at: n ) negated.
		self changeOrder: n+1 bounds: (range copyWithout: location)]!

indexOfElement: n
	"Answer the index the anElement item  (i.e. the n-th item, not the item = to n) within the receiver."

	1 to: self size do:
		[:i | (self basicAt: i) = n ifTrue: [^ i]].
	^ self error: 'An element does not exist in this peal'!

setBase: baseSet.
	"Initialize the base set and everything else."

	base _ baseSet.
	directions _ Array new: self size withAll: 1.
	position _ 1.
	1 to: self size do: [:i | self basicAt: i put: i].
	finished _ false.!

swap: index1 with: index2
	"We need to redefine this, since we redefined at: & at:put: to map trhrough the base collection"

	| temp |
	temp _ self basicAt: index1.
	self basicAt: index1 put: (self basicAt: index2).
	self basicAt: index2 put: temp! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Peal class
	instanceVariableNames: ''!


!Peal class methodsFor: 'instance creation'!

new: size
	"Create a new peal on the given size with 1..size as the base set."

	| newPeal |
	newPeal _ self basicNew: size.
	newPeal setBase: (1 to: size).
	^ newPeal!

upon: baseSet
	"Create a new peal on the base set given"

	| newPeal |
	newPeal _ self basicNew: baseSet size.
	newPeal setBase: baseSet.
	^ newPeal! !

!Peal class methodsFor: 'examples'!

exampleRand
	"Play a two-tier peal with different tempi"
	"Peal exampleRand"

	| rand pBase pArray1 pArray2 tempo1 tempo2 score |
	rand _ Random new.
	pBase _ 24 + (rand next * 48) rounded.
	pArray1 _ Array new: 2 + ((rand next * 2) rounded).
	1 to: pArray1 size do: [:ind | pArray1 at: ind put: (pBase + (rand  next * 8) rounded)].
	pBase _ 48 + (rand next * 48) rounded.
	pArray2 _ Array new: 2 + ((rand next * 3) rounded).
	1 to: pArray2 size do: [:ind | pArray2 at: ind put: (pBase + (rand  next * 6) rounded)].
	tempo1 _ 120 + (rand next * 60) rounded. "slow"
	tempo2 _ 20 + (rand next * 20) rounded. "fast"
	Transcript show: 'Peal: ', pArray1 printString; tab; show: tempo1 printString; cr; 
			show: pArray2 printString; tab; show: tempo2 printString; cr; cr.
	StructureDescription peal: pArray1 tempo: tempo1
						and: pArray2 tempo: tempo2.
	(Delay forMilliseconds: ((tempo1 max: tempo2) * (pArray1 size max: pArray2 size) * (pArray1 size max: pArray2 size) * (pArray1 size max: pArray2 size))) wait!

pealExample1
	"Play a two-tier peal with different tempi"
	"Peal pealExample1"

	StructureDescription peal: #(82 79 77 82) tempo: 500
		and: #( 47 45 43) tempo: 90! !

Cloud variableSubclass: #DynamicCloud
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
DynamicCloud comment:
'Dynamic clouds are clouds that have starting and ending conditions (ranges or selection sets) and interpolate between them over their duration.'!


!DynamicCloud methodsFor: 'playing'!

eventList
	"make the cloud's events--assume that the receiver's pitch, loudness and durations are actually ARRAYS of two intervals (for the start and end ranges)"

	| pattern rand number length start percent pitchRange p1 p2 p3 p4 amplRange a1 a2 a3 a4 voiceRange v1 v2 v3 v4 |
	pattern _ EventList new.
	rand _ Random new.
	number _ duration asFloat * density.
	length _ (1000 / density) rounded.
	start _ 0.
	(pitch isMemberOf: Array)
		ifTrue: [pitchRange _ pitch first.
			p1 _ pitch first first.
			p2 _ pitch first last.
			p3 _ pitch last first - p1.
			p4 _ pitch last last - p2]
		ifFalse: [pitchRange _ pitch].
	(loudness isMemberOf: Array)
		ifTrue: [amplRange _ loudness first.
			a1 _ loudness first first.
			a2 _ loudness first last.
			a3 _ loudness last first - a1.
			a4 _ loudness last last - a2]
		ifFalse: [amplRange _ loudness].
	(voice isMemberOf: Array)
		ifTrue: [voiceRange _ voice first.
			v1 _ voice first first.
			v2 _ voice first last.
			v3 _ voice last first - v3.
			v4 _ voice last last - v4]
		ifFalse: [voiceRange _ voice].
	1 to: number truncated do:
		[ :count |
		pattern add: (NoteEvent 
			dur: (length * 2 + (length * rand next) rounded)
			pitch: ((pitchRange first rounded) + 
					((rand next * (pitchRange last - pitchRange first)) rounded))
			ampl: ((amplRange first rounded) + 
					((rand next * (amplRange last - amplRange first)) rounded))
			voice: (voice == nil 
						ifTrue: [nil] 
						ifFalse: [(voiceRange first rounded) + 
					((rand next * (voiceRange last - voiceRange first)) rounded)]))
			at: start.
		start _ start + length.
	percent _ count / number.
	(pitch isMemberOf: Array)
		ifTrue: [pitchRange _ p1 + (p3 * percent) to: p2 + (p4 * percent)].
	(loudness isMemberOf: Array)
		ifTrue: [amplRange _ a1 + (a3 * percent) to: a2 + (a4 * percent)].
	(voice isMemberOf: Array)
		ifTrue: [voiceRange _ v1 + (v3 * percent) to: v2 + (v4 * percent)]].
	^pattern! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DynamicCloud class
	instanceVariableNames: ''!


!DynamicCloud class methodsFor: 'examples'!

example1
	"DynamicCloud example1"
	"play a 2 second cloud that goes from low to high and soft to loud"

	(DynamicCloud  dur: 6  
		pitch: #((30 to: 44) (50 to: 50))
		ampl: #((20 to: 40) (90 to: 120))
		voice: (1 to: 8)
		density: 20) eventList "edit" playPrim!

example2
	"DynamicCloud example2"
	"play a 6 second cloud that goes from soft to loud on the same pitch"

	(DynamicCloud  dur: 8  
		pitch: (20 to: 20)
		ampl: #((20 to: 30) (80 to: 120))
		voice: (1 to: 1)
		density: 20) eventList "edit" playPrim!

example3
	"DynamicCloud example3"
	"play a 2 second cloud that focuses on its center"

	(DynamicCloud  dur: 6  
		pitch: #((30 to: 54) (40 to: 40))
		ampl: #((20 to: 40) (90 to: 120))
		voice: (1 to: 8)
		density: 10) eventList "edit" playPrim!

exampleRand
	"DynamicCloud exampleRand"
	"play a 2 second cloud that focuses on its center"

	(DynamicCloud  dur: 6  
		pitch: #((30 to: 54) (40 to: 40))
		ampl: #((20 to: 40) (90 to: 120))
		voice: (1 to: 8)
		density: 10) eventList "edit" playPrim.
	(Delay forMilliseconds: 5000) wait! !

Peal variableSubclass: #PlainPeal
	instanceVariableNames: 'reason parent start end baseDuration durationStack output noteIndex secondMeasure pieceStart '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
PlainPeal comment:
'PlainPeals implement the standard medieval bell-pealing theory as an event list generator.  This implementation was written by Mark Lentczner (mark@Apple.com) in 1986.

Instance variables:
	reason <>
	parent <>
	start <>
	end <>
	baseDuration <>
	durationStack <>
	output <>
	noteIndex <>
	secondMeasure <>
	pieceStart <>
'!


!PlainPeal methodsFor: 'access'!

baseDuration
	"Answer the receiver's base duration for a note."

	^ baseDuration!

baseDuration: baseTime

	baseDuration _ baseTime!

complete
	"Fill in any blanks about playing the peal."

	start isNil & end isNil
		ifTrue: [ ^ self error: 'Neither start nor end time given' ].
	parent isNil & baseDuration isNil
		ifTrue: [ ^ self error: 'Not enough information for a non-derived peal' ].
	baseDuration isNil ifTrue: [ baseDuration _ parent baseDuration ].
	end isNil ifTrue: [ end _ start + self totalDuration ].
	start isNil ifTrue: [ start _ end - self totalDuration ].!

depth

	parent isNil
		ifTrue: [ ^ 0 ]
		ifFalse: [ ^ parent depth + 1 ]!

endTime

	^ end!

endTime: time

	end _ time!

parent: p reason: r

	(p isNil) & (r ~= #firstPeal)
		ifTrue: [ ^ self error: 'A peal without a parent must be the first peal' ].
	parent _ p.
	reason _ r.!

reason

 	^ reason!

startTime

	^ start!

startTime: time

	start _ time!

totalDuration
	"Answer the receiver's total time duration given no tempo changes."

	| order |
	order _ self size.
	^ baseDuration *
		((order + (1/2)) * order factorial + order) ceiling! !

!PlainPeal methodsFor: 'playing'!

acceptDuration: dur for: approxTime
	"A contained child has started up, so used the new duration for a while."

	| currDuration |
	currDuration _ dur.
	[approxTime * (currDuration - baseDuration) > baseDuration]
		whileTrue: [ currDuration _ currDuration // 2 ].
	[approxTime * (currDuration - baseDuration) < baseDuration]
		whileTrue: [ currDuration _ currDuration * 2 ].

	durationStack addFirst: currDuration.
	(reason = #contains)
		ifTrue: [ parent acceptDuration: dur
						for: (pieceStart + end - Time millisecondClockValue) ]!

firstSchedule

	^ pieceStart + start!

playDoubleNote0: n at: time
	"Play a double note of the receiver and answer the next time a note should play"

	| dur |
	dur _ durationStack first * 2.
	output play: n at: time for: dur.
	^ time + dur!

playDoubleNote: n at: time
	"Play a double note of the receiver and answer the next time a note should play"

	| dur |
	dur _ durationStack first * 2.
	output play: n at: time dur: dur amp: 100.
	^ time + dur!

playNote1: n at: time
	"Play a note of the receiver and answer the next time a note should play"

	| dur |
	dur _ durationStack first.
	output play: n at: time for: dur.
	^ time + dur!

playNote: n at: time
	"Play a note of the receiver and answer the next time a note should play"

	| dur |
	dur _ durationStack first.
	output play: n at: time dur: dur amp: 100.
	^ time + dur!

playOn: channel at: begin

	durationStack _ OrderedCollection with: baseDuration.
	output _ channel.
	pieceStart _ begin.
	noteIndex _ 0.
	secondMeasure _ false.!

retractDuration
	"A contained child has finished up, so backUp a duration"

	durationStack removeFirst.
	(reason = #contains) ifTrue: [ parent retractDuration ]!

scheduleAt: time
	"Play the next note of the receiver"

	| order nextTime |
	order _ self size.
	(noteIndex = 0) ifTrue:
		[reason = #contains ifTrue:
			[ parent acceptDuration: baseDuration
					for: self totalDuration ].
		noteIndex _ 1].
	finished & secondMeasure ifTrue:
		[reason = #contains ifTrue: [ parent retractDuration ].
		^ nil ].
	(noteIndex = order) & secondMeasure
		ifFalse: [nextTime _ self playNote: (self at: noteIndex) at: time]
		ifTrue: [nextTime _ self playDoubleNote: (self at: noteIndex) at: time].

	noteIndex _ noteIndex + 1.
	noteIndex > order ifTrue:
		[self change.
		noteIndex _ 1.
		secondMeasure _ secondMeasure not].
	 ^ nextTime! !

!PlainPeal methodsFor: 'private'!

display 
	"Hack Hack Hack"

	| bic pt c |
	pt _ (start // 180 + 30) @ (self depth * 32 + 30).
	c _ '?csezf' at: 1 + (#(contains starts ends staggers firstPeal) indexOf: reason).
	(String with: c) , self size printString displayAt: (pt + (2@-3)).
	bic _ "QD"Pen new.
	bic north.
	bic place: pt.
	bic turn: 180; go: 20; turn: 180; go: 10; turn: 90.
	bic go: (end - start // 180).
	bic turn: 90; go: 10; turn: 180; go: 20.! !

Cloud variableSubclass: #SelectionCloud
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
SelectionCloud comment:
'Selection clouds are created with a set of pitch and amplitude values and return eventLists selected from these values.'!


!SelectionCloud methodsFor: 'playing'!

eventList
	"make the cloud's events"
	"assume that the receiver's pitch, loudness and durations are actually intervals"

	| pattern rand number length start |
	pattern _ EventList new.
	rand _ Random new.
	number _ duration asFloat * density.
	length _ (1000 / density) rounded.
	start _ 0.
	number truncated timesRepeat:
		[ pattern add: (NoteEvent 
			dur: (length * 2 "+ (2000 * rand next) rounded")
			pitch: (pitch atRandom: rand)
			ampl: (loudness atRandom: rand)
			voice: (voice atRandom: rand))
			at: start.
		start _ start + length].
	^pattern! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SelectionCloud class
	instanceVariableNames: ''!


!SelectionCloud class methodsFor: 'examples'!

example1
	"SelectionCloud example1"
	"play a low 2 second selection cloud"

	(SelectionCloud  dur: 2  
		pitch: #(36 38 40 43 45)
		ampl: #(80 80 120)
		voice: #(1 3 5 7)
		density: 16) eventList "edit" "asScore" playPrim!

exampleRand
	"SelectionCloud exampleRand"
	"play a selection cloud"

	| rand pBase pArray aArray score |
	rand _ Random new.
	pBase _ 20 + (rand next * 40) rounded.
	pArray _ Array new: 2 + ((rand next * 4) rounded).
	1 to: pArray size do: [:ind | pArray at: ind put: (pBase+(rand  next * 12) rounded)].
	pBase _ 20 + (rand next * 60) rounded.
	aArray _ Array new: 2 + ((rand next * 6) rounded).
	1 to: aArray size do: [:ind | aArray at: ind put: (pBase+(rand  next * (120 - pBase)) rounded)].
	score _ (SelectionCloud  dur: (3 + (rand next * 4) rounded)
		pitch: pArray
		ampl: aArray
		voice: (1 to: 8)
		density: (3 + (rand next * 15) rounded)) eventList.
"	score edit.
"	score playPrim.
	(Delay forMilliseconds: (score duration + 1000)) wait! !

Roll variableSubclass: #Trill
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
Trill comment:
'A trill is like a roll, except that it can be given an eventList for repetition.'!


!Trill methodsFor: 'playing'!

eventList
	"return an eventList for me"

	| newList |
	newList _ EventList newAnonymous.
	0 to: number do:
		[ :count |
			newList add: (self defaultEventClass 
					dur: noteDuration 
					pitch: (pitch value at: ((count \\ pitch value size) + 1)) 
					ampl: loudness) 
				at: (delta * count) ].
	^newList! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Trill class
	instanceVariableNames: ''!


!Trill class methodsFor: 'instance creation'!

length: aNumber rhythm: aDuration note: aNote
	"return a new roll of the specified length..."

	^self new length: aNumber rhythm: aDuration note: aNote!

length: aNumber rhythm: aDuration notes: aNote
	"return a new trill of the specified length..."

	^self new length: aNumber rhythm: aDuration note: aNote! !

SelectionCloud variableSubclass: #DynamicSelectionCloud
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-EventGenerators'!
DynamicSelectionCloud comment:
'Dynamic selection clouds are described by their beginning and ending pitch and amplitude sets and interpolate between them.'!


!DynamicSelectionCloud methodsFor: 'playing'!

eventList
	"make the dynamic selection cloud's events"
	"assume that the receiver's pitch, loudness and durations are actually 2-D arrays"

	| pattern rand number length start |
	pattern _ EventList new.
	rand _ Random new.
	number _ duration asFloat * density.
	length _ (1000 / density) rounded.
	start _ 0.
	1 to: number do:
		[ :counter |
		pattern add: (NoteEvent 
			dur: (length * 2 "+ (1000 * rand next) rounded")
			pitch: ((rand next > (counter / number)) 
					ifTrue: [ (pitch at: 1) atRandom: rand  ]
					ifFalse: [ (pitch at: 2) atRandom: rand ])
			ampl: (loudness atRandom: rand)
			voice: (voice atRandom: rand))
			at: start.
		start _ start + length].
	^pattern!

eventList0
	"make the dynamic selection cloud's events"
	"assume that the receiver's pitch, loudness and durations are actually 2-D arrays"

	| pattern rand number length start |
	pattern _ EventList new.
	rand _ Random new.
	number _ duration asFloat * density.
	length _ (1000 / density) rounded.
	start _ 0.
	1 to: number do:
		[ :counter |
		pattern add: (NoteEvent 
			dur: (length + (1000 * rand next) rounded)
			pitch: ((((counter / number) * (1 + (rand next * (counter / number)))) > 1.0)
					ifFalse: [ (pitch at: 1) atRandom  ]
					ifTrue: [ (pitch at: 2) atRandom ])
			ampl: loudness atRandom
			voice: voice atRandom)
			at: start.
		start _ start + length].
	^pattern! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DynamicSelectionCloud class
	instanceVariableNames: ''!


!DynamicSelectionCloud class methodsFor: 'examples'!

example1
	"DynamicSelectionCloud example1"
	"play a selection cloud that focuses onto a trill"

	(DynamicSelectionCloud  dur: 4
		pitch: #( #(36 38 40 43 45) #(53 54) )
		ampl: #(80 80 120)
		voice: #(1 3 5 7)
		density: 16) eventList "edit" "asScore" playPrim!

example2
	"DynamicSelectionCloud example2"
	"play a selection cloud that makes a transition from one note to another"

	(DynamicSelectionCloud  dur: 6
		pitch: #( #(48 49) #(74 75) )
		ampl: #(80 80 120)
		voice: #(1 3 5 7)
		density: 14) eventList "edit" "asScore" playPrim!

exampleRand
	"DynamicSelectionCloud exampleRand"
	"play a selection cloud that focuses onto a trill"

	| rand pBase pArray1 pArray2 aArray1 aArray2 score |
	rand _ Random new.
	pBase _ 20 + (rand next * 60) rounded.
	pArray1 _ Array new: 1 + ((rand next * 8) rounded).
	1 to: pArray1 size do: [:ind | pArray1 at: ind put: (pBase+(rand  next * 12) rounded)].
	pBase _ 20 + (rand next * 60) rounded.
	pArray2 _ Array new: 2 + ((rand next * 3) rounded).
	1 to: pArray2 size do: [:ind | pArray2 at: ind put: (pBase+(rand  next * 12) rounded)].
	pBase _ 20 + (rand next * 60) rounded.
	aArray1 _ Array new: 2 + ((rand next * 6) rounded).
	1 to: aArray1 size do: [:ind | aArray1 at: ind put: (pBase+(rand  next * (120 - pBase)) rounded)].
	Transcript show: 'DynSel Cloud: ', pArray1 printString; cr; show: pArray2 printString; cr; cr.
	score _ (DynamicSelectionCloud  dur: (4 + (rand next * 4) rounded)
		pitch: (Array with: pArray1 with: pArray2)
		ampl: aArray1
		voice: (1 to: 8)
		density: ( 4 + (rand next * 15) rounded)) eventList.
"	score edit.
"	score playPrim.
	(Delay forMilliseconds: (score duration + 1000)) wait! !