/**************************************************************************************

Intended to be used with Lee Boynton's midi drivers on the Macintosh II.
 see Inside Macintosh, II-181 et seq for this header and these device driver routines 
 
 7-11-88, g.e. garnett
 
**************************************************************************************/

#include "userprim.h"
/* Devices.h includes OSUtils.h which includes Types.h which defines OSErr */
#include <Devices.h>
/* Files.h defines IOParam, a macintosh parameter block */
#include <Files.h>
#include "midi.h"
/* this is the structure needed to access driver. See Inside Macintosh II-181 et seq */
static IOParam parameterBlock;
/* this is the basic MIDI data structure required by the driver */
static MIDI_Event aMidiEvent;
/* this is the driver status parameters clockTime and 8 short extras*/
static MIDI_Parameters parms; 

/*	the driver rotuines can be invoked from Smalltalk-80 via calls to the primitive
defined by the following numbers (user primitives are always negative)	*/

#define OPENnumber -101
#define CLOSEnumber -102
#define NEXTnumber -103
#define PUTnumber -104
#define STATUSnumber -105
#define SETnumber -106
#define PLAYnumber -107
#define NEXTEVENTnumber -108
#define SETCLOCKnumber -109

/**************************************************************************************

	 Simply installs the primitives in the VM 
	 
**************************************************************************************/
char*
UPinstall()
	{
		upHandle openMidi();	/* defines the function name */
		upHandle closeMidi();
		upHandle nextMidi();
		upHandle putMidi();
		upHandle statusMidi();
		upHandle setMidi();
		upHandle playEvent();
		upHandle nextEvent();
		upHandle setClock();
		
		
		UPaddPrimitive( OPENnumber, openMidi, 0);
		UPaddPrimitive( CLOSEnumber, closeMidi, 0);
		UPaddPrimitive( NEXTnumber, nextMidi, 1);
		/* writeMidi sends a byteArray  */
		UPaddPrimitive( PUTnumber, putMidi, 2);
		UPaddPrimitive( STATUSnumber, statusMidi, 1);
		UPaddPrimitive( SETnumber, setMidi, 2);
		UPaddPrimitive( PLAYnumber, playEvent, 2);
		UPaddPrimitive( NEXTEVENTnumber, nextEvent, 1);
		UPaddPrimitive( SETCLOCKnumber, setClock, 2);
		return(" MIDI Driver Installed ");
	}
	

/*	 then comes the actual code that is executed by the primitive	*/

/**************************************************************************************

					openMidi(aSmalltalkObject)
					
	open the device driver and initialize everything, start clock
	ignores the smalltalk instance variable, name
	
	 Return: True, if successful, otherwise, errorMessage
	 
**************************************************************************************/

upHandle openMidi( receiver)
	upHandle receiver;
{
	
	short error;
	char *name = ".MIDI"; /* the name of the DRVR resource. Presumably, the VM 
		"application" has this resource defined, via ResEDit? */
	
	UPbegin(receiver,0);	/* first executable line of every user primitive */

	error = DriverOpen(&parameterBlock, name);
	if (error)
		UPreturnHandle(UPCtoSTint(error));
	UPreturnTrue();	/* successfully opened driver */
}
/**************************************************************************************

					closeMidi(aSmalltalkObject)
					
	close the device driver and deallocates memory.
	
	Return: True, if successful, otherwise, errorMessage
	 
**************************************************************************************/

upHandle closeMidi( receiver )
	upHandle receiver;
{
	
	short error;
	int i;
	char *clearPB;

	UPbegin(receiver,0);	/* first executable line of every user primitive */
	error = DriverClose(&parameterBlock);
	if (error)
		UPreturnHandle(UPCtoSTint(error));
	clearPB = &parameterBlock;
	for (i=0; i<sizeof(parameterBlock);i++)
		*(clearPB+i) = 0;
	UPreturnTrue();	/* successfully closed driver */
}
/**************************************************************************************

					nextMidi(aSmalltalkObject, aSTByteArray)
					
	Get the next midi event and store it in aSTByteArray.
	Bytes 1 through 4 -> time,
		  5 modeByte (raw or otherwise)
		  6 status (channel) byte
		  7 data1
		  8 data2
	Return: True, if successful, otherwise, errorMessage
	
**************************************************************************************/

upHandle nextMidi( receiver, byteArray)
	upHandle receiver, byteArray;
{
	/* byteArray, will be a 4-byte timeCode, followed by a type byte (0 means a 
	single byte of raw data will be returned, a 1 means a status byte will be
	returned (MIDI statusbyte) followed by up to two midi data 
	nextMIDI returns true if operation was successful */ 

	short error;
	char *aPointer;
	int i;
	
	UPbegin(receiver,1);	/* first executable line of every user primitive */
	UPmustBeByteArray( byteArray);
	error = DriverRead(&parameterBlock,&aMidiEvent, sizeof(MIDI_Event));

	if (error)
		UPfail(UPallocateString("Problem trying to read driver"));
	aPointer = &(aMidiEvent.time);
	/* remember, Smalltalk is only handling 32 bit integers at the moment and
	a long int in C is just 32 bits! */
	
	for (i=1; i<5; i++)
	{
		UPbyteAtPut(byteArray, i, *aPointer++);
	}
	UPbyteAtPut(byteArray, 5, aMidiEvent.type);
	UPbyteAtPut(byteArray, 6, aMidiEvent.status);
	UPbyteAtPut(byteArray, 7, aMidiEvent.data1);
	UPbyteAtPut(byteArray, 8, aMidiEvent.data2);

	UPreturnTrue();	/* successfully received midiData */

}
/**************************************************************************************

					nextEvent(aSmalltalkObject, aSTByteArray)
					
	Get the next midi event and store it in aSTByteArray.
	Bytes 1 modeByte, raw or otherwise,
		  2 status (channel) byte
		  3 data1
		  4 data2
	Return: milliSecondTime, if successful, otherwise, errorMessage
	
	 
**************************************************************************************/

upHandle nextEvent( receiver, byteArray)
	upHandle receiver, byteArray;
{
	/* byteArray, will be a mode byte (0 means a single byte of raw data will be 
	returned, a 1 means a status byte will be returned (MIDI statusbyte) followed 
	by up to two midi data. nextEvent returns the time the event was received
	if operation was successful, else an error msg */ 

	short error;
	
	UPbegin(receiver,1);	/* first executable line of every user primitive */
	UPmustBeByteArray( byteArray);
	error = DriverRead(&parameterBlock,&aMidiEvent, sizeof(MIDI_Event));

	if (error)
		UPfail(UPallocateString("Problem trying to read driver"));

	/* remember, Smalltalk is only handling 32 bit integers at the moment and
	a long int in C is just 32 bits! */
	UPbyteAtPut(byteArray, 1, aMidiEvent.type);
	UPbyteAtPut(byteArray, 2, aMidiEvent.status);
	UPbyteAtPut(byteArray, 3, aMidiEvent.data1);
	UPbyteAtPut(byteArray, 4, aMidiEvent.data2);
	UPreturnHandle(UPallocateInteger(aMidiEvent.time));	/* successfully received midiData */

}
/**************************************************************************************

					putMidi(aSmalltalkObject, aSTByteArray, aSTInteger)
					
	Put the next aSTInteger elements of a midi event, aSTByteArray, out the midiPort
	Bytes 1 through 4 -> time,
		  6 status (channel) byte
		  7 data1
		  8 data2
	Return: True, if successful, otherwise, errorMessage
	 
**************************************************************************************/

upHandle putMidi( receiver, byteArray, aCount)
	upHandle receiver, aCount, byteArray;
{
	
	short error;
	int i;
	upInt count;
	upChar *aByte;

	UPbegin(receiver,2);	/* first executable line of every user primitive */
	UPmustBeByteArray( byteArray);
	count = UPSTtoCint( aCount);

	aByte = &aMidiEvent;
	for (i=1;i<=count;i++)
	{
		*aByte++ = UPbyteAt(byteArray,i);
	}
	error = DriverWrite(&parameterBlock,&aMidiEvent, count);
	if (error)
		UPfail(UPallocateString("Problem trying to write to driver"));
	UPreturnTrue();
}
/**************************************************************************************

					playMidi(aSmalltalkObject, aSTByteArray, aSTMSClockTime)
					
	Put the next 4 elements of a midi event, aSTByteArray, out the midiPort.
	aMidiEvent must be packed as follows:
	Bytes 1 through 4 -> time,
		  6 status (channel) byte
		  7 data1
		  8 data2
	Return: True, if successful, otherwise, errorMessage
	 
**************************************************************************************/

upHandle playEvent( receiver, byteArray, aTime)
	upHandle receiver, byteArray, aTime;
{
	
	short error;
	upInt time;

	UPbegin(receiver,2);	/* first executable line of every user primitive */
	UPmustBeByteArray( byteArray);
	time = UPSTtoCint(aTime);
	
	aMidiEvent.time = (long)time;
	aMidiEvent.type = UPbyteAt(byteArray,1);
	aMidiEvent.status = UPbyteAt(byteArray,2);
	aMidiEvent.data1 = UPbyteAt(byteArray,3);
	aMidiEvent.data2 = UPbyteAt(byteArray,4);
	error = DriverWrite(&parameterBlock,&aMidiEvent, sizeof(MIDI_Event));
	if (error)
		UPfail(UPallocateString("Problem trying to play event"));
	UPreturnTrue();
}
/**************************************************************************************

					statusMidi(aSmalltalkObject, aSTInteger)
					
	Get the status of the midiDriver based on the control code in aSTInteger.

	aSTInteger = 0,	get clock time
		returns (4-bytes) clock time
	aSTInteger = 4,	readMode
		returns 0 (in first two bytes) for raw mode, otherwise normal midi parsing
	aSTInteger = 5,	receiveCount
		returns two byte integer which is number of raw bytes received
		followed by an input overflow flag (non-zero means overflow occured)
	aSTInteger = 6,	transmitCount
		returns two byte integer which is number of raw bytes transmitted
		followed by an output overflow flag (non-zero means overflow occured)
	aSTInteger = 7,	tickSize
			returns (4-bytes) tick Size in milliseconds

	
	Return: aSTInteger, if successful, otherwise, errorMessage
	 
**************************************************************************************/

upHandle statusMidi( receiver, aControlCode)
	upHandle receiver, aControlCode;
{
	/* params is an array of 11 (2-byte) integers which is written into by the
	DriverStatus routine. This could be optimized further by having the DriverStatus
	routine only access the four bytes it really needs; it seems the midi
	driver only uses at most the first two (small) ints so I will have statusMidi 
	only return those two, as a ST integer*/
	
	short error;
	short csCode;
	int parms[11];
	upInt *returnParm;

	UPbegin(receiver,1);	/* first executable line of every user primitive */
	UPmustBeInteger( aControlCode);
	csCode = (short)UPSTtoCint( aControlCode);

	error = DriverStatus(&parameterBlock,csCode, parms);
	if (error)
		UPreturnHandle(UPCtoSTint(error));
	returnParm = parms;
	UPreturnHandle(UPallocateInteger(*returnParm));
}
/**************************************************************************************

					setMidi(aSmalltalkObject, aSTInteger, anotherSTInteger)
					
	Set the status of the midiDriver based on the control code in aSTInteger.

	aSTInteger = 0,	set clock time
		set midiClock to (4-bytes) anotherSTInteger in milliseconds
	aSTInteger = 1,	kill IO
		ignores anotherSTInteger (but still gets copied)
	aSTInteger = 2,	receive On
		set receive On
	aSTInteger = 3,	receive off
		set receive Off
	aSTInteger = 4,	readMode
		anotherSTInteger is 0 (in first two bytes) set to raw mode,
		otherwise normal midi parsing
	aSTInteger = 7,	tickSize
			sets (4-byte) tick Size to anotherSTInteger in milliseconds

	
	Return: true, if successful, otherwise, errorMessage
	 
**************************************************************************************/

upHandle setMidi( receiver, aControlCode, data)
	upHandle receiver, aControlCode, data;
{
	/* data is a 2-byte integer which is read by the
	DriverControl routine.  Do not use this for  settting the midi Clock,
	use setClock() instead. */
	
	short error;
	short csCode;
	int parms[11];

	UPbegin(receiver,2);	/* first executable line of every user primitive */
	UPmustBeInteger( aControlCode);
	UPmustBeInteger( data);
	csCode = (short)UPSTtoCint( aControlCode);

	*(long *)parms = UPSTtoCint( data);
	
	error = DriverControl(&parameterBlock,csCode, parms);
	if (error)
		UPfail(UPallocateString("Problem setting driver"));
	UPreturnTrue();
}
/**************************************************************************************

					setClock(aSmalltalkObject, aSTInteger, anotherSTInteger)
					
	Set the status of the midiDriver based on the control code in aSTInteger.

	aSTInteger = 0,	set clock time
		set midiClock to (4-bytes) anotherSTInteger in milliseconds
	aSTInteger = 7,	tickSize
			sets (4-byte) tick Size to anotherSTInteger in milliseconds

	
	Return: true, if successful, otherwise, errorMessage
	 
**************************************************************************************/

/* use this instead of setMidi() to handle setting the midi Clock */
upHandle setClock( receiver, aControlCode, aMSClockTime)
	upHandle receiver, aControlCode, aMSClockTime;
{
	/* aMSClockTime is a 4-byte integer which is read by the
	DriverControl routine as two 2-byte ints. */
	
	short error;
	short csCode;
	int parms[11];

	UPbegin(receiver,2);	/* first executable line of every user primitive */
	UPmustBeInteger( aControlCode);
	UPmustBeInteger( aMSClockTime);
	csCode = (short)UPSTtoCint( aControlCode);
	*(long *)parms = UPSTtoCint( aMSClockTime);

	/* do some error checking, if not clock-Time error */
	
	error = DriverControl(&parameterBlock,csCode, parms);
	if (error)
		UPfail(UPallocateString("Problem setting  clock of midi driver"));
	UPreturnTrue();
}
