/*
MidiPrim.c
Last change: January 14, 1988.

Copyright 1989 John H. Maloney. All rights reserved.

This module implements MIDI primitives for ParcPlace Systems
Smalltalk, Version 2.3 for the Apple Macintosh. The primitives are:

<-1> OpenPrim	- initializes a serial port for MIDI including
				  configuring the SCC chip, allocating input and output
				  buffers, and initializing the interrupt vectors
<-2> ClosePrim	- disables interrupts and frees buffers
<-3> PutPrim	- queues data for transmission
<-4> GetPrim	- reads data from the input queue
<-5> NoOp		- does nothing; used to measure primitive overhead

The modem port is port 0 and the printer port is port 1.

Compile with the "-dMPW" switch to compile using MPW C.
You currently can't build Smalltalk user primitives using
Lightspeed C, however you should be able to compile this file
if you turn of the "require prototypes" preference.

NOTE: In order to allow the linker to correctly install this primitive
in the Smalltalk interpreter, you must have a copy of the interpreter
in the same directory. The linker will install the user primitive
code segment directly into the interpreter.

******************** MPW 2.0 Build Script *******************
Asm SetUpA5.a
C -dMPW MidiPrimDriver.c
C -dMPW -dMAC MidiPrim.c
Link -w -c PSS3 
	SetUpA5.a.o 
	MidiPrimDriver.c.o 
	MidiPrim.c.o 
	Smalltalk.lib 
	"{CLibraries}"CRuntime.o 
	"{Libraries}"Interface.o 
	"{CLibraries}"CInterface.o 
	"{CLibraries}"StdCLib.o 
		-o Smalltalk-80
************************************************************/

/* define "MPW" to compile with MPW C */
#ifdef MPW
#include <Memory.h>
#include <OSUtils.h>
#include <Types.h>
#else
#include <MemoryMgr.h>
#include <OSUtil.h>
#include <Proto.h>
#endif

#include "cext.h"
#include "MidiPrim.h"
#include "userprim.h"

upInt OpenPrim(upHandle, upHandle);
private upInt OpenPrim(RR, stPort)
	upHandle RR, stPort;
{
	UPbegin(RR, 1);
	MidiOpen(UPSTtoCint(stPort));
	UPreturnHandle(RR);
}

upInt ClosePrim(upHandle, upHandle);
private upInt ClosePrim(RR, stPort)
	upHandle RR, stPort;
{
	UPbegin(RR, 1);
	MidiClose(UPSTtoCint(stPort));
	UPreturnHandle(RR);
}

upInt GetPrim(upHandle, upHandle, upHandle, upHandle);
private upInt GetPrim(RR, stPort, stBytes, stCount)
	upHandle RR, stPort, stBytes,stCount;
{
	byte buffer[1000];
	upInt requestSize, received;

	UPbegin(RR, 3);

	requestSize = min(UPSTtoCint(stCount), 1000);
	received = MidiGet(UPSTtoCint(stPort), &buffer[0], requestSize);
	received = UPcopyCtoSTbytes(stBytes, &buffer[0], received, 1);

	UPreturnHandle(UPCtoSTint(received));
}

upInt PutPrim(upHandle, upHandle, upHandle, upHandle);
private upInt PutPrim(RR, stPort, stBytes, stCount)
	upHandle RR, stPort, stBytes,stCount;
{
	byte buffer[1000];
	int sendSize;

	UPbegin(RR, 3);

	sendSize = min(UPSTtoCint(stCount), 1000);
	sendSize = UPcopySTtoCbytes(stBytes, &buffer[0], sendSize, 1);
	MidiPut(UPSTtoCint(stPort), &buffer[0], sendSize);

	UPreturnHandle(RR);
}

upInt NoOp(upHandle);
upInt NoOp(RR)
	upHandle RR;
{
	UPbegin(RR, 0);
	UPreturnHandle(RR);
}

char * UPinstall(void);
char * UPinstall()
{
	UPaddPrimitive(-1, OpenPrim, 1);
	UPaddPrimitive(-2, ClosePrim, 1);
	UPaddPrimitive(-3, PutPrim, 3);
	UPaddPrimitive(-4, GetPrim, 3);
	UPaddPrimitive(-5, NoOp, 0);
	return "Maloney's MIDI Primitives Installed";
}

int num2dec(void);
num2dec()
{
	/* routine to stub out the SANE library */
}
