/* MidiPrim.h -- Macintosh MIDI primitives for ParcPlace Smalltalk */

/* Midi driver operations */
void	MidiOpen(int port);
void	MidiClose(int port);
void	MidiPutByte(int port, byte dataByte);
void	MidiPut(int port, byte *buffer, int count);
int		MidiGet(int port, byte *buffer, int count);

/* The sizes of the input and output buffers and masks for computing
 * the wrap arounds. NOTE: So that we can use a mask to compute the
 * modulus operation, buffer sizes must be even powers of two. */
#define IN_BUF_SIZE 8192
#define OUT_BUF_SIZE 8192

typedef struct {
	ushort mask;	/* (= size - 1); used to do modulo size arithmetic */
	ushort head;
	ushort tail;
	byte data[];
} ByteQueue;

typedef struct {
	ushort errors;
	ushort inputOverruns;
	boolean sending;
	ByteQueue *in;
	ByteQueue *out;
} MIDIPort;

/* ByteQueue operations */
ByteQueue *	MakeQueue(int);
void		Insert(ByteQueue *, byte);
byte		Remove(ByteQueue *);

/* More ByteQueue operations, implemented as macros */
#define QFull(QPtr) (((QPtr->tail + 1) & QPtr->mask) == QPtr->head)
#define QEmpty(QPtr) (QPtr->head == QPtr->tail)
#define QCount(QPtr) ((QPtr->tail - QPtr->head) & QPtr->mask)
#define QInsert(queue, dataByte)				\
	{	queue->data[queue->tail++] = dataByte;	\
		queue->tail &= queue->mask;				\
	}
#define QRemove(queue, dataByte)				\
	{	dataByte = queue->data[queue->head++];	\
		queue->head &= queue->mask;				\
	}
