'From Smalltalk-80, Version 2.3 of 13 June 1988 on 14 January 1989 at 2:08:17 pm'!

Object subclass: #MidiPrim
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Midi'!
MidiPrim comment:
'Smalltalk Midi Primitive
Copyright 1989, John Maloney. All rights reserved.
Permission to use this code for research and experimental purposes
is hereby granted provided this copyright notice is retained intact.

1. Be sure to have a Midi interface attached the port you are using. The interface should have a clock speed of 1 Mhz (most do, including the one from Apple). It is possible to hang the system if you open a port that does not have a Midi interface attached (and turned on) because the port depends on a clock signal supplied by the Midi interface -- no clock, no action!!

2. The Midi driver is interrupt driven and the interrupt code is part of the Smalltalk interpreter. Thus, if you quit from Smalltalk with one of the Midi ports open and an interrupt occurs (due to a character arriving, say) then your Mac will crash because the drivers aren''t there anymore. Included in this filein is a modification of Smalltalk''s quit method that closes all Midi ports. (Closing a non-open port is harmless.) It is recommended that you include a similar mechanism in any applications you develop yourself. You will have to re-open the Midi port(s) you were using after restarting the image.

3. There is a limit of 1000 bytes per call to the Midi put primitive; you can pass it a bigger buffer but the extra bytes will not be sent.

4. The Midi put primitive puts its bytes in an internal buffer until all previous bytes have been sent. If there is not enough room in the buffer, the put primitive waits until there is room. If there IS room in in the buffer, the bytes of the current call are buffered and the call returns to Smalltalk immediately even though not all the bytes have actually been sent.

5. If the transmit buffer is completely full, it takes nearly three seconds to empty itself. (Midi data is sent at 3125 bytes/second, so you have to send a lot of data fast to actually fill up the 8192 byte transmit buffer.) If you close the Midi port before the buffer is empty, some bytes will never be sent. My practice is to keep the Midi port open continuously during a Smalltalk session but if your application opens and closes the port you may wish to put a three second delay before each close operation to absolutle guarantee that all the bytes have been sent.
'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MidiPrim class
	instanceVariableNames: ''!


!MidiPrim class methodsFor: 'primitive tests'!

noopTest
	"Noop is the null primitive, used to measure the overhead of calling a user primitive. This test returns the time in milliseconds taken by 1000 calls to Noop. On a Mac II this should be about 200 milliseconds; on an SE it should be about 360 milliseconds. Your mileage may vary."
	"MidiPrim noopTest"

	^Time millisecondsToRun:
		[1000 timesRepeat: [self primNoop]]!

openCloseTest
	"This tests opening and closing port 0, the modem port."
	"Midi openCloseTest"

	self primOpen: 0.
	self primClose: 0.!

receiveTest
	"This test opens port 0, the modem port, receives Midi data for 5 seconds, and returns the number of bytes received. Pound on the keys or something to cause Midi data to be sent. Most Midi keyboards send an 'active sensing' byte five or six times a second, so you probably won't receive zero bytes unless the Midi cables are not plugged in or the keyboard is off."
	"Midi receiveTest"

	| bytes count startTime |
	bytes _ ByteArray new: 1000.
	count _ 0.
	startTime _ Time millisecondClockValue.
	self primOpen: 0.
	[(Time millisecondClockValue - startTime) < 5000]
		whileTrue:
			[count _ count +
				(self primGet: 0 bytes: bytes size: 1000)].
	self primClose: 0.
	^count!

sendTest
	"This test sends 10000 bytes to port 0, the modem port, in a sequence of 10 byte buffers, and returns the number of milliseconds required. You might experiment with sending different numbers of bytes per call to primPut:bytes:count:, up to 1000. (There is a limit of 1000 bytes per call to the put primitive; you can give it a bigger buffer but the extra bytes will not be sent.) Note that the put primitive buffers characters bytes until all previous bytes have been transmitted. transmission"
	"Midi sendTest"

	| bytes time |
	bytes _ ByteArray new: 1000 withAll: 248. "Midi clock byte"
	self primOpen: 0.
	time _ Time millisecondsToRun: [
		1000 timesRepeat:
			[self primPut: 0 bytes: bytes count: 10]].
	self primClose: 0.
	^time! !

!MidiPrim class methodsFor: 'primitives'!

primClose: portNumber
	"Closes the given port. portNumber should be 0 for the modem port and 1 for the printer port. Does nothing if the given Midi port is not open."

	<primitive: -2>
	self primitiveFailed.!

primGet: portNumber bytes: aByteArray size: aNumber
	"Copy up to (aNumber min: aByteArray size) bytes into aByteArray and answer the number of bytes actually received. Does nothing if the given Midi port is not open."

	<primitive: -4>
	self primitiveFailed.!

primNoop
	"The null primitive; it does nothing."

	<primitive: -5>
	self primitiveFailed.!

primOpen: portNumber
	"Opens the given port. portNumber should be 0 for the modem port and 1 for the printer port."

	<primitive: -1>
	self primitiveFailed.!

primPut: portNumber bytes: aByteArray count: aNumber
	"Send aNumber bytes from aByteArray to the given Midi port. (Usually aNumber is simply the size of aByteArray.)  Does nothing if the given Midi port is not open."
	"Warning: You can only send 1000 bytes per call. Extra bytes will be ignored."
	"Warning: Bytes are buffered for eventual tranmission; see my class comment."

	<primitive: -3>
	self primitiveFailed.! !

!SystemDictionary methodsFor: 'system backup/out'!

quit
	"Close open files then exit to another operating system on the host machine, 
	if one exists.  All state changes in the object space since the last snapshot are lost."

	SystemCall putScrap.
	self releaseExternalViews.
	self changed: #aboutToQuit.
	"Close Midi ports (does nothing if they are not open)."
	MidiPrim primClose: 0.
	MidiPrim primClose: 1.
	self quitPrimitive! !
