'From Smalltalk-80, Version 2.2 of July 4, 1987 on 18 February 1988 at 3:00:39 pm'!

Cursor addClassVarName: 'CursorConstants'!

!Cursor methodsFor: 'printing'!

printOn: aStream  
	"Append to the argument aStream a sequence of characters that identifies the receiver."

	self storeOn: aStream base: 10! !

!Cursor methodsFor: 'name'!

name
	"Answer the string labelling the receiver."

	^name!

name: aString
	"Make the argument aString, be the label for the receiver."

	name _ aString! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!Cursor class methodsFor: 'class initialization'!

initialize
	"Create all the standard cursors."

	"Cursor initialize."

	CursorConstants _ Dictionary new.
	CursorConstants at: #blank put: (Cursor new name: 'blank').

	self initNormal.
	self initXeq.
	self initRead.
	self initWrite.
	self initUp.
	self initDown.
	self initMarker.
	self initScroll.
	self initOrigin.
	self initCorner.
	self initTopRight.
	self initBottomLeft.
	self initCrossHair.
	self initCaret.
	self initWait.
	self initGarbage.
	self initThumbsUp.
	self initThumbsDown.
	self initTopRight.
	self initBottomLeft.
	self initBull.
	self initHand.!

initializeNew
	"Create all the new wierd cursors."
	"Cursor initializeNew."

	self initCircle.
	self initTriangle.
	self initDiamond.
	self initDot.
	self initBDot.
	self initSpot.
	self initHole.
	self initNote.
	self initBar.
	self initLine.
	self initBox.
	self initScrollButton.
	self initSpec.
	self initZ1.
	self initZ2! !

!Cursor class methodsFor: 'instance creation'!

extent: extentPoint fromArray: anArray offset: offsetPoint 
	"Answer a new instance of the receiver with width and height specified by
	extentPoint, offset by offsetPoint, and bits from anArray.  Provide an error
	notification if the extentPoint is not 16@16."

	extentPoint = (16 @ 16)
		ifTrue: 
			[^super
				extent: extentPoint
				fromArray: anArray
				offset: offsetPoint]
		ifFalse: [self error: 'cursors must be 16@16']!

extent: extentPoint fromArray: anArray offset: offsetPoint name: aString 
	"Answer a new instance of the receiver with width and height 
	specified by extentPoint, offset by offsetPoint, and bits from 
	anArray.   Provide an error notification if the extentPoint is not 16@16."

	extentPoint = (16 @ 16)
		ifTrue: [^(super
				extent: extentPoint
				fromArray: anArray
				offset: offsetPoint)
				name: aString]
		ifFalse: [self error: 'cursors must be 16@16']!

new	
	"Answer a new instance of the receiver that is a blank image."

	"Cursor new bitEdit."

	^self
		extent: 16 @ 16
		fromArray: Array new
		offset: 0 @ 0! !

!Cursor class methodsFor: 'constant access'!

constantNamed: aSymbol put: aCursor
	"Put aCursor under the name aSymbol in my constant dictionary"

	^CursorConstants at: aSymbol put: aCursor!

constantsFromFile: fileName 
	"Load my constant dictionary with the contents of the file named fileName."

	| stream |
	stream _ (FileStream oldFileNamed: fileName) readOnly.
	CursorConstants _ Dictionary readFrom: stream.
	stream close!

constantsToFile: fileName 
	"Store the icons in my constant dictionary to the file named fileName."

	| stream |
	stream _ FileStream newFileNamed: fileName.
	CursorConstants storeOn: stream.
	stream close! !

!Cursor class methodsFor: 'constant initialization'!

initBottomLeft
	Cursor constantNamed: #bottomLeft put: (Cursor
			extent: 16@16
			fromArray: #(
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1111111111111111
		2r1111111111111111)
			offset: -2@-2 name: 'bottomLeft')!

initBull
	Cursor constantNamed: #bull put: (Cursor 
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000011111000000
		2r0001100100110000
		2r0010000100001000
		2r0100000100000100
		2r0100000100000100
		2r1000000100000010
		2r1000000100000010
		2r1111111111111110
		2r1000000100000010
		2r1000000100000010
		2r0100000100000100
		2r0100000100000100
		2r0010000100001000
		2r0001100100110000
		2r0000011111000000)
			offset: -8@-9 name: 'bull').!

initCaret
	Cursor constantNamed: #caret put: (Cursor
	  extent: 16@16
	  fromArray: #(
		2r110000000
		2r110000000
		2r1111000000
		2r11111100000
		2r11001100000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: -8@0 name: 'caret').!

initCorner
	Cursor constantNamed: #corner put: (Cursor 
			extent: 16@16
			fromArray: #( 7 11 7 11 7 11 7 11 7 11 7 11 21847 43691 65535 65535)
			offset: -14@-14
			name: 'corner')!

initCrossHair
	Cursor constantNamed: #crossHair put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r1111111111111110
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0)
			offset: -7@-7 name: 'crossHair')!

initDown
	Cursor constantNamed: #down put: (Cursor 
			extent: 16@16
			fromArray: #(
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r1111110000000000
		2r0111110000000000
		2r0011110000000000
		2r0001110000000000
		2r0000110000000000
		2r0000010000000000)
			offset: -5@-7 name: 'down')!

initGarbage
	Cursor constantNamed: #garbage put: (Cursor
	extent: 16@16
	fromArray: #(
		2r111111100000
		2r1000000011000
		2r1000000000100
		2r111111100100
		2r1111111111000
		2r1011111101000
		2r1000000001000
		2r1001001001000
		2r1001001001000
		2r1001001001000
		2r1001001001000
		2r1001001001000
		2r1001001001000
		2r1001001001000
		2r100000010000
		2r11111100000)
	offset: 0@0 name: 'garbage')!

initHand
	Cursor constantNamed: #hand put: (Cursor
			extent: 16 @ 16
			fromArray: #(
		2r110000000
		2r1101001110000
		2r10011001001000
		2r10011001001010
		2r1001001001101
		2r1001001001001
		2r110100000001001
		2r1001100000000001
		2r1000100000000010
		2r100000000000010
		2r10000000000010
		2r1000000000100
		2r1000000000100
		2r100000001000
		2r10000001000
		2r10000001000)
			offset: 0 @ 0
			name: 'hand').!

initializeNumberCursors
	"set up 10 cursors for the 10 digits"
	"Cursor initializeNumberCursors"

	| form cur i |
	NumberCursors _ Array new: 10.
	i _ 0.
	#('0' '1' '2' '3' '4' '5' '6' '7' '8' '9') do:
		[:str |
		form _ str asText asParagraph asForm.
		cur _ Cursor new.
		form displayOn: cur.
		NumberCursors at: (i _ i+1) put: cur]!

initMarker
	Cursor constantNamed: #marker put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0
		2r0
		2r0
		2r0000001000000000
		2r0000001110000000
		2r0000001111100000
		2r1111111111111000
		2r1111111111111110
		2r1111111111111000
		2r0000001111100000
		2r0000001110000000
		2r0000001000000000
		2r0
		2r0
		2r0
		2r0)
			offset: -7@-7 name: 'mark')!

initNormal
	Cursor constantNamed: #normal put: (Cursor
			extent: 16@16
			fromArray: #(
		2r1000000000000000
		2r1100000000000000
		2r1110000000000000
		2r1111000000000000
		2r1111100000000000
		2r1111110000000000
		2r1111111000000000
		2r1111100000000000
		2r1111100000000000
		2r1001100000000000
		2r0000110000000000
		2r0000110000000000
		2r0000011000000000
		2r0000011000000000
		2r0000001100000000
		2r0000001100000000)
	offset: 0@0 name: 'normal')!

initOrigin
	Cursor constantNamed: #origin put: (Cursor
			extent: 16@16
			fromArray: #(
		2r1111111111111111
		2r1111111111111111
		2r1101010101010101
		2r1110101010101010
		2r1101000000000000
		2r1110000000000000
		2r1101000000000000
		2r1110000000000000
		2r1101000000000000
		2r1110000000000000
		2r1101000000000000
		2r1110000000000000
		2r1101000000000000
		2r1110000000000000
		2r1101000000000000
		2r1110000000000000)
			offset: -2@-2 name: 'origin')!

initRead
	Cursor constantNamed: #read put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000110000000110
		2r0001001000001001
		2r0001001000001001
		2r0010000000010000
		2r0100000000100000
		2r1111101111100000
		2r1000010000100000
		2r1000010000100000
		2r1011010110100000
		2r0111101111000000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: 0@0
	name: 'read')!

initScroll
	Cursor constantNamed: #scroll put: (Cursor 
			extent: 16@16
			fromArray: #(
		2r0000100000000000
		2r0000110000000000
		2r0000111000000000
		2r0000111100000000
		2r0000111110000000
		2r0000111111000000
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r0000110000000000
		2r1111110000000000
		2r0111110000000000
		2r0011110000000000
		2r0001110000000000
		2r0000110000000000
		2r0000010000000000)
			offset: -5@-7 name: 'scroll')!

initScrollButton
	Cursor constantNamed: #scrollButton put: (Cursor
	extent: 16@16
	fromArray: #( 384 960 2016 384 384 8580 24966 65535 65535 24966 8580 384 384 2016 960 384)
	offset: -5@-7 name: 'scrollButton')!

initThumbsDown
	Cursor constantNamed: #thumbsDown put: (Cursor
			extent: 16@16
			fromArray:
				#(
		2r111111110000
		2r100000001000
		2r1111100000111
		2r10000000000000
		2r11111100000000
		2r10000000000000
		2r11111100000000
		2r10000000000000
		2r1000010000011
		2r111110001100
		2r10001000
		2r10001000
		2r1111000
		2r1001000
		2r1101000
		2r10000)
			offset: 0@0 name: 'thumbsDown').!

initThumbsUp
	Cursor constantNamed: #thumbsUp put: (Cursor
			extent: 16@16
			fromArray: 
				#(
		2r10000
		2r1101000
		2r1001000
		2r1111000
		2r10001000
		2r10001000
		2r111110001100
		2r1000010000011
		2r10000000000000
		2r11111100000000
		2r10000000000000
		2r11111100000000
		2r10000000000000
		2r1111100000111
		2r100000001000
		2r111111110000)
			offset: -15@0 name: 'thumbsUp').!

initTopRight
	Cursor constantNamed: #topRight put: (Cursor 
			extent: 16@16
			fromArray: #(
		2r1111111111111111
		2r1111111111111111
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011
		2r0000000000000011)
			offset: -14@-14
			name: 'topRight')!

initUp
	Cursor constantNamed: #up put: (Cursor 
			extent: 16@16
			fromArray: #(
		2r1000000000000000
		2r1100000000000000
		2r1110000000000000
		2r1111000000000000
		2r1111100000000000
		2r1111110000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000
		2r1100000000000000)
	 		offset: 0@-7 name: 'up')!

initWait
	Cursor constantNamed: #wait put: (Cursor
			extent: 16@16
			fromArray: #(
		2r1111111111111111
		2r1000000000000001
		2r0100000000000010
		2r0010000000000100
		2r0001110000111000
		2r0000111101110000
		2r0000011011100000
		2r0000001111000000
		2r0000001111000000
		2r0000010110100000
		2r0000100010010000
		2r0001000110001000
		2r0010001101000100
		2r0100111111110010
		2r1011111111111101
		2r1111111111111111)
			offset: 0@0
			name: 'wait')!

initWrite
	Cursor constantNamed: #write put: (Cursor
	extent: 16@16
	fromArray: #(
		2r0000000000000110
		2r0000000000001111
		2r0000000000010110
		2r0000000000100100
		2r0000000001001000
		2r0000000010010000
		2r0000000100100000
		2r0000001001000011
		2r0000010010000010
		2r0000100100000110
		2r0001001000001000
		2r0010010000001000
		2r0111100001001000
		2r0101000010111000
		2r0110000110000000
		2r1111111100000000)
	offset: 0@0
	name: 'write')!

initXeq
	Cursor constantNamed: #xeq put: (Cursor
			extent: 16@16
			fromArray: #(
		2r1000000000010000
		2r1100000000010000
		2r1110000000111000
		2r1111000111111111
		2r1111100011000110
		2r1111110001000100
		2r1111111001111100
		2r1111100001101100
		2r1111100011000110
		2r1001100010000010
		2r0000110000000000
		2r0000110000000000
		2r0000011000000000
		2r0000011000000000
		2r0000001100000000
		2r0000001100000000)
	offset: 0@0 
	name: 'xeq')! !


!Cursor class methodsFor: 'special'!

displayInView
	"display all defined cursor constants in a pretty square"
	"Cursor displayInView."

	| aFHView aFHController aColl aNum col lin theForm ePos cPos iPos lPos cnt aStr step aSSView cArray|

	step _ 80.
	aColl _ CursorConstants values.		"get ordered collection of Cursors"
	aNum _ aColl size.						"count 'em"
	cArray _ Array new: aNum.
	col _ (aNum sqrt + 1) asInteger.		"basic display = square"
	lin _ aNum // col.
	col * lin < aNum ifTrue: [				"can't i avoid this? (i.e., aNum = 70 or 75, col=9..)"
		lin _ lin + 1 ].
	ePos _ ((col * step) + 16) @ ((lin * step/2) + 16).		"display extent"
	cPos _ 16 @ 8.							"starting pos"
	iPos _ step@0.							"increment pos"
	lPos _ ((col - 1) * (step negated)) @ (step/2).			"line increment pos"
	theForm _ Form extent: ePos.			"set up form area"
	cnt _ 1.
	aColl do: [ :each | 						"loop through all instances"
		cArray at: cnt put: each.
		each displayOn: theForm at: (cPos - each offset).		"display cursor"
		aStr _ each name.					"get name"
		(aStr asDisplayText) displayOn: theForm at: (cPos + (-8@20)).
		cnt _ cnt + 1.						"update count and position"
		(cnt \\ col = 1) ifTrue: 				"if end of line.."
			[cPos _ cPos + lPos ]
		ifFalse: 
			[cPos _ cPos + iPos ]
	].											"end of select for all cursors"
	aFHView _ FormHolderView new model: theForm.
	aFHController _ FormHitController new 
		grid: (step @ (step/2)); 
		size: col@lin; 
		array: cArray.
	aFHView controller: aFHController.
	aSSView _ StandardSystemView new.
	aSSView addSubView: aFHView.
	aSSView label: 'Cursors'.
	aSSView borderWidth: 2.
	aSSView minimumSize: ePos; maximumSize: ePos.
	aSSView controller open.! !


Cursor initialize!


!Cursor class methodsFor: 'cursor constants'!

blank
	"Answer the instance of the receiver that is all white."

	^CursorConstants at: #blank ifAbsent: [nil]!

bottomLeft
	"Answer the instance of the receiver that is a bottom left corner."

	^CursorConstants at: #bottomLeft ifAbsent: [nil]!

bull
	"Answer the instance of the receiver that is a bull's-eye."

	^CursorConstants at: #bull ifAbsent: [nil]!

caret
	"Answer the instance of the receiver that is a caret."

	^CursorConstants at: #caret ifAbsent: [nil]!

corner
	"Answer the instance of the receiver that is the shape of the bottom right corner of
	a rectangle."

	^CursorConstants at: #corner ifAbsent: [nil]!

crossHair
	"Answer the instance of the receiver that is the shape of a cross."

	^CursorConstants at: #crossHair ifAbsent: [nil]!

down
	"Answer the instance of the receiver that is the shape of an arrow facing downward."

	^CursorConstants at: #down ifAbsent: [nil]!

execute
	"Answer the instance of the receiver that is the shape of an arrow slanted left
	with a star next to it."

	^CursorConstants at: #xeq ifAbsent: [nil]!

garbage
	"Answer the instance of the receiver that is the shape of a garbage can with 
	open lid."

	^CursorConstants at: #garbage ifAbsent: [nil]!

hand
	"Answer the instance of the receiver that is the shape of a hand."

	^CursorConstants at: #hand ifAbsent: [nil]!

marker
	"Answer the instance of the receiver that is displayed when thumb-scrolling."

	^CursorConstants at: #marker ifAbsent: [nil]!

normal
	"Answer the instance of the receiver that is the shape of an arrow slanted left."

	^CursorConstants at: #normal ifAbsent: [nil]!

number: index
	"return a cursor that looks like the given number"

	^NumberCursors at: index \\ 10 + 1!

origin
	"Answer the instance of the receiver that is the shape of the top left 
	corner of a rectangle."

	^CursorConstants at: #origin ifAbsent: [nil]!

read
	"Answer the instance of the receiver that is the shape of eyeglasses."

	^CursorConstants at: #read ifAbsent: [nil]!

scroll
	"Answer the instance of the receiver that is up and down arrows."

	^CursorConstants at: #scroll ifAbsent: [nil]!

scrollButton
	"Answer the instance of the receiver that is up and down arrows."

	^CursorConstants at: #scrollButton ifAbsent: [nil]!

thumbsDown
	"Answer the instance of the receiver that is a thumbs down."

	^CursorConstants at: #thumbsDown ifAbsent: [nil]!

thumbsUp
	"Answer the instance of the receiver that is thumbs up."

	^CursorConstants at: #thumbsUp ifAbsent: [nil]!

topRight
	"Answer the instance of the receiver that is a top right corner."

	^CursorConstants at: #topRight ifAbsent: [nil]!

up
	"Answer the instance of the receiver that is the shape of an arrow facing upward."

	^CursorConstants at: #up ifAbsent: [nil]!

wait
	"Answer the instance of the receiver that is the shape of an hour glass."

	^CursorConstants at: #wait ifAbsent: [nil]!

write
	"Answer the instance of the receiver that is the shape of a pen writing."

	^CursorConstants at: #write ifAbsent: [nil]! !

!Cursor class methodsFor: 'new constant initialization'!

initBar
		Cursor constantNamed: #bar put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0111111111111110
		2r0111111111111110
		2r0111111111111110
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0)
			offset: -7@-7 name: 'bar').!

initBDot
		Cursor constantNamed: #bDot put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0000000111000000
		2r0000001111100000
		2r0000011111110000
		2r0000111111111000
		2r0001111111111000
		2r0001111111111000
		2r0001111111110000
		2r0000111111100000
		2r0000011111000000
		2r0000001110000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0)
			offset: -7@-7 name: 'bDot').!

initBox

		Cursor constantNamed: #box put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0111111111111110
		2r0111111111111110
		2r0110000000000110
		2r0110000000000110
		2r0110000000000110
		2r0110000000000110
		2r0110000000000110
		2r0111111111111110
		2r0111111111111110
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0)
			offset: -7@-7 name: 'box').!

initCircle
		Cursor constantNamed: #circle put: (Cursor 
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000011111000000
		2r0001100000110000
		2r0010000000001000
		2r0100000000000100
		2r0100000000000100
		2r0100000000000100
		2r0100000000000100
		2r0010000000001000
		2r0001100000110000
		2r0000011111000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000)
			offset: -8@-9 name: 'circle').!

initCircle2
	"Cursor initCircle2"

		Cursor constantNamed: #circle2 put: (Cursor 
			extent: 16@16
			fromArray: #( 0 0 0 0 128 864 1040 2056 1040 864 128 0 0 0 0 0)
			offset: -8@-9 name: 'circle').!

initDiamond
		Cursor constantNamed: #diamond put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000100000000
		2r0000001110000000
		2r0000011111000000
		2r0000111111100000
		2r0001111111110000
		2r0011111111111000
		2r0111111111111100
		2r1111111111111111
		2r0111111111111110
		2r0011111111111100
		2r0001111111111000
		2r0000111111110000
		2r0000111111100000
		2r0000011111000000
		2r0000001110000000
		2r0000000100000000)
			offset: -7@-7 name: 'diamond').!

initDot
		Cursor constantNamed: #dot put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0
		2r0
		2r0
		2r0
		2r0000000110000000
		2r0000001111000000
		2r0000011111100000
		2r0000111111110000
		2r0000111111110000
		2r0000011111100000
		2r0000001111000000
		2r0000000110000000
		2r0
		2r0
		2r0
		2r0)
	offset: -8@-8 name: 'dot').!

initHole
		Cursor constantNamed: #hole put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0
		2r0
		2r0000000000000000
		2r0000001111000000
		2r0000011111100000
		2r0000110000110000
		2r0001100000011000
		2r0011000000001100
		2r0011000000001100
		2r0001100000011000
		2r0000110000110000
		2r0000011111100000
		2r0000001111000000
		2r0000000000000000
		2r0
		2r0)
	offset: -8@-8 name: 'hole').!

initLine
		Cursor constantNamed: #line put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0111111111111110
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0)
			offset: -7@-7 name: 'line').!

initNote
		Cursor constantNamed: #note put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0000001111000000
		2r0000011001100000
		2r0000110000110000
		2r0000110000110000
		2r0000011001100000
		2r0000001111000000
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: -8@-8 name: 'note').!

initSpec
		Cursor constantNamed: #spec put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000000000000
		2r0000000000000000
		2r0111100000111100
		2r0000000000000000
		2r0000000000000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000000000000
		2r0000000000000000
		2r0)
			offset: -7@-7 name: 'spec').!

initSpot
		Cursor constantNamed: #spot put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0000000110000000
		2r0000001111000000
		2r0000001111000000
		2r0000000110000000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: -8@-8 name: 'spot').!

initSquare
	Cursor constantNamed: #square put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0000001111000000
		2r0000001111000000
		2r0000001111000000
		2r0000001111000000
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0
		2r0)
	offset: -8@-8 name: 'square')!

initTriangle
		Cursor constantNamed: #triangle put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000001000000000
		2r0000001100000000
		2r0000011110000000
		2r0000011111000000
		2r0000111111100000
		2r0000111111110000
		2r0001111111111000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0)
			offset: -7@-7 name: 'triangle').!

initZ1
		Cursor constantNamed: #z1 put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0001111111110000
		2r0000000100000000
		2r0000000100000000
		2r0000000100000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0)
			offset: -7@-7 name: 'z1').!

initZ2
		Cursor constantNamed: #z2 put: (Cursor
			extent: 16@16
			fromArray: #(
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0000000100000000
		2r0000000100000000
		2r0000001110000000
		2r0000011111000000
		2r0011111111111000
		2r0000011111000000
		2r0000001110000000
		2r0000000100000000
		2r0000000100000000
		2r0000000000000000
		2r0000000000000000
		2r0000000000000000
		2r0)
			offset: -7@-7 name: 'z2').! !

!Cursor class methodsFor: 'new constants'!

bar
	"Answer the instance of me that is the shape of a short bar."
	^CursorConstants at: #bar ifAbsent: [nil]!

bDot
	"Answer the instance of me that is the shape of a small dot."
	^CursorConstants at: #bDot ifAbsent: [nil]!

box
	"Answer the instance of me that is the shape of a box."
	^CursorConstants at: #box ifAbsent: [nil]!

circle
	"Answer the instance of me that is the shape of a  mid-sized circle."
	^CursorConstants at: #circle ifAbsent: [nil]!

circle2
	"Answer the instance of me that is the shape of a  mid-sized circle."
	^CursorConstants at: #circle2 ifAbsent: [nil]!

diamond
	"Answer the instance of me that is the shape of a small diamond."
	^CursorConstants at: #diamond ifAbsent: [nil]!

dot
	"Answer the instance of me that is the shape of a small dot."
	^CursorConstants at: #dot ifAbsent: [nil]!

hole
	"Answer the instance of me that is the shape of a hole or note-head."
	^CursorConstants at: #hole ifAbsent: [nil]!

line
	"Answer the instance of me that is the shape of a line."
	^CursorConstants at: #line ifAbsent: [nil]!

note
	"Answer the instance of me that is the shape of a different note-head."
	^CursorConstants at: #note ifAbsent: [nil]!

spec
	"Answer the instance of me that is the shape of a special bulls eye."
	^CursorConstants at: #spec ifAbsent: [nil]!

spot
	"Answer the instance of me that is the shape of a a little spot."
	^CursorConstants at: #spot ifAbsent: [nil]!

square
	"Answer the instance of me that is the shape of a square."
	^CursorConstants at: #square ifAbsent: [nil]!

triangle
	"Return the TriangleCursor"

	^CursorConstants at: #triangle ifAbsent: [nil]!

z1
	"Answer the instance of me that is the shape of a short star."
	^CursorConstants at: #z1 ifAbsent: [nil]!

z2
	"Answer the instance of me that is the shape of a different short star."
	^CursorConstants at: #z2 ifAbsent: [nil]! !

Form subclass: #Cursor
	instanceVariableNames: 'name '
	classVariableNames: 'CurrentCursor CursorConstants NumberCursors '
	poolDictionaries: ''
	category: 'Graphics-Display Objects'!
Cursor comment:
'Instances of class Cursor are each represented as a 16 x 16 dot matrix suitable for use as the current cursor.

Instance Variable:
	name	<String>  providing a label for the cursor 

Class Variables:
	CurrentCursor	<Cursor> the cursor that is currently being displayed on the screen
	NumberCursors	<Array>
	CursorConstants	<Dictionary>
	The following are all the names of instances of class Cursor that are predefined in the system and stored in the dictionary CursorConstants.  They are accessible with messages to the class.

	BlankCursor BottomLeftCursor BullCursor CaretCursor CornerCursor CrossHairCursor DownCursor GarbageCursor HandCursor MarkerCursor NormalCursor OriginCursor ReadCursor ScrollCursor SquareCursor ThumbsDownCursor ThumbsUpCursor TopRightCursor UpCursor WaitCursor WriteCursor XeqCursor
	'!


Cursor initializeNew!
