MouseMenuController subclass: #BarGaugeController
	instanceVariableNames: 'oldValue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Gauges'!
BarGaugeController comment:
'BarGaugeController uses the view''s realBox for calculating the model''s new value so that the slider can be inset from the view''s insetDBox.
It also caches its old value so that is doesn''t update the model continuously when one holds the same value.'!


!BarGaugeController methodsFor: 'controlling'!

controlActivity
	sensor redButtonPressed 
		ifTrue: [^self redButtonActivity].
	sensor yellowButtonPressed
		ifTrue: [^self yellowButtonActivity].
	super controlActivity!

isControlActive
	"answer whether or not i'm active based on the state of the red and blue buttons and the cursor position"

	^(sensor redButtonPressed) or: [
		self viewHasCursor & sensor blueButtonPressed not]!

redButtonActivity
	"change the value of my model's model based on the cursor position and the view's orientation"

	| value range box newVal |
	box _ view realBox.					"get the view's inset slider box"
	range _ view range.					"and the view's interval"
	view orientation == #vertical 		"if vert. use y offset from the bottom"
		ifTrue: 
			[newVal _ (box corner y - sensor cursorPoint y) / box extent y].

	view orientation == #horizontal 		"if horiz. use x offset from the left"
		ifTrue: 
			[newVal _ (sensor cursorPoint x - box origin x) / box extent x].

	newVal _ (newVal * (range last - range first)) + range first.
	newVal _ ((newVal truncated min: (range last max: range first)) 
				max: (range first min: range last)).
	newVal = oldValue ifTrue: [ ^nil ].
	oldValue _ newVal.
"	Transcript show: newVal asInteger printString; cr."
	^model perform: view changeMsg with: newVal! !

View subclass: #GaugeView
	instanceVariableNames: 'range face origin value oldValue orientation changeMsg partMsg percentMsg '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Gauges'!


!GaugeView methodsFor: 'accessing'!

changeMsg
	"return the curernt change message selector"

	^changeMsg!

face

	"Return the instance variable 'face'."

	^ face!

oldValue

	"Return the instance variable 'oldValue'."

	^ oldValue!

origin

	"Return the instance variable 'origin'."

	^ origin!

origin: theOrigin

	"Accept the argument, 'theOrigin', as the new instance variable 'origin'."

	origin _ theOrigin.!

partMsg

	"Return the instance variable 'partMsg'."

	^ partMsg!

range
	"answer the range interval"

	^range!

range: theRange

	"Accept the argument, 'theRange', as the new instance variable 'range'."

	range _ theRange.!

value

	"Return the instance variable 'value'."

	^ value! !

!GaugeView methodsFor: 'initialize-release'!

on: anObject aspect: aspectM change: changeM range: aRange
	"actually set up the gauge view here"

	self model: anObject.
	partMsg _ aspectM.
	changeMsg _ changeM.
	range _ aRange.
	value _ model perform: partMsg!

on: anObject aspect: aspectM change: changeM range: aRange initialValue: val
	"actually set up the gauge view here"

	self model: anObject.
	partMsg _ aspectM.
	changeMsg _ changeM.
	range _ aRange.
	value _ val!

setFace
	"set the face form"

	face _ Form extent: insetDisplayBox extent.
	face white! !

!GaugeView methodsFor: 'displaying'!

displayFace
	"by default, show the face form"

				"if the face is uninitialized or we've been shrunken, reset face form"
	((face == nil) or: [face extent ~= insetDisplayBox extent])
		ifTrue: [self setFace].
	face displayOn: Display at: insetDisplayBox origin!

displayValue
	"override this in subclasses"
		
	^self subclassResponsibility!

displayView
	"show the components of the gauge view"

	self displayFace.
	self displayValue!

update: anAspect
	"by default, just query the model for the new value and redisplay"

	((anAspect == nil) or: [ anAspect == partMsg ]) ifTrue: [
		value _ model perform: partMsg.
		self displayView
	]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GaugeView class
	instanceVariableNames: ''!


!GaugeView class methodsFor: 'instance creation'!

on: anObject aspect: aspectMsg change: changeMsg range: aRange
	"Create a 'pluggable' gaugeView viewing anObject.
	aspectMsg is sent to read the value of the model.
	It is also used as the changed: parameter for this view.
	changeMsg is sent to inform anObject of a new value.
	range is an interval for the range of the display."

	^self new on: anObject aspect: aspectMsg change: changeMsg range: aRange!

on: anObject aspect: aspectMsg change: changeMsg range: aRange initialValue: val
	"Create a 'pluggable' gaugeView viewing anObject.
	aspectMsg is sent to read the value of the model.
	It is also used as the changed: parameter for this view.
	changeMsg is sent to inform anObject of a new value.
	range is an interval for the range of the display.
	initialValue is a default starting value."

	^self new on: anObject aspect: aspectMsg change: changeMsg range: aRange initialValue: val! !

GaugeView subclass: #RectangularGauge
	instanceVariableNames: 'box realBox sBorder labels style emphasis '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Gauges'!


!RectangularGauge methodsFor: 'initialize-release'!

initRealBox


	realBox _ ((insetDisplayBox origin + (insetDisplayBox extent * box origin) + (sBorder@sBorder))
		extent: insetDisplayBox extent * box extent - ((sBorder*2)@(sBorder*2))) rounded.
	realBox left: realBox left truncated.
	realBox top: realBox top truncated.
	realBox right: realBox right truncated.
	realBox bottom: realBox bottom truncated.

	^realBox!

makeLabels
	"make the display labels on the face form"

	| step pos labelStrings counter |
	style == nil ifTrue: [ style _ TextStyle default ].
	emphasis == nil ifTrue: [ emphasis _ 1 ].
	labels == nil
		ifTrue: ["no labels"
			box == nil ifTrue: [box _ self insetDisplayBox]]
		ifFalse: [
			((labels == true) or: [labels class == Array]) ifTrue:
					["display label values"
					orientation == #vertical ifTrue: [
						pos _ (3 @ (insetDisplayBox extent y - realBox extent y) / 2).
						step _ 0 @ (realBox extent y / (range size - 1)).
						step _ step - (0 @ (20 / range size)).
						(labels class == Array) ifTrue: [
								counter _ 0.
								range reverseDo: [ :aVal | 
									(DisplayText text: 
											(Text string: (labels at: (counter _ counter+1)) emphasis: emphasis) 
											textStyle: style) 
										displayOn: face at: pos.
									pos _ pos + step]]
							ifFalse: [
								range reverseDo: [ :aVal | 
									(DisplayText text: 
											(Text string: aVal printString emphasis: emphasis) 
											textStyle: style) 
										displayOn: face at: pos.
									pos _ pos + step]]].

					orientation == #horizontal ifTrue: [
						pos _ ((insetDisplayBox extent x - realBox extent x) / 2) @ 0.
						step _ (realBox extent x / (range size - 1)) @ 0.
						step _ step - ((20 / range size) @ 0).
						range do: [ :aVal | 
							aVal printString asDisplayText displayOn: face at: pos.
							pos _ pos + step]]]].

	face border: ((realBox align: insetDisplayBox origin with: 0@0) expandBy: sBorder)
		width: sBorder!

setFace
	"set the face form and draw the labels and slide border"

	super setFace.
	sBorder == nil ifTrue: [ sBorder _ 1 ].
	self initRealBox.
	self makeLabels.! !

!RectangularGauge methodsFor: 'displaying'!

display

	oldValue _ range first.
	self displayBorder.
	self reallyDisplayFace.
	self initRealBox.
	self displayValue!

reallyDisplayFace
	"by default, show the face form"

				"if the face is uninitialized or we've been shrunken, reset face form"
	((face == nil) or: [face extent ~= insetDisplayBox extent])
		ifTrue: [self setFace].
	face displayOn: Display at: insetDisplayBox origin! !

!RectangularGauge methodsFor: 'accessing'!

box

	"Return the instance variable 'box'."

	^ box!

box: theBox

	"Accept the argument, 'theBox', as the new instance variable 'box'."

	box _ theBox.!

emphasis: theemphasis
	"Accept the argument, 'theemphasis', as the new instance variable 'emphasis'."

	emphasis _ theemphasis.!

labels: theLabels

	"Accept the argument, 'theLabels', as the new instance variable 'labels'."

	labels _ theLabels.!

orientation

	"Return the instance variable 'orientation' (usually #vertical or #horizontal)."

	^ orientation!

orientation: theOrientation

	"Accept the argument, 'theOrientation', as the new instance variable 'orientation'."

	orientation _ theOrientation.!

realBox
	
	realBox == nil ifFalse: [
		^realBox ].
	^self initRealBox!

sBorder: theBorder

	"Accept the argument, 'theBorder', as the new instance variable 'sBorder'."

	sBorder _ theBorder.!

style: theStyle
	"Accept the argument, 'theStyle', as the new instance variable 'style'."

	style _ theStyle.! !

RectangularGauge subclass: #BarGauge
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Gauges'!


!BarGauge methodsFor: 'initialize-release'!

setFace
	"set the face form and draw the labels and slide border"

	super setFace.
	sBorder == nil ifTrue: [ sBorder _ 1 ].
	self initRealBox.
	self makeLabels.! !

!BarGauge methodsFor: 'displaying'!

displayFace
	"override this to make display more efficient"

	"if the face is uninitialized or we've been shrunken, reset face form "
	(face == nil or: [face extent ~= insetDisplayBox extent])
		ifTrue: 
			[self setFace.
			super displayFace]!

displayValue
	"show the filled rectangle for the current value using the proper orientation"

	| extVal rFirst rLast offset oldOffset originPos extentPos |
	rFirst _ range first.
	rLast _ range last.
	orientation == #vertical
		ifTrue: 
			[extVal _ realBox extent y.
					"offsets from the top in pixels; calculated from value and range "
			offset _ (((rLast - value) / (rLast - rFirst)) * extVal) asInteger.
			oldOffset _ (((rLast - oldValue) / (rLast - rFirst)) * extVal) asInteger.
					"top-left corner and size of box to fill gray"
			originPos _ realBox origin + (0 @ (offset min: oldOffset)).
			extentPos _ (realBox extent x @ (offset - oldOffset) abs) + (0@0)].
	orientation == #horizontal
		ifTrue:
			[extVal _ realBox extent x.
			offset _ (((value - rFirst) / (rLast - rFirst)) * extVal) asInteger.
			oldOffset _ (((oldValue - rFirst) / (rLast - rFirst)) * extVal) asInteger.
			originPos _ (realBox origin - (1@0)) + ((offset min: oldOffset) @ 0).
			extentPos _ (offset - oldOffset) abs @ (realBox extent y - sBorder)].

				"fill the display box with darkGray"
	Display
		fill: (originPos extent: extentPos)
		rule: Form over
		mask: (value > oldValue ifTrue: [Form darkGray] ifFalse: [Form white])!

reallyDisplayFace
	"by default, show the face form"

				"if the face is uninitialized or we've been shrunken, reset face form"
	((face == nil) or: [face extent ~= insetDisplayBox extent])
		ifTrue: [self setFace].
	face displayOn: Display at: insetDisplayBox origin!

update: anAspect
	"by default, just query the model for the new value and redisplay.
	This time, compare the new value to the cached old one and update only if it's different."

	| tempVal |
	((anAspect == nil) or: [ anAspect == partMsg ]) ifTrue: [
			tempVal _ model perform: partMsg.
			tempVal = value ifFalse: [
				oldValue _ value.
				value _ tempVal.
				self displayView
	] ]! !

!BarGauge methodsFor: 'controller access'!

defaultControllerClass
	"answer the class of my usual controller"

	^BarGaugeController! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BarGauge class
	instanceVariableNames: ''!


!BarGauge class methodsFor: 'examples'!

sample5a

	"set up a simple gauge view"
	"BarGauge sample5a"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 70@350.
	topView borderWidth: 2.
	topView label: 'Bar Gauge'.
	model _ NumberValue new value: 30.
	me _ BarGauge on: model aspect: #value change: #value: range: (0 to: 100 by: 25).
	me labels: true;
		box: ((0.5@0) corner: 1@1);
		orientation: #vertical.

	topView addSubView: me.

	topView controller open.!

sample5b

	"set up a simple gauge view"
	"BarGauge sample5b"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 350@50.
	topView borderWidth: 2.
	topView label: 'Bar Gauge'.
	model _ NumberValue new value: 3.
	me _ BarGauge on: model aspect: #value change: #value: range: (1 to: 5).
	me labels: true;
		box: ((0@0.5) corner: 1@1);
		orientation: #horizontal.

	topView addSubView: me.

	topView controller open.!

sample5c

	"set up a simple gauge view"
	"BarGauge sample5c"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 350@50.
	topView borderWidth: 2.
	topView label: 'Bar Gauge'.
	model _ NumberValue new value: 3.
	me _ BarGauge on: model aspect: #value change: #value: range: (0 to: 10 by: 2).
	me labels: true;
		box: ((0.02@0.5) corner: 0.98@0.95);
		orientation: #horizontal.
	topView addSubView: me.
	topView controller open.!

sample5d

	"set up a simple gauge view"
	"BarGauge sample5d"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 80@350.
	topView borderWidth: 2.
	topView label: 'Bar Gauge'.
	model _ NumberValue new value: 300.
	me _ BarGauge on: model aspect: #value change: #value: range: (0 to: 1000 by: 200).
	me labels: true;
		box: ((0.5@0.02) corner: 0.94@0.98);
		orientation: #vertical.
	topView addSubView: me.
	topView controller open.! !

GaugeView subclass: #ValueGauge
	instanceVariableNames: 'style emphasis typeCoerceMsg '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Gauges'!


!ValueGauge methodsFor: 'displaying'!

displayValue
	| val label | 

	typeCoerceMsg == nil 
			ifFalse: [ value _ value perform: typeCoerceMsg ].
	label _ DisplayText text: (Text string: value printString emphasis: emphasis)
					textStyle: style.
	label displayOn: Display
		at: insetDisplayBox center - (label extent // 2)
		clippingBox: insetDisplayBox.! !

!ValueGauge methodsFor: 'initialize-release'!

on: anObject aspect: aspectM change: changeM range: aRange
	"actually set up the value view here"

	self model: anObject.
	partMsg _ aspectM.
	changeMsg _ changeM.
	range _ aRange.
	value _ model perform: partMsg.
	self style: (TextStyle styleNamed: #default).
	self emphasis: 1! !

!ValueGauge methodsFor: 'accessing'!

emphasis: theEmphasis
	"Accept the argument, 'theEmphasis', as the new instance variable 'emphasis'."

	emphasis _ theEmphasis.!

style: theStyle
	"Accept the argument, 'theStyle', as the new instance variable 'style'."

	style _ theStyle.!

typeCoerceMsg: thetypeCoerceMsg
	"Accept the argument, 'thetypeCoerceMsg', as the new instance variable 'thetypeCoerceMsg'."

	typeCoerceMsg _ thetypeCoerceMsg.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ValueGauge class
	instanceVariableNames: ''!


!ValueGauge class methodsFor: 'examples'!

sample1a

	"set up a simple gauge view"
	"ValueGauge sample1a"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 80@350.
	topView borderWidth: 2.
	topView label: 'Bar Gauge with Value View'.
	model _ NumberValue new value: 7.
	me _ BarGauge on: model aspect: #value change: #value: range: (0 to: 10 by: 2).
	me labels: true;
		box: ((0.5@0.02) corner: 0.94@0.98);
		orientation: #vertical.
	topView addSubView: me
			in: (0@0 extent: 1@0.9)
			borderWidth: 1.

	me _ ValueGauge on: model aspect: #value change: #value: range: (0 to: 1000 by: 200).
	topView addSubView: me
			in: (0@0.9 extent: 1@0.1)
			borderWidth: 1.

	topView controller open.!

sample1b

	"set up a simple gauge view"
	"ValueGauge sample1b"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 80@350.
	topView borderWidth: 2.
	topView label: 'Bar Gauge with Value View'.
	model _ NumberValue new value: 7.
	me _ BarGauge on: model aspect: #value change: #value: range: (0 to: 10 by: 2).
	me labels: true;
		box: ((0.5@0.02) corner: 0.94@0.98);
		orientation: #vertical.
	topView addSubView: me
			in: (0@0 extent: 1@0.9)
			borderWidth: 1.

	me _ ValueGauge on: model aspect: #value change: #value: range: (0 to: 10 by: 2).
	topView addSubView: me
			in: (0@0.9 extent: 1@0.05)
			borderWidth: 1.

	topView addSubView: (GaugeButtonView
							on: model
							label: Cursor up deepCopy
							change: #increment)
			in: (0@0.95 extent: 0.5@0.05)
			borderWidth: 1.

	topView addSubView: (GaugeButtonView
							on: model
							label: Cursor down deepCopy
							change: #decrement)
			in: (0.5@0.95 extent: 0.5@0.05)
			borderWidth: 1.

	topView controller open.! !

RectangularGauge subclass: #SlideGauge
	instanceVariableNames: 'spread oldSpread '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Gauges'!


!SlideGauge methodsFor: 'initialize-release'!

on: anObject aspect: aspectM change: changeM percent: percentM range: aRange
	"actually set up the gauge view here"

	self model: anObject.
	partMsg _ aspectM.
	changeMsg _ changeM.
	percentMsg _ percentM.
					"value is a point"
	value _ (model perform: partMsg).
	(percentMsg isKindOf: Symbol)
		ifTrue: [ spread _ (model perform: percentMsg) ]
		ifFalse: [ spread _ percentMsg ].
	origin _ 0@0.			"default"
	range _ aRange!

setFace
	
	oldValue _ 0.
	oldSpread _ 0.
	super setFace! !

!SlideGauge methodsFor: 'displaying'!

displayFace

	"if the face is uninitialized or we've been shrunken, reset face form "
	(face == nil or: [face extent ~= insetDisplayBox extent])
		ifTrue: 
			[self setFace.
			super displayFace]!

displayValue
	"show the filled rectangle for the current value using the proper orientation"

	orientation == #vertical
		ifTrue: [ self displayValueVertical ]
		ifFalse: [ self displayValueHorizontal ]!

displayValueHorizontal
	"show the filled rectangle for the current value using the proper orientation"
	"SlideGauge sample3b"

	| extVal rFirst rLast offset oldOffset originPos cornerPos oldOrigin oldCorner rect oldRect |
	rFirst _ range first.
	rLast _ range last.
	extVal _ realBox extent x.
	offset _ ((((value * (1 + (spread/100))) - rLast) / (rLast - rFirst)) * extVal) asInteger.
	oldOffset _ ((((oldValue * (1 + (oldSpread/100))) - rLast) / (rLast - rFirst)) * extVal) asInteger.
	originPos _ realBox origin + ((offset max: 0)@0).
	oldOrigin _ realBox origin + ((oldOffset max: 0)@0).
	cornerPos _ (((realBox origin x + offset) + (realBox extent x * spread // 100)) min: realBox corner x) @ realBox corner y.
	oldCorner _ (((realBox origin x + oldOffset) + (realBox extent x * oldSpread // 100)) min: realBox corner x) @ realBox corner y.
	rect _ originPos corner: cornerPos.
	oldRect _ oldOrigin corner: oldCorner.
	(rect intersects: oldRect)
		ifTrue: [ (oldRect areasOutside: rect) do:
							[ :aRect |    Display white: aRect	].
						(rect areasOutside: oldRect) do:
							[ :aRect |    Display darkGray: aRect	]]
		ifFalse: [ Display white: oldRect.
						Display darkGray: rect.	].!

displayValueVertical
	"show the filled rectangle for the current value using the proper orientation"

	| extVal rFirst rLast offset oldOffset originPos cornerPos oldOrigin oldCorner rect oldRect |
	rFirst _ range first.
	rLast _ range last.
	extVal _ realBox extent y.
					"offsets from the top in pixels; calculated from value and range "
	offset _ (((rLast - (value * (1 + (spread/100))))
					 / (rLast - rFirst)) * extVal) asInteger.
	oldOffset _ (((rLast - (oldValue * (1 + (oldSpread/100))))
					 / (rLast - rFirst)) * extVal) asInteger.
"	Transcript show: ' ', offset printString."
					"top-left corner and size of box to fill gray"
		originPos _ realBox origin + (0 @ (offset max: 0)).
		oldOrigin _ realBox origin + (0 @ (oldOffset max: 0)).
		cornerPos _ realBox corner x @ 
			(((realBox origin y + offset) + (realBox extent y * spread // 100)) min: realBox corner y).
		oldCorner _ realBox corner x @ 
			(((realBox origin y + oldOffset) + (realBox extent y * oldSpread // 100)) min: realBox corner y).

	rect _ originPos corner: cornerPos.
	oldRect _ oldOrigin corner: oldCorner.

	(rect intersects: oldRect)
		ifTrue:	[	(oldRect areasOutside: rect) do:
							[ :aRect |    Display white: aRect	].
						(rect areasOutside: oldRect) do:
							[ :aRect |    Display darkGray: aRect	]]
		ifFalse:[	Display white: oldRect.
						Display darkGray: rect.	].!

update: anAspect
	"by default, just query the model for the new value and redisplay.
	This time, compare the new value to the cached old one and update only if it's different."

	| tempVal |
	((anAspect == nil) or: [ anAspect == partMsg ]) ifTrue: [
			tempVal _ (model perform: partMsg) asInteger.
			(tempVal = value) ifFalse: [
				oldValue _ value.
				oldSpread _ spread.
				value _ tempVal.
				self displayView
	] ] ifFalse: [
		anAspect == percentMsg ifTrue: [
			tempVal _ (model perform: percentMsg) asInteger.
			(tempVal = value) ifFalse: [
				oldValue _ value.
				oldSpread _ spread.
				spread _ tempVal.
				self displayView
	] ] ]! !

!SlideGauge methodsFor: 'accessing'!

percentMsg

	"Return the instance variable 'percentMsg'."

	^ percentMsg!

percentMsg: thePercentMsg

	"Accept the argument, 'thePercentMsg', as the new instance variable 'percentMsg'."

	percentMsg _ thePercentMsg.! !

!SlideGauge methodsFor: 'controller access'!

defaultControllerClass
	"answer the class of my usual controller"

	^BarGaugeController! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SlideGauge class
	instanceVariableNames: ''!


!SlideGauge class methodsFor: 'examples'!

sample3a

	"set up a simple gauge view"
	"SlideGauge sample3a"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 80@350.
	topView borderWidth: 2.
	topView label: 'Slide Gauge'.
	model _ NumberValue new value: 20.
	me _ SlideGauge on: model aspect: #value change: #value: percent: #value range: (0 to: 100 by: 20).
	me labels: true;
		orientation: #vertical;
		box: ((0.5@0.02) corner: 0.94@0.98);
		origin: 0@0.
	topView addSubView: me.
	topView controller open.!

sample3b
	"set up a simple gauge view"
	"SlideGauge sample3b"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 350@80.
	topView borderWidth: 2.
	topView label: 'Slide Gauge'.
	model _ NumberValue new value: 20.
	me _ SlideGauge on: model 
		aspect: #value 
		change: #value: 
		percent: #value 
		range: (0 to: 100 by: 20).
	me labels: true;
		orientation: #horizontal;
		box: ((0.5@0.02) corner: 0.94@0.98);
		origin: 0@1.
	topView addSubView: me.
	topView controller open.! !

!SlideGauge class methodsFor: 'instance creation'!

on: anObject aspect: aspectMsg change: changeMsg percent: percentM range: aRange
	"Create a 'pluggable' SlideView viewing anObject.
	aspectMsg is sent to read the value of the model.
	It is also used as the changed: parameter for this view.
	changeMsg is sent to inform anObject of a new value.
	range is an interval for the range of the display."

	^self new on: anObject aspect: aspectMsg change: changeMsg percent: percentM range: aRange! !

View subclass: #GaugeButtonView
	instanceVariableNames: 'label changedMsg '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Gauges'!


!GaugeButtonView methodsFor: 'displaying'!

displayView

	self clearInside.
	label displayOn: Display
		at: self insetDisplayBox center - (label extent // 2) "- label offset"
		clippingBox: self insetDisplayBox.!

indicatorReverse

	Display reverse: self insetDisplayBox.! !

!GaugeButtonView methodsFor: 'controller access'!

defaultControllerClass


	^SwitchController! !

!GaugeButtonView methodsFor: 'private'!

setLabel: aLabel change: aSelector

	aLabel == nil
		ifTrue:	[ label _ Form extent: 0@0	]
		ifFalse:	[ label _ aLabel ].

	(label isKindOf: DisplayObject)
		ifFalse:	[ label _ label asDisplayText	].

	changedMsg _ aSelector.
	self controller selector: aSelector! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GaugeButtonView class
	instanceVariableNames: ''!


!GaugeButtonView class methodsFor: 'instance creation'!

on: aModel label: aLabel change: aSelector
	"Create a new view, with a specified label, and a selector to send to the model
	when the view is red-button-selected"

	| view | 

	view _ self new.
	view model: aModel.
	view setLabel: aLabel change: aSelector.
	^view! !

SlideGauge subclass: #BeadGauge
	instanceVariableNames: 'screenFace bead '
	classVariableNames: 'BeadForm '
	poolDictionaries: ''
	category: 'Interface-Gauges'!


!BeadGauge methodsFor: 'accessing'!

bead: aForm
	"set a new bead form"

	bead _ aForm! !

!BeadGauge methodsFor: 'displaying'!

beadForm
	"Return the shape used to round the ends of the bead"

	bead == nil 
		ifTrue: [ ^BeadForm ]
		ifFalse: [ ^bead ]!

displayValue
	"show the filled rectangle for the current value using the proper orientation"

	orientation == #vertical
		ifTrue: [ self displayValueVertical ]
		ifFalse: [ self displayValueHorizontal ]!

displayValueHorizontal
	| offsetRealBox rFirst rLast extVal offset originPos cornerPos rect | 
	"show the filled rectangle for the current value using the proper orientation.
		Because of the odd shape, the easiest way is to copy the face, draw the bead
		on the copy, and display the copy."
	"BeadGauge sample3c"

	(screenFace == nil or: [screenFace extent ~= face extent])
		ifTrue:	[ screenFace _ Form extent: face extent	].
	face displayOn: screenFace.
	offsetRealBox _ realBox align: insetDisplayBox origin with: 0@0.
	rFirst _ range first.
	rLast _ range last.
	extVal _ offsetRealBox width.
								"offsets from the left in pixels; calculated from value and range "
	offset _ ((value / (rLast - rFirst)) * extVal) asInteger.

								"top-left corner and size of box to fill gray"
	originPos _ offsetRealBox leftCenter + (offset @ (self beadForm height // -2)).
	cornerPos _ offsetRealBox leftCenter + ((((offsetRealBox origin x + offset) + (offsetRealBox extent x * spread // 100)) 
				min: offsetRealBox corner x) @ (self beadForm height // 2)).

	rect _ originPos corner: cornerPos.
	self beadForm
		displayOn: screenFace
			at: rect leftCenter
			clippingBox: screenFace boundingBox
			rule: Form over
			mask: Form darkGray;
		displayOn: screenFace
			at: rect rightCenter
			clippingBox: screenFace boundingBox
			rule: Form over
			mask: Form darkGray.
	screenFace darkGray: rect.
	screenFace displayOn: Display at: insetDisplayBox origin
		clippingBox: insetDisplayBox
		rule: Form over
		mask: Form black.!

displayValueVertical
	| offsetRealBox rFirst rLast extVal offset originPos cornerPos rect | 
	"show the filled rectangle for the current value using the proper orientation.
		Because of the odd shape, the easiest way is to copy the face, draw the bead
		on the copy, and display the copy."

	(screenFace == nil or: [screenFace extent ~= face extent])
		ifTrue:	[screenFace _ Form extent: face extent].
	face displayOn: screenFace.
	offsetRealBox _ realBox align: insetDisplayBox origin with: 0@0.
	rFirst _ range first.
	rLast _ range last.
	extVal _ offsetRealBox height.
								"offsets from the top in pixels; calculated from value and range "
	offset _ (((rLast - (value * (1 + (spread/100))))
					 / (rLast - rFirst)) * extVal) asInteger.
"	Transcript show: ' ', offset printString."
								"top-left corner and size of box to fill gray"
	originPos _ offsetRealBox topCenter + (self beadForm width // -2 @ (offset max: 0)).
	cornerPos _ offsetRealBox bottomCenter x + (self beadForm width // 2) @ 
			(((offsetRealBox origin y + offset) + (offsetRealBox extent y * spread // 100)) 
				min: offsetRealBox corner y).
	rect _ originPos corner: cornerPos.
	self beadForm
			displayOn: screenFace
			at: rect topCenter
			clippingBox: screenFace boundingBox
			rule: Form over
			mask: Form darkGray;
		displayOn: screenFace
			at: rect bottomCenter
			clippingBox: screenFace boundingBox
			rule: Form over
			mask: Form darkGray.
	screenFace darkGray: rect.

	screenFace displayOn: Display at: insetDisplayBox origin
		clippingBox: insetDisplayBox
		rule: Form over
		mask: Form black.!

initRealBox
	"Create a new realBox that has the same top and bottom, but whose left and right are
		at the center of the default realBox.  This new realBox will act as the wire along
		which the bead slides"

	| center | 
	super initRealBox.
	orientation == #vertical ifTrue:
		[labels == false ifFalse:
			[center _ realBox center x rounded.
			realBox left: center;
					right: center]].
	^realBox! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BeadGauge class
	instanceVariableNames: ''!


!BeadGauge class methodsFor: 'initialize'!

initialize
	"BeadGauge initialize"

	BeadForm _ Form dotOfSize: 13! !

!BeadGauge class methodsFor: 'examples'!

sample3a

	"set up a simple gauge view"
	"BeadGauge sample3a"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 80@350.
	topView borderWidth: 2.
	topView label: 'Bead Gauge'.
	model _ NumberValue new value: 20.
	me _ BeadGauge on: model aspect: #value change: #value: percent: #value range: (0 to: 100 by: 20).
	me labels: true;
		orientation: #vertical;
		box: ((0.5@0.02) corner: 0.94@0.98);
		origin: 0@0.
	topView addSubView: me.
	topView controller open.!

sample3b
	"set up a simple gauge view"
	"BeadGauge sample3b"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 100@350.
	topView borderWidth: 2.
	topView label: 'Bead Gauge'.
	model _ NumberValue new value: 20.
	me _ BeadGauge compositeOn: model 
				aspect: #value change: #value: 
				percent: #value range: (0 to: 100 by: 20).
	topView addSubView: me.
	topView controller open.!

sample3c

	"set up a simple gauge view"
	"BeadGauge sample3c"

	| me model topView |
	topView _ StandardSystemView new.
	topView minimumSize: 400@80.
	topView borderWidth: 2.
	topView label: 'Bead Gauge'.
	model _ NumberValue new value: 20.
	me _ BeadGauge on: model 
		aspect: #value 
		change: #value: 
		percent: 20
		range: (0 to: 100 by: 20).
	me labels: false;
		orientation: #horizontal;
		box: ((0.02@0.02) corner: 0.98@0.98);
		origin: 0@0.
	topView addSubView: me.
	topView controller open.! !

!BeadGauge class methodsFor: 'instance creation'!

compositeOn: anObject aspect: aspectMsg change: changeMsg percent: percentM range: aRange
	"Create a composite 'pluggable' BeadView viewing anObject.
	It consists of a beadGauge, a valueView and +/- buttons."

	| myView me |
	me _ self new on: anObject aspect: aspectMsg change: changeMsg 
			percent: percentM range: aRange.
	me labels: true; orientation: #vertical;
		box: ((0.5@0.02) corner: 0.94@0.98); origin: 0@0;
		style: (TextStyle styleNamed: #small); emphasis: 1.
	myView _ View new.
	myView addSubView: me
			in: (0@0 extent: 1@0.8) borderWidth: 1.
	myView addSubView: 
			((ValueGauge on: anObject aspect: aspectMsg change: changeMsg range: aRange)
					emphasis: 2; typeCoerceMsg: #truncated)
			in: (0@0.8 extent: 1@0.1) borderWidth: 1.
	myView addSubView: 
			((GaugeButtonView on: anObject label: (Cursor up deepCopy offset: 6@0)
					change: ('increment', aspectMsg) asSymbol) insideColor: Form white)
			in: (0@0.9 extent: 0.5@0.1) borderWidth: 1.
	myView addSubView: 
			((GaugeButtonView on: anObject label: (Cursor down deepCopy offset: 4@0)
					change: ('decrement', aspectMsg) asSymbol) insideColor: Form white)
			in: (0.5@0.9 extent: 0.5@0.1) borderWidth: 1.
	^myView! !

BeadGauge initialize!


BeadGauge subclass: #BeadScrollGauge
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Gauges'!


!BeadScrollGauge methodsFor: 'displaying'!

displayValue
	"show the filled rectangle for the current value using the proper orientation"

	range _ model scrollExtentFor: partMsg.
	orientation == #vertical
		ifTrue: [ self displayValueVertical ]
		ifFalse: [ self displayValueHorizontal ]! !