View subclass: #OutlineBrowser
	instanceVariableNames: 'currentCategory '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Outlines'!
OutlineBrowser comment:
'An OutrlineBrowser holds onto an Outline and creates a simple two-paned view on the one-level hierarchy using pluggable view components (list and text views).
The Browser implements key-list menu items for adding new keys, removing or renaming existing ones, and inspecting or saving the outline.
Text menu items are implemented for execution and the like.

Instance variable:
	currentCategory		<Text> selected item in the list of the model''s keys'!


!OutlineBrowser methodsFor: 'accessing'!

currentCategory: aVal
	"set the selected category from outside"

	currentCategory _ aVal.
	self changed: #text! !

!OutlineBrowser methodsFor: 'organization list'!

organization: anOrg
	"stuff the model's outline"

	^model organization: anOrg!

organizationList
	"return the model's list"

	^model organizationList!

organizationMenu
	"return the menu for use in the list view"

	currentCategory == nil
		ifTrue: [^ActionMenu
					labels: 'add category\save\inspect' withCRs
					lines: #(1 2)
					selectors: #(addCategory saveAll inspect)].
	^ActionMenu
		labels: 'add category\rename\remove\save\inspect' withCRs
		lines: #(3 4)
		selectors: #(addCategory renameCategory removeCategory saveCategory inspect)! !

!OutlineBrowser methodsFor: 'text'!

acceptText: aText
	"accept the new text as the model's text for the current category"

	currentCategory == nil ifTrue: [^false].
	model at: currentCategory put: aText copy.
	self changed: #text.
	^true!

text
	"return the current text"

	currentCategory == nil ifTrue: [
		^'' asText
	] ifFalse: [
		^model at: currentCategory
	]!

textMenu
	"return a fancy version of the menu for the text view"

	currentCategory == nil ifTrue: [ ^nil ].
	^ActionMenu
			labels: 'again\undo\copy\cut\paste\do it\print it\inspect\accept\cancel\hardcopy\style\font' withCRs
			lines: #(2 5 8 10 11)
			selectors: #(again undo 
				copySelection cut paste 
				doIt printIt inspectIt 
				accept cancel 
				hardcopy 
				changeTextStyle changeFont)! !

!OutlineBrowser methodsFor: 'menu messages'!

addCategory
	"prompt for adding a new category"

	|newCategory|
	newCategory _ FillInTheBlank request: 'New Category' initialAnswer: (''). 
	newCategory = '' ifTrue: [^self].
	model at: newCategory put: Text new.
	currentCategory _ newCategory.
	self changed: #organizationList!

inspect
	"look at my model"

	^model inspect!

removeCategory
	"prompt the user and remove the selected category"

	(self confirm: 'Are you certain that you
want to remove category ', currentCategory, '?')
		ifTrue: 
		[model removeKey: currentCategory.
		currentCategory _ nil.
	].
	self changed: #organizationList.!

renameCategory
	"prompt the user for a new name and rename the selected category"

	| newCategory |
	newCategory _ FillInTheBlank request: 'New Category' initialAnswer: currentCategory. 
	newCategory = '' ifTrue: [^self].
	model at: newCategory put: (model at: currentCategory).
	model removeKey: currentCategory.
	currentCategory _ newCategory.
	self changed: #organizationList.!

saveAll
	"save the whole outline"

	model saveAll!

saveCategory
	"save the currentCategory"

	model saveCategory: currentCategory! !

!OutlineBrowser methodsFor: 'do it'!

doItContext
	^ nil!

doItReceiver
	^ nil!

doItValue: ignored! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OutlineBrowser class
	instanceVariableNames: ''!


!OutlineBrowser class methodsFor: 'instance creation'!

openFile: aName
	"OutlineBrowser openFile: 'HSTKv5.2Demo.st'."

	^self openWithButtonsOn: (Outline load: aName) label: aName!

openOn: anOrganization label: aLabel
	"OutlineBrowser openOn: Outline new label: 'Maintenance' "

	| me topView listView textView|
	me _ self new
			model: anOrganization.
	topView _ StandardSystemView new
			label: aLabel;
			minimumSize: 200@350;
			borderWidth: 1.
	listView _ SelectionInListView
		on: me
		aspect: #organizationList
		change: #currentCategory:
		list: #organizationList
		menu: #organizationMenu
		initialSelection: nil.
	textView _ CodeView 
		on: me
		aspect: #text
		change: #acceptText:
		menu: #textMenu.
	topView addSubView: me.
	textView controller: AlwaysAcceptCodeController new.
	topView addSubView: listView in: (0@0 extent: 1@0.3) borderWidth: 1.
	topView addSubView: textView in: (0@0.3 extent: 1@0.7) borderWidth: 1.
	topView controller open!

openWithButtonsOn: anOrganization label: aLabel
	"OutlineBrowser openWithButtonsOn: Outline new label: 'Maintenance' "
	"OutlineBrowser openWithButtonsOn: Outline someInstance label: 'System' "

	| me topView listView textView|
	me _ self new model: anOrganization.
	topView _ StandardSystemView new
			label: aLabel; minimumSize: 200@200; borderWidth: 1.
	listView _ SelectionInListView
		on: me aspect: #organizationList change: #currentCategory: 
		list: #organizationList menu: #organizationMenu initialSelection: nil.
	textView _ CodeView
		on: me aspect: #text change: #acceptText:
		menu: #textMenu.
	textView controller: AlwaysAcceptCodeController new.
	topView addSubView: listView 
			in: (0@0 extent: 1@0.25) borderWidth: 1.
	textView add5ButtonPanelOn: textView controller 
		to: topView in: (0@0.25 extent: 1@0.08).
	topView addSubView: textView 
			in: (0@0.33 extent: 1@0.67) borderWidth: 1.
	topView controller open! !

!OutlineBrowser class methodsFor: 'subview creation'!

add5ButtonPanelOn: model to: topView in: rectangle
	"add the button panel
	buttons are: doIt printIt inspect accept cancel "

	| number size offset button labels selectors |
	number _ 5.
	offset _  rectangle width / number.
	size _ rectangle origin extent: (offset @ rectangle height).
	labels _ Array new: number.
	labels at: 1 put: (Form constantNamed: #doIt);
		at: 2 put: (Form constantNamed: #printIt);
		at: 3 put: (Form constantNamed: #inspect);
		at: 4 put: ((Cursor thumbsUp copy) offset: 0@0);
		at: 5 put: ((Cursor thumbsDown copy) offset: 0@2).
	selectors _ #(doIt printIt inspectIt accept cancel).
	1 to: labels size do: 
		[ :counter |
		topView addSubView: 
			(self makeActionButtonFor: (labels at: counter) 
					selector: (selectors at: counter) on: model)
			in: size 
			borderWidth: 1.
	size _ size moveBy: offset@0 ]!

makeActionButtonFor: label selector: selector on: model
	"build and return a switch view on the model to perform the given selector"

	| button switchView |
	button _ Button newOff.
	button onAction: [model perform: selector].
	switchView _ SwitchView new model: button.
	switchView label: label.
"	switchView controller selector: #turnOn."
	^switchView! !

Model subclass: #Outline
	instanceVariableNames: 'organization fileName keySeparator entrySeparator '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Outlines'!
Outline comment:
'An outline is a simple interface to a text structure for browsing.
I can hold onto a keyed text list, parse it from/save it to files, and be used with an OutlineBrowser.

Instance variables:
	organization 		<Dictionary> the list of text key -> text entry
	fileName 			<String> the file i was read in from (when relevant)
	keySeparator		<String> the separator between a key and its entry
	entrySeparator 	<String> the separator an entry and the following key
'!


!Outline methodsFor: 'initialize-release'!

initialize
	"set up meaningful default values"

	organization _ Dictionary new.
	keySeparator _ String with: (Character cr).
	entrySeparator _ String with: (Character cr) with: (Character cr).! !

!Outline methodsFor: 'accessing'!

at: aKey
	"return the value out of the organization"

	^organization at: aKey!

at: aKey put: aVal
	"plug the value into the organization"

	organization at: aKey put: aVal!

name: aName
	"plug in the file name"

	fileName _ aName!

organization
	"return my organization (dictionary)"

	^organization!

organization: anOrg
	"stuff in my outline from the outside (being careful about nils)"

	anOrg == nil ifTrue: [ ^self ].
	organization _ anOrg!

organizationList
	"return the keys of the outline"

	^organization keys asSortedCollection!

removeKey: aKey
	"rip the value out of the organization"

	organization removeKey: aKey! !

!Outline methodsFor: 'parsing'!

parseFrom: aStream keySeparator: keyStr entrySeparator: entryStr
	"read my dictionary in from a stream, using the given separators (as strings)"

	| tmp key body |
	entrySeparator class == Character
		ifTrue: [entrySeparator _ String with: entryStr]
		ifFalse: [entrySeparator _ entryStr].
	keySeparator class == Character
		ifTrue: [keySeparator _ String with: keyStr]
		ifFalse: [keySeparator _ keyStr].
	[aStream atEnd]
		whileFalse: 
			[tmp _ ReadStream on: (aStream upToAll: entrySeparator).
			key _ tmp upToAll: keySeparator.
			body _ tmp upToEnd asText.
			organization at: key put: body]! !

!Outline methodsFor: 'fileIn/Out'!

saveAll
	"prompt the user for a name and write the whole outline on a fileStream"

	| aFS aName |
	aName _ fileName.
	aName == nil ifTrue: [ aName _ '' ].
	aName _ FillInTheBlank 
			request: 'file name for saved outline: ' 
			initialAnswer: aName.
	aName = '' ifTrue: [ ^self ].
	aFS _ FileStream fileNamed: aName.
	Cursor write showWhile:
			[self storeOn: aFS ].
	aFS close.!

saveCategory: aKey
	"prompt the user for a name and write the current text on a fileStream"

	| aFS aName |
	aName _ fileName.
	aName == nil ifTrue: [ aName _ '' ].
	aName _ FillInTheBlank 
			request: 'file name for saved outline: ' 
			initialAnswer: aName.
	aFS _ FileStream fileNamed: aName.
	self storeCategory: aKey on: aFS.
	aFS close.!

storeCategory: aCategory on: aStream
	"store the chosen category on a stream"

	aStream nextPutAll: aCategory, keySeparator, 
			(organization at: aCategory), entrySeparator!

storeOn: aStream
	"store the outline on a stream"

	organization keys asSortedCollection do: [ :aKey |
		aStream nextPutAll: aKey, keySeparator, 
				(organization at: aKey), entrySeparator
	]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Outline class
	instanceVariableNames: ''!


!Outline class methodsFor: 'instance creation'!

new
	"initialize me by default"

	^super new initialize! !

!Outline class methodsFor: 'loading'!

load: aFileName
	"read an outline in from the given file, using blank lines and double blank lines as key and entry separators"
	"Outliner load: 'WorkSpace'."

	^self load: aFileName keySep: '

'
		entrySep: '


'!

load: aFileName keySep: aKS entrySep: anES
	"read an outline in from the given file, using tabs and end-of-lines lines as key and entry separators"
	"Outline load: '/etc/passwd' keySep: ':' entrySep: (String with: Character cr)."

	| file org |
	file _(FileStream oldFileNamed: aFileName).
	org _self new.
	org name: aFileName.
	org parseFrom: file
		keySeparator: aKS
		entrySeparator: anES.
	^org! !