'From Smalltalk-80, Version 2.3 of 13 June 1988 on 23 December 1988 at 1:26:50 pm'!

!CodeView methodsFor: 'subview creation'!
add5ButtonPanelOn: theModel to: topView in: rectangle
	"add the button panel
	buttons are: doIt printIt inspect accept cancel "

	| number size offset button labels selectors |
	number _ 5.
	offset _  rectangle width / number.
	size _ rectangle origin extent: (offset @ rectangle height).
	labels _ Array new: number.
	labels at: 1 put: (Form constantNamed: #doIt);
		at: 2 put: (Form constantNamed: #printIt);
		at: 3 put: (Form constantNamed: #inspect);
		at: 4 put: ((Cursor thumbsUp copy) offset: 0@0);
		at: 5 put: ((Cursor thumbsDown copy) offset: 0@2).
	selectors _ #(doIt printIt inspectIt accept cancel).
	self addButtonPanelOn: theModel 
		labels: labels 
		selectors: selectors 
		to: topView 
		in: rectangle!

addButtonPanelOn: theModel labels: labels selectors: selectors to: topView in: rectangle
	"add the button panel
	buttons are: doIt printIt inspect accept cancel "

	| number size offset button |
	offset _  rectangle width / labels size.
	size _ rectangle origin extent: (offset @ rectangle height).
	1 to: labels size do: 
		[ :counter |
		topView addSubView: 
			(self makeActionButtonFor: (labels at: counter) 
					selector: (selectors at: counter) on: theModel)
			in: size 
			borderWidth: 1.
	size _ size moveBy: offset@0 ]!

makeActionButtonFor: label selector: selector on: theModel
	"build and return a switch view on the model to perform the given selector"

	| button switchView |
	button _ Button newOff.
	button onAction: [theModel perform: selector].
	switchView _ SwitchView new model: button.
	switchView label: label.
"	switchView controller selector: #turnOn."
	^switchView! !

!FileModel class methodsFor: 'instance creation'!
open: aFileModel named: aString 
	"Scheduled a view whose model is aFileModel and whose label is aString."
	| topView textView |
	topView _ StandardSystemView model: aFileModel label: aString minimumSize: 180 @ 180.
	textView _ CodeView on: aFileModel aspect: #text change: #acceptText:from:
				menu: #textMenu initialSelection: nil.
	textView add5ButtonPanelOn: textView controller 
		to: topView in: (0@0 extent: 1@0.08).
	topView addSubView: textView
		in: (0@0.08 extent: 1@0.92) borderWidth: 1.
	topView controller open!

open: aFileModel named: aString withController: aController
	"Scheduled a FileModel view with changes made in another editor."
	| topView textView |
	topView _ StandardSystemView model: aFileModel label: aString minimumSize: 180 @ 180.
	textView _ CodeView on: aFileModel aspect: #text change: #acceptText:from:
				menu: #textMenu initialSelection: nil.
	textView controller: aController.
	textView add5ButtonPanelOn: textView controller 
		to: topView in: (0@0 extent: 1@0.08).
	topView addSubView: textView in: (0@0.08 extent: 1@0.92) borderWidth: 1.
	topView controller open! !

!InspectorView class methodsFor: 'instance creation'!
openWithButtons: anInspector
	"Create and schedule an instance of me on the model, anInspector."
	
	^(self buildScheduledViewWithButtonsOn: anInspector) controller open!

viewWithButtons: anInspector in: area of: superView
	"Create proportioned List and Code views on anInspector in area of superView"

	| mid codeView |
	mid _ area left + (area width * 0.3).
	superView addSubView:
		(SelectionInListView
			on: anInspector printItems: anInspector printItems
			oneItem: false aspect: #field change: #field: list: #fieldList
			menu: #fieldMenu initialSelection: #field)
		in: (area copy right: mid; top: area top + 0.1) borderWidth: 1.
	self addListButtonsTo: superView 
			in: (area copy bottom: area top + 0.1; right: mid) 
			for: anInspector.
	codeView _ CodeView on: anInspector aspect: #text
			change: #acceptText:from:
			menu: #textMenu initialSelection: nil.
	superView addSubView: codeView
		in: (area copy left: mid; top: area top + 0.1) borderWidth: 1.
	self addTextButtonsTo: superView 
			in: (area copy bottom: area top + 0.1; left: mid) 
			for: codeView controller!

view: anInspector in: area of: superView
	"Create proportioned List and Code views on anInspector in area of superView"

	| mid textView |
	mid _ area left + (area width * 0.3).
	superView addSubView:
		(SelectionInListView
			on: anInspector printItems: anInspector printItems
			oneItem: false aspect: #field change: #field: list: #fieldList
			menu: #fieldMenu initialSelection: #field)
		in: (area copy right: mid) borderWidth: 1.
	textView _ CodeView on: anInspector aspect: #text
			change: #acceptText:from:
			menu: #textMenu initialSelection: nil.
	textView add5ButtonPanelOn: textView controller 
			to: superView in: ((area copy left: mid) bottom: area top + 0.08).
	superView addSubView: textView
		in: ((area copy left: mid) top: area top + 0.08) borderWidth: 1! !

!InspectorView class methodsFor: 'private'!
addListButtonsTo: aView in: aRectangle for: model
	"listView buttons are: inspect"

	| number size offset button |
	number _ 1.
	offset _ aRectangle width / number.
	size _ aRectangle origin extent: (offset @ aRectangle height).
	button _ self makeActionButtonFor: (Form constantNamed: #inspect)
			selector: #inspectField 
			on: model.
	aView addSubView: button 
			in: size borderWidth: 1.!

addTextButtonsTo: aView in: aRectangle for: model
	"listView buttons are: accept | cancel | copy | cut | paste"

	| number size offset button |
	number _ 5.
	offset _ aRectangle width / number.
	size _ aRectangle origin extent: (offset @ aRectangle height).
	button _ self makeActionButtonFor: ((Cursor thumbsUp copy) offset: 0@0)
		selector: #accept on: model.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: ((Cursor thumbsDown copy) offset: 0@2)
		selector: #cancel on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: 'copy' asDisplayText
		selector: #copySelection on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: 'cut' asDisplayText
		selector: #cut on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: 'paste' asDisplayText 
		selector: #paste on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.!

buildScheduledView: anInspector 
	| topView |
	topView _ self model: anInspector label: anInspector object class name minimumSize: 180 @ 180.
	self view: anInspector in: (0@0 extent: 1@1) of: topView.
	^ topView!

buildScheduledViewWithButtonsOn: anInspector
	"build a newStyle inspector view on the argument"
 
	| topView |
	topView _ self model: anInspector 
		label: anInspector object class name 
		minimumSize: 180 @ 180.
	self viewWithButtons: anInspector in: (0@0 extent: 1@1) of: topView.
	^ topView!

makeActionButtonFor: label selector: selector on: model
	"build and return a switch view on the model to perform the given selector"

	| button switchView |
	button _ Button newOff.
	button onAction: [model perform: selector].
	switchView _ SwitchView new model: button.
	switchView label: label.
	switchView controller selector: #turnOn.
	^switchView! !


!AlwaysAcceptCodeController methodsFor: 'menu messages'!
accept
	(model changeRequestFrom: view)
		ifFalse: [^view flash].
	self controlTerminate.
	(view accept: self text from: self)
		ifTrue: [initialText _ paragraph text copy]
		ifFalse: [view flash].
	(view containsPoint: sensor cursorPoint) 
		ifTrue: [self controlInitialize]! !

!TextController methodsFor: 'menu messages'!
accept
	(self textHasChanged and: [model changeRequestFrom: view])
		ifFalse: [^view flash].
	self controlTerminate.
	(view accept: self text from: self)
		ifTrue: [super accept]
		ifFalse: [view flash].
	(view containsPoint: sensor cursorPoint) 
		ifTrue: [self controlInitialize]! !

!CodeController methodsFor: 'menu messages'!
doIt
	"Evaluate the current text selection as an expression"
	| result selectionStart oldTextSize selection |
	self controlTerminate.
	selectionStart _ startBlock stringIndex.
	oldTextSize _ self text size.
	selection _ self selection.
	result _ 
		model doItReceiver class evaluatorClass new
				evaluate: self selectionAsStream
				in: model doItContext
				to: model doItReceiver
				notifying: self
				ifFail: 
					[self controlInitialize.
					^#failedDoit].
	self selection asString = selection asString ifFalse:
		[self selectFrom: selectionStart  "Reselect doIt range after compiler interaction"
			to: selectionStart + selection size - 1 + (self text size - oldTextSize)].
	Smalltalk logChange: self selection string.
	model doItValue: result.
	(view containsPoint: sensor cursorPoint) 
		ifTrue: [self controlInitialize].
	^result! !

!CodeController methodsFor: 'menu messages'!
inspectIt
	"Evaluate the current text selection as an expression"
	| result selectionStart oldTextSize selection |
	self controlTerminate.
	selectionStart _ startBlock stringIndex.
	oldTextSize _ self text size.
	selection _ self selection.
	result _ 
		model doItReceiver class evaluatorClass new
				evaluate: self selectionAsStream
				in: model doItContext
				to: model doItReceiver
				notifying: self
				ifFail: 
					[(view containsPoint: sensor cursorPoint) 
					ifTrue: [self controlInitialize].
					^#failedDoit].
	self selection asString = selection asString ifFalse:
		[self selectFrom: selectionStart  "Reselect doIt range after compiler interaction"
			to: selectionStart + selection size - 1 + (self text size - oldTextSize)].
	Smalltalk logChange: self selection string.
	model doItValue: result.
	result inspect! !

!ParagraphEditor methodsFor: 'menu messages'!
cancel
	"Restore the text of the paragraph to be the text saved since initialization or
	the last accept."

	self controlTerminate.
	UndoSelection _ paragraph text.
	view clearInside.
	self changeParagraph: (paragraph text: initialText).
	paragraph displayOn: Display.
	self scrollToTop.
	(view containsPoint: sensor cursorPoint) 
		ifTrue: [self controlInitialize]! !
