'From Smalltalk-80, Version 2.4 of 28 January 1989 on 16 February 1989 at 3:53:51 pm'!

!ScreenController class methodsFor: 'class initialization'!

initializeHSTK
	"Initialize the System Menu for HSTK V5.0."
	"ScreenController initializeHSTK.
	ScreenController allInstancesDo: [:c | c initializeYellowButtonMenu]"

	ScreenYellowButtonMenu _
		PopUpMenu
			labels: 
'restore display
garbage collect
exit project
browser
workspace
file list
file editor
terminal
project
screen saver
system transcript
system outline
suspend
save
quit'
			lines: #(3 9 10 12).
	ScreenYellowButtonMessages _
			#(restoreDisplay garbageCollect exitProject
			openBrowser openWorkspace openFileList openFileEditor openTerminal openProject 
			openScreenSaver
			openTranscript openSystemOutline
			openDeskTop save quit).! !


!MidiDevice methodsFor: 'initialization'!

initialize: portNum
	"set me up on the given output port."

	Transcript show: 'MidiDevice init...'; cr.
"	port == nil ifTrue: [
		port _ (MidiPort portNamed: portNum) readAppendStream ].
	port == nil ifTrue: [
		port _ (MidiPort allInstances at: 1) readAppendStream ].
	port setMidiMode.
"	num _ portNum.
	MStream nextPutAll: 'MidiInit: ', portNum printString; cr.
	^self!

reInitialize
	"restore my port when restarting me"

	port == nil 
		ifTrue: [port _ (MidiPort open: num) readAppendStream].
	port == nil 
		ifTrue: [port _ (MidiPort allInstances at: 1) readAppendStream ].
	port setMidiOut.
	Transcript show: 'MidiPort reInit...'; cr.! !

!Midi class reorganize!
('mousing around' fromMouse)
('playing' allOff play:at:dur:amp: play:at:dur:amp:voice:)
('constants' midiMode rawMode)
('primitive operations' close getClockTime getMode getReceiveCount getTickSize getTransmitCount isOpen killIO next: nextEvent: open openAndInitialize put:count: set:data: setClockTime: setReadMode: setReceiveOff setReceiveOn setTickSize: status: test)
('accessing' getLong:at: putLong:into:at:)
!



!Object methodsFor: 'testing'!

isEventList
	"to find them in a crowd"

	^false!

isNumber
	"not me"
	
	^false! !

!Object methodsFor: 'converting'!

=>  anObject

	^ (EventAssociation key: self value: anObject)! !


!Score methodsFor: 'performance'!

playFrom: time rate: aRate
	"Plays the entire score at the given rate. aRate is 1 for normal speed, 2 for twice speed, 3/4 for three-quarter speed, etc."

	| mSecsAtStart currTime |
	MidiInterface initialize.
		"Scan for the first note at or after the given time, processing all control changes and 		program changes during this scanning to establish the proper synthesizer state."
	self reset; sendEventsThrough: time - 1 silent: true.
	mSecsAtStart _ (Time millisecondClockValue) - (time * 10 // aRate).
	[self done or: [Sensor blueButtonPressed]]
		whileFalse: [
			currTime _ ((Time millisecondClockValue - mSecsAtStart) * aRate) // 10.
			self sendEventsThrough: currTime silent: false.
			MidiInterface flushOutbuf.
		].
"	MidiInterface allNotesOff."	"Turn off notes in case I was interrupted."
	MidiInterface flushOutbuf.! !

!Score methodsFor: 'private'!

turnOffActiveNotes
	"Turns off all notes in the active list."

	activeNotes do: [ :note |
		MidiInterface noteOff: (note midiKey) chan: (note chan)].
	activeNotes _ activeNotes species new.! !


!MidiInterface class methodsFor: 'port control'!

openPort
	"Open the RS-232C port unless debugging is turned on."
	"MidiInterface openPort."

	(Debug not & Port isNil)
		ifTrue: [Port _ MidiPort open: '/dev/ttyb'.
			 Port setRawMode; setMidiMode].

	SendBuf _ BufferStream on: (String new: 100).! !

!MidiInterface class methodsFor: 'low level - output'!

send: midiCode with: arg1 with: arg2 on: channel
	"Sends a two-argument midi channel message. Checks arguments and channel number."

	| ch cmd |

	ch _ channel - 1.
	cmd _ midiCode bitOr: ch.

	((ch bitAnd: 15) == ch)
		ifFalse: [self error: 'bad channel number']
		ifTrue:
			[(((arg1 bitAnd: 127) == arg1) and: [(arg2  bitAnd: 127) == arg2])
				ifFalse: [self error: 'bad Midi argument']
				ifTrue:
					[Debug
						ifTrue:
							[Transcript show:
										"(Time millisecondClockValue / 1000) asInteger printString,"
										((cmd printStringRadix: 16), ' ',
										 (arg1 printStringRadix: 16), ' ',
										 (arg2 printStringRadix: 16), '\') withCRs]
						ifFalse:
							[SendBuf nextPut: cmd asCharacter.
							 SendBuf nextPut: arg1 asCharacter.
							 SendBuf nextPut: arg2 asCharacter]]].! !

!MidiRecorder reorganize!
('initialize-release' reset resetMidiTable)
('midi filtering' ignoreChan: ignoreCmd: recordOnly:)
('midi monitor' monitor printCmd:with:with: testmonitor:)
('record/overdub' echoTest overDub:playFrom:rate: record)
('private - recording' recordData recordData: recordSysExclusive: recordTwo: start stop)
('private - table msgs' badTableEntry: data: endSysExclusive: ignore: ignoreOne: ignoreTwo: recordOne: undefined:)
('private - other' asScore convertTimes: convertTimes:rate: growAll processByte: setDur:endTime:)
!



!SlideGauge methodsFor: 'initialize-release'!

on: anObject aspect: aspectM change: changeM percent: percentM range: aRange
	"actually set up the gauge view here"

	self model: anObject.
	partMsg _ aspectM.
	changeMsg _ changeM.
	percentMsg _ percentM.
					"value is a point"
	value _ (model perform: partMsg).
	(percentMsg isKindOf: Symbol)
		ifTrue: [ spread _ (model perform: percentMsg) ]
		ifFalse: [ spread _ percentMsg ].
	origin _ 0@0.			"default"
	range _ aRange! !


!CodeView methodsFor: 'subview creation'!

add5ButtonPanelOn: theModel to: topView in: rectangle
	"add the button panel
	buttons are: doIt printIt inspect accept cancel "

	| number size offset button labels selectors |
	number _ 5.
	offset _  rectangle width / number.
	size _ rectangle origin extent: (offset @ rectangle height).
	labels _ Array new: number.
	labels at: 1 put: (Form constantNamed: #doIt);
		at: 2 put: (Form constantNamed: #printIt);
		at: 3 put: (Form constantNamed: #inspect);
		at: 4 put: ((Cursor thumbsUp copy) offset: 0@0);
		at: 5 put: ((Cursor thumbsDown copy) offset: 0@2).
	selectors _ #(doIt printIt inspectIt accept cancel).
	self addButtonPanelOn: theModel 
		labels: labels 
		selectors: selectors 
		to: topView 
		in: rectangle!

addButtonPanelOn: theModel labels: labels selectors: selectors to: topView in: rectangle
	"add the button panel
	buttons are: doIt printIt inspect accept cancel "

	| number size offset button |
	offset _  rectangle width / labels size.
	size _ rectangle origin extent: (offset @ rectangle height).
	1 to: labels size do: 
		[ :counter |
		topView addSubView: 
			(self makeActionButtonFor: (labels at: counter) 
					selector: (selectors at: counter) on: theModel)
			in: size 
			borderWidth: 1.
	size _ size moveBy: offset@0 ]!

makeActionButtonFor: label selector: selector on: theModel
	"build and return a switch view on the model to perform the given selector"

	| button switchView |
	button _ Button newOff.
	button onAction: [theModel perform: selector].
	switchView _ SwitchView new model: button.
	switchView label: label.
"	switchView controller selector: #turnOn."
	^switchView! !


!InspectorView class methodsFor: 'instance creation'!

openWithButtons: anInspector
	"Create and schedule an instance of me on the model, anInspector."
	
	^(self buildScheduledViewWithButtonsOn: anInspector) controller open!

view: anInspector in: area of: superView
	"Create proportioned List and Code views on anInspector in area of superView"

	| mid textView |
	mid _ area left + (area width * 0.3).
	superView addSubView:
		(SelectionInListView
			on: anInspector printItems: anInspector printItems
			oneItem: false aspect: #field change: #field: list: #fieldList
			menu: #fieldMenu initialSelection: #field)
		in: (area copy right: mid) borderWidth: 1.
	textView _ CodeView on: anInspector aspect: #text
			change: #acceptText:from:
			menu: #textMenu initialSelection: nil.
	textView add5ButtonPanelOn: textView controller 
			to: superView in: ((area copy left: mid) bottom: area top + 0.08).
	superView addSubView: textView
		in: ((area copy left: mid) top: area top + 0.08) borderWidth: 1!

viewWithButtons: anInspector in: area of: superView
	"Create proportioned List and Code views on anInspector in area of superView"

	| mid codeView |
	mid _ area left + (area width * 0.3).
	superView addSubView:
		(SelectionInListView
			on: anInspector printItems: anInspector printItems
			oneItem: false aspect: #field change: #field: list: #fieldList
			menu: #fieldMenu initialSelection: #field)
		in: (area copy right: mid; top: 0.1) borderWidth: 1.
	self addListButtonsTo: superView 
			in: (area copy bottom: 0.1; right: mid) 
			for: anInspector.
	codeView _ CodeView on: anInspector aspect: #text
			change: #acceptText:from:
			menu: #textMenu initialSelection: nil.
	superView addSubView: codeView
		in: (area copy left: mid; top: 0.1) borderWidth: 1.
	self addTextButtonsTo: superView 
			in: (area copy bottom: 0.1; left: mid) 
			for: codeView controller.! !

!InspectorView class methodsFor: 'private'!

addListButtonsTo: aView in: aRectangle for: model
	"listView buttons are: inspect"

	| number size offset button |
	number _ 1.
	offset _ aRectangle width / number.
	size _ aRectangle origin extent: (offset @ aRectangle height).
	button _ self makeActionButtonFor: (Form constantNamed: #inspect)
			selector: #inspectField 
			on: model.
	aView addSubView: button 
			in: size borderWidth: 1.!

addTextButtonsTo: aView in: aRectangle for: model
	"listView buttons are: accept | cancel | copy | cut | paste"

	| number size offset button |
	number _ 5.
	offset _ aRectangle width / number.
	size _ aRectangle origin extent: (offset @ aRectangle height).
	button _ self makeActionButtonFor: ((Cursor thumbsUp copy) offset: 0@0)
		selector: #accept on: model.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: ((Cursor thumbsDown copy) offset: 0@2)
		selector: #cancel on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: 'copy' asDisplayText
		selector: #copySelection on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: 'cut' asDisplayText
		selector: #cut on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: 'paste' asDisplayText 
		selector: #paste on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.!

buildScheduledViewWithButtonsOn: anInspector
	"build a newStyle inspector view on the argument"
 
	| topView |
	topView _ self model: anInspector 
		label: anInspector object class name 
		minimumSize: 180 @ 180.
	self viewWithButtons: anInspector in: (0@0 extent: 1@1) of: topView.
	^ topView!

makeActionButtonFor: label selector: selector on: model
	"build and return a switch view on the model to perform the given selector"

	| button switchView |
	button _ Button newOff.
	button onAction: [model perform: selector].
	switchView _ SwitchView new model: button.
	switchView label: label.
	switchView controller selector: #turnOn.
	^switchView! !


!MidiDevice class methodsFor: 'class initialization'!

initialize
	"Set up the one class variable"
	"MidiDevice initialize"

	MStream _ "Transcript" WriteStream on: (String new: 1024).! !
MidiDevice initialize!

!Voice class reorganize!
('class initialization' initialize reset)
('constant access' defaultVoice named:)
!



!EventScheduler methodsFor: 'running'!

runAppointments
	"Set up the first meetings and then run them all till no one wants a meeting anymore."

	running ifTrue: [ ^nil ].
	self setUpFirstAppointments.
	self running: true.
	[(running == false) or: [appointments isEmpty]] whileFalse:
		[self callNextAppointment].
	self running: false.! !

!PlainPealComposition reorganize!
('initialization' initialize)
('composition' compose composeLength:initialOrder: composeOneLength:initialOrder:)
('reasons' cogent:with: optionsFor:from:to:)
('creation' containsIn:from:to: endsIn:from:to: firstPealOrder:from:to: pitchesFor: staggersIn:from:to: startsIn:from:to:)
('spliting' nextChild:from:to: noSplit:from:to: split:from:to: splitOnce:from:to: splitThrice:from:to: splitTwice:from:to:)
('private' addPeal: play totalDurationFor:base:)
!



!EventInspectorView class methodsFor: 'private'!

addListButtonsTo: aView in: aRectangle for: model
	"listView buttons are: inspect"

	| number size offset button |
	number _ 2.
	offset _ aRectangle width / number.
	size _ aRectangle origin extent: (offset @ aRectangle height).
	button _ self makeActionButtonFor: (Form constantNamed: #inspect)  selector: #inspectField on: model.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: 'add prop' asDisplayText selector: #addProperty on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.! !

!EventList reorganize!
('initialize-release' initializeAnonymous initializeNamed:)
('accessing' add: add:at: addAll: addModifier: code: deepCopy defaultEventClass duration: empty events events: index: isEventList links linkTo: next removeModifier: reset shallowCopy species tempo:)
('collecting' collect: do: expanded expandedFrom:to: includes: notesDo: remove1: remove: select: timesDo:)
('processing' addVoice: apply:to: apply:to:startingAt: chooseDuration: chooseLoudness: chooseVoice: scaleDuration: scaleLoudness: scaleTempo: setDuration: setLoudness: setVoice: spreadDuration: spreadLoudness:)
('scheduling' play playAt: playOn: playOn:at: playPrim playPrimAt: scheduleAt:)
('user interface' edit inspect)
('code conversion' asScore findNoteOff0: findNoteOff: setDurations setDurations0)
('printing' printOn:)
('private' map: map:at:)
('code conversions' setRanges)
!



!EventList methodsFor: 'scheduling'!

play
	"play NoteEvent data from my instance variables starting now"
	"(EventLists named: #temp.1) play."

	^self asScore play
"	^self playOn: self voice at: (Time millisecondClockValue)
	^self playPrim"! !

MidiVoice comment:
'A MidiVoice is used to play note events onto one or more output devices.

I store MidiDevices in my instruments and play events onto them when asked to.

Instance variables:
	instruments	<set of Associations>	my instruments, stored as channel -> MidiDevice
'!


!EventListDictionaryInspectorView class methodsFor: 'private'!

addListButtonsTo: aView in: aRectangle for: model
	"listView buttons are: inspect play edit"

	| number size offset button |
	number _ 3.
	offset _ aRectangle width / number.
	size _ aRectangle origin extent: (offset @ aRectangle height).
	button _ self makeActionButtonFor: (Form constantNamed: #inspect) 
		selector: #inspectField on: model.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: (Form constantNamed: #eighthNote) 
		selector: #play on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: ((Icon constantNamed: #score) form figure) 
		selector: #edit on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.!

buildScheduledViewOn: anInspector
	"build a newStyle inspector view on the argument"
 
	| topView |
	topView _ super buildScheduledViewWithButtonsOn: anInspector.
	topView label: 'Event List Dictionary'.
"	topView cacheRefresh: false."
	^ topView! !


!MidiVoice class methodsFor: 'instance creation'!

newNamed: aName onDevice: aMidiDevice channel: aChannel
	"set up a MidiVoice on the given device and channel"
	"MidiVoice newNamed: 'xxx' onDevice: (MidiFB01 newOn: 1) channel: 1"

	| me |
	me _ self new initializeNamed: aName.
	me instruments add: (aChannel -> aMidiDevice).
	^me! !
