MouseMenuController subclass: #EventListEditorController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-ListEditors'!
EventListEditorController comment:
'EventListEditorControllers are used for the EventListEditors and send messages 
to their models, who are generally EventListEditors.'!


!EventListEditorController methodsFor: 'initialize-release'!

initialize
	"set up my YBMenu"

	super initialize.
	self yellowButtonMenu: (PopUpMenu labels: 
'delete
select
group
inspect
play
redraw')
		yellowButtonMessages: #(delete select group inspect play redraw)! !

!EventListEditorController methodsFor: 'control defaults'!

controlActivity 
	"Wait for a button press and put up a note head.  Then poll for a duration."
	| screenPoint formPoint evt |
	super controlActivity.
	(sensor redButtonPressed) ifTrue: 
		[ sensor leftShiftDown 
			ifFalse: [ screenPoint _ sensor cursorPoint.      
					"send message to ElEd view--he'll wait for button up."
"				Transcript show: 'sP: ', screenPoint printString; tab."
				evt _ view eventAt: screenPoint.
"				Transcript show: 'nP: ', evt key printString, 
					'@', evt value pitch printString; cr."
				model add: evt ]
			ifTrue: [ ^self scrollActivity ]]!

isControlActive
	"Inherits control from superclass MouseMenuController.
	As the blue blutton menu is used for reframing views only, EE Functions are to 
be activated
	by pressing the other two buttons."

	^super isControlActive & sensor blueButtonPressed not!

scrollActivity
	"Implement Psychic Scrolling the new way."

	| pt x y pos amount |
	pos _ ((view insetDisplayBox extent) / 2) + view insetDisplayBox origin.
	[sensor redButtonPressed]
		whileTrue: [
				pt _ sensor cursorPoint.
				amount _ (view containsPoint: pt) ifTrue: [2] ifFalse: [4].
				x _ pt x.
				y _ pt y.
				x < pos x ifTrue: [view scrollRight: amount]
				ifFalse: [view scrollLeft: amount].
				y < pos y ifTrue: [view scrollDown: amount]
				ifFalse: [view scrollUp: amount].
				view displayView.
			]! !

!EventListEditorController methodsFor: 'menu messages'!

delete
	"delete old events"

 	| note |
	Cursor crossHair showWhile: [
		sensor waitButton.
		note _ self locateEventAt: sensor cursorPoint ].
	Transcript show: note printString; cr.
	note isNil ifFalse: [ model remove: note ].
	view update.
	self centerCursorInView.!

erase
	"clear out the list"

	model eraseList.
	self centerCursorInView!

group
	"select some sub EventList"

 	| note point |
	[sensor yellowButtonPressed] whileFalse: [
		Cursor crossHair showWhile: [
			sensor waitButton.
			sensor yellowButtonPressed ifFalse: [
				point _  sensor cursorPoint.
				note _ self locateEventAt: point.
			note isNil ifFalse: [
				model select: note.
				view select: note at: point ].
			sensor waitNoButton ]].
	].
	sensor waitNoButton.
	view update!

inspect
	"inspect the event list"

	sensor leftShiftDown
		ifFalse: [ model inspectList ]
		ifTrue: [ view inspect ].
	self centerCursorInView!

play
	"play or start/stop the selected EventList"
	
	| start |
	sensor leftShiftDown 
		ifTrue: ["model startStop"
				start _ Time millisecondClockValue.
				[model play] forkAt: 4.
				self followFromPaging: start]
		ifFalse: [model play].
	self centerCursorInView.!

redraw
	"redisplay the view from scratch"

	view redraw!

select
	"select an event an inspect it."

	| note |
	Cursor crossHair showWhile: [
		sensor waitButton.
		note _ self locateEventAt: sensor cursorPoint ].
	note isNil ifFalse: [ note inspect ].
	view update.
	self centerCursorInView! !

!EventListEditorController methodsFor: 'private'!

followFrom: start

	| end now scale sync |
	end _ start + model list duration.
	view offsetX: 0.
	scale _ view timeScale.
	sync _ scale * 150.
	[now _ Time millisecondClockValue.
	now < (end + 1000)] whileTrue:
			[(now > (start + sync) and: [now < (end - sync)])
				ifTrue: 
					[view offsetX: 
						(((now - start - sync) / scale) negated)].
"			Transcript show: (now-start) printString; tab."
			view highlight: now-start.
			Processor yield].!

followFromPaging: start

	| end now scale width oWidth |
	end _ start + model list duration.
	view offsetX: 0.
	scale _ view timeScale.
	width _ view boundingBox width * 3.
	oWidth _ width.
"	Transcript show: 'width=', width printString; cr."
	[now _ Time millisecondClockValue.
	now < (end + 1000)] whileTrue:
			[view highlight: now-start.
			(now - start) >= width
				ifTrue: ["Transcript show: 'scroll...'; cr."
						view offsetX: (((now - start) / scale) negated).
						width _ width + oWidth].
			Processor yield].!

locateEventAt: aPoint
	"find the nearest event to the given point."

	| notePoint timeThreshold pitchThreshold |
	timeThreshold _ 80. "ms units"
	pitchThreshold _ 1. "arbitrary"
	notePoint _ view roundToNote:
			(view relativeCoordinatesFor: (aPoint - view insetDisplayBox origin)).
"	Transcript show: 'looking for: ', notePoint printString; cr."
	model list events do: [ :anA |
		(((anA key - notePoint x) abs) < timeThreshold) ifTrue: [
			(((anA value pitch - notePoint y) abs) < pitchThreshold) ifTrue: [
				^anA]]].
	Transcript show: 'missed...'; cr.
	sensor waitNoButton.
	^nil! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EventListEditorController class
	instanceVariableNames: ''!


View subclass: #EventListView
	instanceVariableNames: 'timeScale timeGrain timeOffset pitchScale pitchGrain pitchOffset form clefForm dOffset icon instrument highlightForm '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-ListEditors'!
EventListView comment:
'EventListView is the abstract class or the various time-line-oriented HSTK 
event list editors.
Its concrete subclasses implement the note-to-position mapping and user interaction.

Instance Variables:
	timeScale 	<Integer> the number of MS per pixel 
	timeGrain 	<Integer> thge time grain size, used for rounding
	timeOffset	<Integer> the offset from 0 for the beginning of the editor
	pitchScale	<Integer> the number of pixels per halfstep
	pitchGrain	<Integer> the pitch grain size, used for rounding
	pitchOffset	<Integer> the pitch for pixel 0
	form		<Form> the display form of the editor
	clefForm	<Form> the form for the clefs
	dOffset		<Point> the display offset, used for scrolling
	icon		<Form> the current notehead icon
	instrument	<Integer> the current default instrument number
	highlightForm	<Form> the form to use for highlioghted notes
	'!


!EventListView methodsFor: 'accessing'!

form
	"return my bitmap image"

	^form!

icon
	"return the current icon"

	^icon!

icon: aCursor
	"set the current icon"

	icon _ aCursor!

instrument: aCursor number: aNumber
	"use the given instrument as my default"

	icon _ aCursor.
	instrument _ aNumber!

offset
	"return my display offset (used for scrolling)"

	^dOffset!

timeScale
	^timeScale! !

!EventListView methodsFor: 'scrolling'!

offsetX
	"return the x value of my offset"

	^dOffset x!

offsetX: newX
	"set the x value of my offset if newX is a legal value"

					"if we need to scroll to next page"
	(newX + (timeOffset / timeScale) - self insetDisplayBox width) 
			>= (form width negated) 
		ifFalse: [timeOffset _ (newX * timeScale negated) truncated.
			self redraw].
					"if we need to scroll to previous page"
	newX + (timeOffset / timeScale) > 0
		ifTrue: [timeOffset _ 0 max: (timeOffset - (form width * timeScale)).
			self redraw].
	dOffset x <= 0
		ifTrue: [dOffset x: newX truncated.
			self changed: #offsetX.
			self displayView.
"			Transcript show: newX asInteger printString, ' '."
"			Sensor leftShiftDown ifTrue: [self halt]"]!

offsetY
	"return the y value of my offset"

	^dOffset y!

offsetY: newY
	"set the y value of my offset if newY is a legal value"

	(dOffset y >= (form extent y negated + (self insetDisplayBox extent y)) 
					and: [dOffset y <= 0])
		ifTrue:
			[ dOffset y: newY truncated.
			self changed: #offsetY.
			self displayView ]!

scrollContinuously
	"scroll me down by amount.  this is called by the scroll button and uses the 
distance from the center of the view to compute the scroll amount."

	| origin pt x y | 
	origin _ self insetDisplayBox center.
	[Sensor redButtonPressed] whileTrue: [].
	[Sensor redButtonPressed] whileFalse: [].
	[Sensor redButtonPressed] whileTrue:
		[ pt _ Sensor cursorPoint - origin.
		x _ pt x.
		y _ pt y.
"		Transcript show: 'scroll: ', pt printString; cr."
		x > 0 ifTrue:
				[ self scrollLeft: ((x / 20) max: 4) truncated ]
			ifFalse:
				[ self scrollRight: ((x negated / 20) max: 4) truncated ].
		y > 0 ifTrue:
				[ self scrollUp: ((y / 20) max: 3) truncated ]
			ifFalse:
				[ self scrollDown: ((y negated / 20) max: 3) truncated ].
		self displayView ]!

scrollDown
	"scroll down 2 pixels"

	self scrollDown: -2.
	self displayView!

scrollDown: amount
	"scroll me down by amount."

	dOffset y < 0 ifTrue:
		[ dOffset _ dOffset + (0@amount)].
	self changed: #offsetY!

scrollExtentFor: partMsg
	"return the range of scroll values that are possible"

	| xVal |
	partMsg == #offsetX
		ifTrue: [xVal _ (model list duration / timeScale) max: form width.
				^(0 to: (xVal - self insetDisplayBox width) negated)].
	^(0 to: (form height - self insetDisplayBox height) negated)!

scrollLeft
	"scroll left 2 pixels"

	self scrollLeft: -2.
	self displayView!

scrollLeft: amount
	"scroll me left by amount."

	(dOffset x - self insetDisplayBox extent x) > (form extent x negated) ifTrue:
		[ dOffset _ dOffset - (amount@0)].
	self changed: #offsetX!

scrollRight
	"scroll right 2 pixels"

	self scrollRight: -2.
	self displayView!

scrollRight: amount
	"scroll me right by amount."

	dOffset x < 0 ifTrue:
		[ dOffset _ dOffset + (amount@0)].
	self changed: #offsetX!

scrollUp
	"scroll up 2 pixels"

	self scrollUp: -2.
	self displayView!

scrollUp: amount
	"scroll me up by amount."

	dOffset y > (form extent y negated + (self insetDisplayBox extent y)) ifTrue:
		[ dOffset _ dOffset - (0@amount)].
	self changed: #offsetY!

xPercent
	"return the percent of the x value i'm currently scrolled (used by scroll bars)"

	^(form extent x / ((dOffset x + 1) *100)) asInteger!

xPercent: aFloat
	"set the x scroll percent"

	dOffset x: (form extent x * aFloat)!

yPercent
	"return the percent of the y value i'm currently scrolled (used by scroll bars)"

	^(form extent y / ((dOffset y + 1) *100)) asInteger!

yPercent: aFloat
	"set the y scroll percent"

	dOffset y: (form extent y * aFloat)! !

!EventListView methodsFor: 'displaying'!

displayView
	"Display an EventListEditorView offset by its offset."

	| box shift |
	box _ self insetDisplayBox.
	form displayOn: Display 
		at: (box origin + dOffset + ((timeOffset / timeScale)@0)) 
		clippingBox: box rule: Form over mask: Form black.
	clefForm displayOn: Display 
		at: ((box origin x)@(box origin y + dOffset y)) 
		clippingBox: box rule: Form paint mask: Form black!

highlight: aTime
	"put a flashing line at the given time"

	| line point |
	point _ (insetDisplayBox left + (aTime / timeScale) + dOffset x)
				@ insetDisplayBox top.
	line _ (Form extent: 3@insetDisplayBox height) darkGray.
	2 timesRepeat: 
		[line displayOn: Display
			at: point
			clippingBox: insetDisplayBox 
			rule: Form reverse 
			mask: Form black].!

redraw
	"used to regenerate view."

	| selections realList |
	"MessageTally spyOn: ["
	Cursor wait showWhile:
		[self initForm.
		selections _ model selection.
		realList _ model list expandedFrom: timeOffset 
							to: timeScale * form width + timeOffset.
		realList do:
			[ :anAssoc |
				self display: anAssoc selected: (selections includes: anAssoc).
				Processor yield].
		self displayView]
	"]"!

refresh
	"used to redisplay view."

	self displayView.!

update: anAspect
	"redraw all on deletion"

	anAspect == #deleted ifTrue: [
		self redraw
	].
	anAspect == #selected ifTrue: [
		self refresh
	].! !

EventListView subclass: #HSEventListView
	instanceVariableNames: 'cachedForm cachedBlt '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-ListEditors'!
HSEventListView comment:
'HSEventListViews are simple editor views that implement Hauer-Steffens-style 
music notation (linear time-to-x and pitch-to-y piano-roll-style notation).
In this notation, lines are drawn on the staff for all accidentals (i.e., there 
are lines where there are black keys on a piano).

Instance variables
	cachedForm		<Form> my background form (staves)
	cachedBlt		<BitBlt> the line-drawer i use for durations of notes

Class instance variables (one per subclass)
	Clef 		<Form> my clef form backup
	Staff 		<Form> my staff form backup'!


!HSEventListView methodsFor: 'initialize-release'!

initialize
	"set up the default parameters and create the clef and staff forms"

	super initialize.
	self defaultParameters.
	form _ self class staff.
	clefForm _ self class clef.
	cachedBlt _ BitBlt destForm: form 
			sourceForm: (Form dotOfSize: 2) 
			halftoneForm: Form black
			combinationRule: Form over 
			destOrigin: 0@0 
			sourceOrigin: 0@0
			extent: form extent
			clipRect: form boundingBox.!

release
	"throw the model away"

	model release.
	super release.! !

!HSEventListView methodsFor: 'interaction'!

eventAt: aPoint 
	"take a point and put up the right Event icon there.  
	Return a NoteEvent to the controller who passes it on to the model. "

	| origin displayPoint roundedPoint duration association relativePoint |
	origin _ insetDisplayBox origin.
										"point relative to whole form"
	relativePoint _ self relativeCoordinatesFor: (aPoint - origin).
"	Transcript show: ' rP: ', relativePoint printString."
										"rounded display point"
	displayPoint _  (self displayCoordinatesFor: relativePoint) + origin.	
										"show note position"
	icon displayOn: Display at: displayPoint
		clippingBox: insetDisplayBox
		rule: Form paint 
		mask: Form black.
	(Sensor redButtonPressed) 			"now track note duration"
		ifTrue: [ duration _ self trackEvent: displayPoint inset: insetDisplayBox ]
		ifFalse: [ duration _ 0. ].
	roundedPoint _ self roundToNote: relativePoint.
						"generate a time -> event association for returning to the controller."
	association _ (roundedPoint x) =>
			(model defaultEventClass 
				dur: (duration * timeScale) asInteger
				pitch: roundedPoint y 
				voice: instrument).
	Transcript show: '  ',  association printString.
									"put the note on the form."
"	self display: association selected: false.	  --not necessary if we believe 
the above scaling works
	self displayView.
"	^association!

trackEvent: pt inset: box
	"track an active event's length while active"

	| newPt oldPt x y xmin xmax savedRect savedForm line |
					"cache the area around the note"
	savedRect _ Rectangle origin: (pt +(-8@-4)) corner: ((box corner x)@(pt y +4)).
	savedForm _ Form fromDisplay: savedRect.
					"create a 2-pixel high black line"
	line _ Line new form: ((Form new) extent: 2 @ 2; black).
	y _ pt y - 1.
	xmin _ pt x.
	xmax _ 0.
	x _ 0.
	line beginPoint: (pt - (0@1)).
					"track the cursor's x-coordinate as long as the redButton's pressed"
	[Sensor redButtonPressed] whileTrue: [
			newPt _ Sensor cursorPoint.
			newPt ~= oldPt ifTrue: [
			x _ (newPt x max: xmin) roundTo: timeGrain.
							"erase the line if moving left."
			xmax > x ifTrue: [
					savedForm displayOn: Display 
							at: savedRect origin].
			line endPoint: x @ y.
							"display the line"
			line displayOn: Display 
				at: 0 @ 0 
				clippingBox: box.
			xmax _ x.
			oldPt _ newPt]].
				"return the x-length of the line"
	^x - pt x! !

!HSEventListView methodsFor: 'displaying'!

display: anAss selected: aBool
	"Display an Event in an EventListEditorView"

	| anE displayPoint myBox voice |
	anE _ anAss value.
	myBox _ form boundingBox.
	displayPoint _ (self formXFromTime: anAss key) @ 
				(self formYFromPitch: anE pitch).
	anE voice == nil 
		ifFalse: [voice _ anE voice ]
		ifTrue: [voice _ 1 ].
"	Transcript show: ' dP: ', displayPoint printString; cr."
	aBool ifTrue: 
		[((MusicConstants at: #HighlightIcons) at: (voice max: 1)) 
			displayOn: form 
			at: displayPoint
			clippingBox: myBox 
			rule: Form paint 
			mask: Form black]
	ifFalse: 
		[((MusicConstants at: #NoteIcons) at: (voice max: 1)) 
			displayOn: form 
			at: displayPoint
			clippingBox: myBox 
			rule: Form paint 
			mask: Form black].
	anE dur notNil ifTrue:
		[displayPoint _ displayPoint + (1@0).
		cachedBlt destForm: form; drawFrom: displayPoint 
			to: displayPoint + ((anE dur asMS / timeScale) truncated @ 0)]!

select: aNote at: aPoint
	"display the fact that the given note has been selected"

	self display: aNote selected: true.
	model changed: #selected.! !

!HSEventListView methodsFor: 'transformations'!

displayCoordinatesFor: aPoint
	"transform the given point from form-relative back into display coordinates"

	| yOff xOff |
	xOff _ clefForm width + dOffset x.
	yOff _ form height + dOffset y.
	^((aPoint x + xOff) @ (yOff - aPoint y))!

formXFromTime: aTime
	"take a note's time and return a display point x coordinate relative to the form."

	^((aTime - timeOffset / timeScale) asInteger) + clefForm width!

formYFromPitch: aPitch
	"take a note's pitch and return a display point y coordinate relative to the form."

	^form height - ((aPitch - pitchOffset) * pitchScale)!

relativeCoordinatesFor: aPoint
	"return the point transformed to be relative the editor's form from the lower-left"

	| yOff xOff |
	xOff _ clefForm width + dOffset x.
	yOff _ form height + dOffset y.
	^self roundToGrid:
		((aPoint x - xOff) @ (yOff - aPoint y ))!

roundToGrid: aPoint
	"take a screen point and justify it to my grain sizes."

	^((aPoint x) roundTo: timeGrain rounded) @ ((aPoint y) roundTo: pitchGrain rounded)!

roundToNote: aPoint
	"take a rounded view point and justify it to my eventListEditor's note sizes."

	| time |
	time _ aPoint x roundTo: timeGrain.
	^((time * timeScale + timeOffset) rounded) @ ((aPoint y // pitchScale + 
pitchOffset) rounded)! !

!HSEventListView methodsFor: 'scrolling'!

scrollContinuously
	"zoom or scroll with this button"

	| factor string |
	Sensor leftShiftDown
		ifFalse: [ ^super scrollContinuously ]
		ifTrue: [ string _ FillInTheBlank
					request: 'please enter a reasonable scroll zoom factor'
					initialAnswer: '1.5'.
			string = '' ifFalse:
				[ factor _ string asNumber.
				factor = 0 ifFalse:
					[ self zoomBy: factor ]]]! !

!HSEventListView methodsFor: 'zooming'!

zoomBy: factor
	"enlarge or shrink the display and redraw"

	pitchScale _ pitchScale / factor.
	pitchGrain _ pitchGrain / factor.
	pitchOffset _ pitchOffset / factor.
	timeScale _ timeScale * factor.
	timeGrain _ timeGrain * factor.
"	Transcript show: 'scale: ', timeScale printString; cr."
	form _ Form extent: 800@400.
	form white.
	clefForm _ Form extent: 32@800.
	clefForm white.	
	self class HSstaff: form clef: clefForm scale: pitchScale grain: pitchGrain.
	self class barLines: form scale: timeScale.
	cachedForm _ form deepCopy.
	self redraw.! !

!HSEventListView methodsFor: 'private'!

defaultParameters
	"set up the default display parameters"

	timeOffset _ 0.				"view offset"
	timeScale _ 20.				"ms/pixel"
	timeGrain _ 10.				"min event offset"
	pitchOffset _ 14.			"lowest note"
	pitchScale _ 4.				"well tempered"
	pitchGrain _ 4.				"spacing between accidentals"
	dOffset _ 0@-150.
	icon _ Cursor note.
	instrument _ 1!

initForm
	"set up my graphical form for scrolling around on."

	cachedForm == nil 
		ifTrue: [ form _ self class staff]
		ifFalse: [ form _ cachedForm deepCopy ]
"	| formSize |
	formSize _ 800@400.
	form _ Form extent: formSize.
	form white.
	self HSstaff.
	self barLines
"! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

HSEventListView class
	instanceVariableNames: 'Clef Staff '!


!HSEventListView class methodsFor: 'class initialization'!

barLines: form scale: theScale
	"Put barlines every other second onto the form."

	| box t1 b1 lf1 rt1 blt offset count |
	box _ form boundingBox.
	t1 _ box top + 1.
	b1 _ box bottom - 1.
	lf1 _ box left.
	rt1 _ box right.
	offset _ 2000 / theScale.			"scale is ms per pixel"
	blt _ BitBlt destForm: form 
			sourceForm: (Form dotOfSize: 1) 
			halftoneForm: Form black
			combinationRule: Form over 
			destOrigin: 0@0 
			sourceOrigin: 0@0
			extent: (b1-t1)@1 
			clipRect: box.
	count _ lf1 + 32 + offset.
	[count < rt1] whileTrue: 
		[ blt drawFrom: count@t1 to: count@b1.
		   count _ count + offset ].!

barLines: form scale: theScale offset: theO
	"Put barlines every other second onto the form."

	| box t1 b1 lf1 rt1 blt offset count |
	box _ form boundingBox.
	t1 _ box top + 1.
	b1 _ box bottom - 1.
	lf1 _ box left.
	rt1 _ box right.
	offset _ 2000 / theScale.			"scale is ms per pixel"
	blt _ BitBlt destForm: form 
			sourceForm: (Form dotOfSize: 1) 
			halftoneForm: Form black
			combinationRule: Form over 
			destOrigin: 0@0 
			sourceOrigin: 0@0
			extent: (b1-t1)@1 
			clipRect: box.
	count _ lf1 + theO + offset.
	[count < rt1] whileTrue: 
		[blt drawFrom: count@t1 to: count@b1.
		   count _ count + offset].!

clef: anInteger at: aPoint bottom: aBottom form: clefForm
	"Display a C clef for the given octave at the given point"

	| oct par |
	(aPoint y + 20) < aBottom ifTrue: 
		[ (DisplayText text: (Text string: 'c' , anInteger printString emphasis: 2) 
				textStyle: TextStyle default)
			displayOn: clefForm at: aPoint + (2 @ -3) rule: Form over ]!

HSstaff: form clef: clefForm scale: theScale grain: theGrain
	"display a Hauer-Stephens-type staff onto a form"

	| box top1 bottom1 left1 right1 aNum count octave blt |
							"get the bounds of the display box"
	box _ form boundingBox.
	top1 _ box top.
	bottom1 _ box bottom.
	left1 _ box left + 1.
	right1 _ box right - 1.
							"cache the line-drawer BitBlt for the staff lines"
	blt _ BitBlt destForm: form 
			sourceForm: (Form dotOfSize: 1) 
			halftoneForm: Form black
			combinationRule: Form over 
			destOrigin: 0@0 
			sourceOrigin: 0@0
			extent: (right1-left1)@1 
			clipRect: box.
									"Count number of octaves"
	octave _ ((bottom1 - top1) / (5 + (theScale * 8) + (theGrain))) asInteger.
	aNum _ top1.
	count _ 3.
"		Transcript show: '[ ', aNum printString, ' < ', bottom1 printString, ']...'; 
cr. "
	[aNum < bottom1] whileTrue: 				" loop from top1 to bottom1 drawing lines "
		[ blt drawFrom: left1@aNum to: right1@aNum.
		count _ count + 1.
		count \\ 5 = 1 ifTrue:  			 " skip @ c "
			[ self clef: octave at: left1@aNum bottom: bottom1 form: clefForm.
			octave _ octave - 1.
			aNum _ aNum + theScale ].			
		count \\ 5 = 4 ifTrue:  		      " skip @ e "
			[ aNum _ aNum + theScale].
		aNum _ aNum +  (theScale * 2)]!

initialize
	"HSEventListView initialize.
	PianoView initialize"

	Staff _ Form extent: 2400@400.
	Staff white.
	Clef _ Form extent: 32@400.
	Clef white.	
	self HSstaff: Staff clef: Clef scale: 4 grain: 4.
	self barLines: Staff scale: 20.! !

!HSEventListView class methodsFor: 'class instance variables'!

clef
	"return the Class InstVar"

	^Clef deepCopy!

staff
	"return the Class InstVar"

	^Staff deepCopy! !

!HSEventListView class methodsFor: 'instance creation'!

open
	"open the default HSEventListView"
	"HSEventListView open"

	^HSEventListView openSimpleOn: EventListEditor defaultEventList!

openDualFancy
	"Open a view for a new eventListEditor composed of two editing subviews with 
special buttons added for standard menu operations"
	"HSEventListView openDualFancy"

	| topView eventListView1 eventListView2 bWidth isWidth bOffset eOffset eWidth 
eHeight 
		lButtonRect1 lButtonRect2 rButtonRect1 rButtonRect2 |

	topView _ StandardSystemView new label: 'Multiple H-S Event List Editor';
			minimumSize: 600@400; 
			borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	bWidth _ 0.025.			bOffset _ 0.02.			isWidth _ 0.035.
	eOffset _ 0.1.				eWidth _ 0.85.			eHeight _ 0.47.
	lButtonRect1 _ ((0.01 @ 0.02) extent: (0.025@eHeight)).
	lButtonRect2 _ ((0.01 @ 0.51) extent: (0.025@eHeight)).
	rButtonRect1 _ (0.955@0.02 extent: isWidth@eHeight).
	rButtonRect2 _ (0.955@0.51 extent: isWidth@eHeight).

	eventListView1 _ self makeEventListViewOn: EventListEditor defaultEventList.
	eventListView2 _ self makeEventListViewOn: EventListEditor defaultEventList.

	self make7ButtonRowOn: eventListView1 
				within: topView rect: lButtonRect1.
	topView 
		addSubView: (self makeAmpGaugeViewOn: eventListView1) 
				in: ((bOffset+bWidth)@bOffset 
					extent: (2*bWidth)@eHeight) borderWidth: 0;
		addSubView: eventListView1 
				in: (eOffset@0.02 extent: eWidth@eHeight) borderWidth: 1;
		addSubView: (self makeISViewOn: eventListView1) 
				in: (rButtonRect1) borderWidth: 1.

	self make7ButtonRowOn: eventListView2 
				within: topView rect: lButtonRect2.
	topView 
		addSubView: (self makeAmpGaugeViewOn: eventListView2) 
				in: ((bOffset+bWidth)@(bOffset + 0.49) 
					extent: (2*bWidth)@eHeight) borderWidth: 0;
		addSubView: eventListView2 
				in: (eOffset@0.51 extent: eWidth@eHeight) borderWidth: 1;
		addSubView: (self makeISViewOn: eventListView2) 
				in: (rButtonRect2) borderWidth: 1.

	topView controller open.!

openDualSimple
	"Open a view for a new eventListEditor composed of two editing subviews with 
simple sliders for the default amplitude"
	"HSEventListView openDualSimple"

	| eventList1 eventList2 topView eventListEditor1 eventListEditor2 
eventListView1 eventListView2 eventListEditorController1 
eventListEditorController2 instSelV1 ampV1 scrV1 menV1 instSelV2 ampV2 scrV2 
menV2 |

	eventList1 _ EventListEditor defaultEventList.							"default event list"
	eventListEditor1 _ EventListEditor new list: eventList1.		"set me up"
	eventListEditorController1 _ EventListEditorController new model: eventListEditor1.
	eventListView1 _ self new initialize; controller: eventListEditorController1; 
model: eventListEditor1.

	eventList2 _ EventListEditor defaultEventList.							"default event list"
	eventListEditor2 _ EventListEditor new list: eventList2.		"set me up"
	eventListEditorController2 _ EventListEditorController new model: eventListEditor2.
	eventListView2 _ self new initialize; controller: eventListEditorController2; 
model: eventListEditor2.
	
	ampV1  _ BarGauge on: eventListEditor1 
		aspect: #ampl change: #ampl: range: (0 to: 128 by: 32) initialValue: 80.
	ampV1 labels: true; box: ((0.5@0.03) corner: 0.94@0.97); orientation: #vertical.
	scrV1 _ SwitchView new model: (Button new onAction: [eventListView1 scrollContinuously]).
	scrV1 label: (Cursor scrollButton offset: 0@0); highlightForm: Form lightGray.
	scrV1 controller selector: #turnOn.
	menV1 _ SwitchView new model: (Button new 
			onAction: [eventListEditorController1 yellowButtonActivity]).
	menV1 label: (Form constantNamed: #menu); highlightForm: Form white.
	menV1 controller selector: #turnOn.
	instSelV1 _  InstrumentSelector new borderWidth: 2; insideColor: Form white; 
eLView: eventListView1.

	ampV2  _ BarGauge on: eventListEditor2 
		aspect: #ampl change: #ampl: range: (0 to: 128 by: 32) initialValue: 40.
	ampV2 labels: true; box: ((0.5@0.03) corner: 0.94@0.97); orientation: #vertical.
	scrV2 _ SwitchView new model: (Button new onAction: [eventListView2 scrollContinuously]).
	scrV2 label: (Cursor scrollButton offset: 0@0); highlightForm: Form lightGray.
	scrV2 controller selector: #turnOn.
	menV2 _ SwitchView new model: (Button new 
			onAction: [eventListEditorController2 yellowButtonActivity]).
	menV2 label: (Form constantNamed: #menu); highlightForm: Form white.
	menV2 controller selector: #turnOn.
	instSelV2 _  InstrumentSelector new borderWidth: 2; insideColor: Form white; 
eLView: eventListView2.

	topView _ StandardSystemView new label: 'Hauer-Steffens Event List Editor';
		minimumSize: 600@400; borderWidth: 2;
		insideColor: Form veryLightGray; icon: (Icon constantNamed: #score).

	topView addSubView: scrV1 in:(0.01@0.02 extent: 0.04@0.05) borderWidth:1;
		addSubView: menV1 in:(0.05@0.02 extent: 0.04@0.05) borderWidth:1;
		addSubView: ampV1 in:(0.01@0.07 extent: 0.08@0.42) borderWidth:1;
		addSubView: eventListView1 in:(0.1@0.02 extent: 0.85@0.47) borderWidth: 1;
		addSubView: instSelV1 in:(0.955@0.02 extent: 0.035@0.47) borderWidth:1;

		addSubView: scrV2 in:(0.01@0.51 extent: 0.04@0.05) borderWidth:1;
		addSubView: menV2 in:(0.05@0.51 extent: 0.04@0.05) borderWidth:1;
		addSubView: ampV2 in:(0.01@0.56 extent: 0.08@0.42) borderWidth:1;
		addSubView: eventListView2 in:(0.1@0.51 extent: 0.85@0.47) borderWidth: 1;
		addSubView: instSelV2 in:(0.955@0.51 extent: 0.035@0.47) borderWidth:1.

	topView controller open.!

openSimpleOn: anEL
	"Open a view for a new eventListEditor without scrollBars"
	"HSEventListView open"

	| topView eventListEditor eventListView eventListEditorController instSelV|
	eventListEditor _ EventListEditor new list: anEL.				"set me up"
	eventListEditorController _ EventListEditorController new
		model: eventListEditor.
	eventListView _ self new.
	eventListView controller: eventListEditorController;
		model: eventListEditor.
	instSelV _  InstrumentSelector new
		eLView: eventListView.
	topView _ StandardSystemView new
			label: 'Simple Event List Editor';
			minimumSize: 400@200;
			borderWidth: 1;
		addSubView: eventListView in: (0@0 extent: 0.95@1) borderWidth: 1;
		addSubView: instSelV in: (0.95@0 extent: 0.05@1) borderWidth: 1.
	topView controller open!

openWith6ButtonsOn: theEventList
	"Open a view for a new eventListEditor composed of one editing subview with 
special buttons added for standard menu operations and a spline editor"
	"HSEventListView openWith6ButtonsOn: EventListEditor defaultEventList" 

	| topView eventListView bWidth bOffset eOffset eWidth eHeight bHeight |
	topView _ StandardSystemView new label: 'New H-S Event List Editor';
			minimumSize: 500@200; borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	bOffset _ 0.02.		bWidth _ 0.035.		bHeight _ 0.96.
	eOffset _ 0.15.		eWidth _ 0.80.		eHeight _ 0.83.
	eventListView _ 
			self makeEventListViewOn: theEventList.
	self make6ButtonRowOn: eventListView 
			within: topView rect: ((bOffset@bOffset) 
						extent: (bWidth@bHeight)).
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
			in: ((bOffset+bWidth+0.01)@bOffset 
					extent: (2*bWidth)@bHeight) 
			borderWidth: 0;
		addSubView: (self makeISViewOn: eventListView) 
			in: (0.955@bOffset extent: 0.035@bHeight) 
			borderWidth: 1;
		addSubView: eventListView 
			in: (eOffset@bOffset extent: eWidth@bHeight) 
			borderWidth: 1.
	topView controller open!

openWith7ButtonsAndSplineOn: theEventList
	"Open a view for a new eventListEditor composed of one editing subview with 
special buttons added for standard menu operations and a spline editor"
	"HSEventListView openWithButtonsAndSplineOn: EventListEditor defaultEventList" 

	| topView eventListView bWidth bOffset eOffset eWidth eHeight bHeight |
	topView _ StandardSystemView new label: 'New H-S Event List Editor';
			minimumSize: 500@200; borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	bOffset _ 0.02.		bWidth _ 0.035.		bHeight _ 0.96.
	eOffset _ 0.15.		eWidth _ 0.80.		eHeight _ 0.83.
	eventListView _ 
			self makeEventListViewOn: theEventList.
	self make7ButtonRowOn: eventListView 
			within: topView rect: ((bOffset@bOffset) 
						extent: (bWidth@bHeight)).
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
			in: ((bOffset+bWidth+0.01)@bOffset 
					extent: (2*bWidth)@bHeight) 
			borderWidth: 0;
		addSubView: (self makeISViewOn: eventListView) 
			in: (0.955@bOffset extent: 0.035@bHeight) 
			borderWidth: 1;
		addSubView: (self makeSplineEdViewOn: eventListView)
			in: (eOffset@bOffset extent: eWidth@(0.98-eHeight-(2*bOffset))) 
			borderWidth: 1;
		addSubView: eventListView 
			in: (eOffset@eOffset extent: eWidth@eHeight) 
			borderWidth: 1.
	topView controller open!

openWith7ButtonsOn: theEventList
	"Open a view for a new eventListEditor composed of one editing subview with 
special buttons added for standard menu operations and a spline editor"
	"HSEventListView openWith7ButtonsOn: EventListEditor defaultEventList" 

	| topView eventListView bWidth bOffset eOffset eWidth eHeight bHeight |
	topView _ "HS"StandardSystemView new label: 'New H-S Event List Editor';
			minimumSize: 500@200; borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	bOffset _ 0.02.		bWidth _ 0.035.		bHeight _ 0.96.
	eOffset _ 0.15.		eWidth _ 0.80.		eHeight _ 0.83.
	eventListView _ 
			self makeEventListViewOn: theEventList.
	self make7ButtonRowOn: eventListView 
			within: topView rect: ((bOffset@bOffset) 
						extent: (bWidth@bHeight)).
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
			in: ((bOffset+bWidth+0.01)@bOffset 
					extent: (2*bWidth)@bHeight) 
			borderWidth: 0;
		addSubView: (self makeISViewOn: eventListView) 
			in: (0.955@bOffset extent: 0.035@bHeight) 
			borderWidth: 1;
		addSubView: eventListView 
			in: (eOffset@bOffset extent: eWidth@bHeight) 
			borderWidth: 1.
	topView controller open!

openWith8ButtonsAndHAndVScrollingOn: theEventList
	"Open a view for a new eventListEditor composed of one editing subview with 
special buttons added for standard menu operations and a spline editor"
	"HSEventListView openWith8ButtonsAndHAndVScrollingOn: EventListEditor 
defaultEventList" 

	| topView eventListView |
	topView _ "HS"StandardSystemView new 
			label: 'H-S Event List Editor';
			minimumSize: 800@260; borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	eventListView _ 
			self makeEventListViewOn: theEventList.
	self make8ButtonRowOn: eventListView 
			within: topView rect: (0.008@0.02 extent: 0.032@0.96).
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
			in: (0.043@0.02 extent: 0.06@0.96) 
			borderWidth: 0;
		addSubView: (self addVerticalScrollBarOn: eventListView) 
			in: (0.109@0.02 extent: 0.021@0.92) 
			borderWidth: 0;
		addSubView: (self addHorizontalScrollBarOn: eventListView) 
			in: (0.13@0.94 extent: 0.82@0.04) 
			borderWidth: 0;
		addSubView: eventListView 
			in: (0.13@0.02 extent: 0.82@0.92) 
			borderWidth: (1@1 extent: 0@0);
		addSubView: (self makeISViewOn: eventListView) 
			in: (0.955@0.02 extent: 0.034@0.96) 
			borderWidth: 1.
	topView controller open!

openWith8ButtonsAndHScrollingOn: theEventList
	"Open a view for a new eventListEditor composed of one editing subview with 
special buttons added for standard menu operations and a spline editor"
	"HSEventListView openWith8ButtonsAndHScrollingOn: EventListEditor 
defaultEventList" 

	| topView eventListView bWidth bOffset eOffset eWidth eHeight bHeight |
	topView _ "HS"StandardSystemView new label: 'H-S Event List Editor';
			minimumSize: 500@200; borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	bOffset _ 0.02.		bWidth _ 0.035.		bHeight _ 0.96.
	eOffset _ 0.15.		eWidth _ 0.80.		eHeight _ 0.83.
	eventListView _ 
			self makeEventListViewOn: theEventList.
	self make8ButtonRowOn: eventListView 
			within: topView rect: ((bOffset@bOffset) 
						extent: (bWidth@bHeight)).
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
			in: ((bOffset+bWidth+0.01)@bOffset 
					extent: (2*bWidth)@bHeight) 
			borderWidth: 0;
		addSubView: (self makeISViewOn: eventListView) 
			in: (0.955@bOffset extent: 0.035@bHeight) 
			borderWidth: 1;
		addSubView: (self addHorizontalScrollBarOn: eventListView) 
			in: (eOffset@0.945 extent: eWidth@0.06) 
			borderWidth: 0;
		addSubView: eventListView 
			in: (eOffset@bOffset extent: eWidth @ (bHeight - 0.04)) 
			borderWidth: 1.
	topView controller open!

openWith8ButtonsOn: theEventList
	"Open a view for a new eventListEditor composed of one editing subview with 
special buttons added for standard menu operations and a spline editor"
	"HSEventListView openWith8ButtonsOn: EventListEditor defaultEventList" 

	| topView eventListView bWidth bOffset eOffset eWidth eHeight bHeight |
	topView _ "HS"StandardSystemView new label: 'H-S Event List Editor';
			minimumSize: 500@200; borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	bOffset _ 0.02.		bWidth _ 0.035.		bHeight _ 0.96.
	eOffset _ 0.15.		eWidth _ 0.80.		eHeight _ 0.83.
	eventListView _ 
			self makeEventListViewOn: theEventList.
	self make8ButtonRowOn: eventListView 
			within: topView rect: ((bOffset@bOffset) 
						extent: (bWidth@bHeight)).
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
			in: ((bOffset+bWidth+0.01)@bOffset 
					extent: (2*bWidth)@bHeight) 
			borderWidth: 0;
		addSubView: (self makeISViewOn: eventListView) 
			in: (0.955@bOffset extent: 0.035@bHeight) 
			borderWidth: 1;
		addSubView: eventListView 
			in: (eOffset@bOffset extent: eWidth@bHeight) 
			borderWidth: 1.
	topView controller open!

openWithInsetSelectorAndSliderOn: theEventList
	"Open a view for a new eventListEditor composed of one editing subview with 
special buttons added for standard menu operations and a spline editor"
	"HSEventListView openWithInsetSelectorAndSliderOn: EventListEditor 
defaultEventList" 

	| topView eventListView eWidth sWidth |
	topView _ StandardSystemView new label: 'Event List Editor';
			minimumSize: 500@200; borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	sWidth _ 0.1.
	eWidth _ 0.94.
	eventListView _ self makeEventListViewOn: theEventList.
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
				in: (0.01@0.03 extent: (sWidth@0.94))
				borderWidth: 0;
		addSubView: eventListView
				in: ((sWidth+ 0.02)@0.03 extent: (eWidth - sWidth - 0.03)@0.94) 
				borderWidth: 1;
		addSubView: (self makeISViewOn: eventListView) 
				in: (eWidth@0.03 extent: ((1 - eWidth - 0.01)@0.94)) 
				borderWidth: 1.
	topView controller open.!

openWithSelectorAndSliderOn: theEventList
	"Open a view for a new eventListEditor composed of one editing subview with 
special buttons added for standard menu operations and a spline editor"
	"HSEventListView openWithSelectorAndSliderOn: EventListEditor 
defaultEventList" 

	| topView eventListView eWidth sWidth |
	topView _ StandardSystemView new label: 'Event List Editor';
			minimumSize: 500@200; borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	sWidth _ 0.1.
	eWidth _ 0.94.
	eventListView _ self makeEventListViewOn: theEventList.
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
				in: (0@0 extent: (sWidth@1))
				borderWidth: 0;
		addSubView: eventListView
				in: (sWidth@0 extent: (eWidth - sWidth)@1) 
				borderWidth: 1;
		addSubView: (self makeISViewOn: eventListView) 
				in: (eWidth@0 extent: ((1-eWidth)@1)) 
				borderWidth: 1.
	topView controller open.!

openWithSelectorOn: theEventList
	"Open a view for a new eventListEditor composed of one editing subview with 
special buttons added for standard menu operations and a spline editor"
	"HSEventListView openWithSelectorOn: EventListEditor defaultEventList" 

	| topView eventListView eWidth |
	topView _ StandardSystemView new label: 'Event List Editor';
			minimumSize: 400@200; borderWidth: 2;
			insideColor: Form gray; 
			icon: (Icon constantNamed: #score).
	eWidth _ 0.94.
	eventListView _ self makeEventListViewOn: theEventList.
	topView addSubView: eventListView
				in: (0@0 extent: eWidth@1) 
				borderWidth: 1;
		addSubView: (self makeISViewOn: eventListView) 
				in: (eWidth@0 extent: ((1-eWidth)@1)) 
				borderWidth: 1.
	topView controller open.! !

!HSEventListView class methodsFor: 'subview creation'!

addHorizontalScrollBarOn: eventListView
	"add a simple h-scroll bar on the given victim"

	| hScrollBar |
	hScrollBar _ BeadScrollGauge on: eventListView 
		aspect: #offsetX 
		change: #offsetX: 
		percent: #xPercent
		range: (0 to: (eventListView form width negated) by: 20).
	hScrollBar labels: false;
		orientation: #horizontal;
		box: ((0@0) corner: 1@1);
		origin: 0@0;
		bead: (Form dotOfSize: 7).
	^hScrollBar!

addVerticalScrollBarOn: eventListView
	"add a simple v-scroll bar on the given victim"

	| vScrollBar |
	vScrollBar _ BeadScrollGauge on: eventListView 
		aspect: #offsetY
		change: #offsetY: 
		percent: #yPercent
		range: (0 to: (eventListView form height negated) by: 20).
	vScrollBar labels: false;
		orientation: #vertical;
		box: ((0@0) corner: 1@1);
		origin: 0@0;
		bead: (Form dotOfSize: 7).
	^vScrollBar!

make6ButtonRowOn: eventListView within: topView rect: offRect
	"create the entire row of buttons and add them in"
	"buttons are: scroll, play, select, group, delete, inspect"

	| bWidth bHeight bXOffset bYOffset |
	bXOffset _ offRect origin x.
	bYOffset _ offRect origin y.
	bWidth _ offRect extent x.
	bHeight _ (offRect extent y) / 6.
	topView
		addSubView: (self makeScrollButtonOn: eventListView) 
				in: (bXOffset@bYOffset extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makePlayButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+bHeight) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeSelectButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*2)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeGroupButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*3)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeDeleteButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*4)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeInspectButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*5)) extent: bWidth@bHeight) 
				borderWidth:1.!

make7ButtonRowOn: eventListView within: topView rect: offRect
	"create the entire row of buttons and add them in"
	"buttons are: scroll, play, select, group, delete, inspect, redraw"

	| bWidth bHeight bXOffset bYOffset |
	bXOffset _ offRect origin x.
	bYOffset _ offRect origin y.
	bWidth _ offRect extent x.
	bHeight _ (offRect extent y) / 7.
	topView
		addSubView: (self makeScrollButtonOn: eventListView) 
				in: (bXOffset@bYOffset extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makePlayButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+bHeight) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeSelectButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*2)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeGroupButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*3)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeDeleteButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*4)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeInspectButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*5)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeRedrawButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*6)) extent: bWidth@bHeight) 
				borderWidth:1.!

make8ButtonRowOn: eventListView within: topView rect: offRect
	"create the entire row of buttons and add them in"
	"buttons are: scroll, play, select, group, delete, inspect, redraw"

	| bWidth bHeight bXOffset bYOffset |
	bXOffset _ offRect origin x.
	bYOffset _ offRect origin y.
	bWidth _ offRect extent x.
	bHeight _ (offRect extent y) / 8.
	topView
		addSubView: (self makeScrollButtonOn: eventListView) 
				in: (bXOffset@bYOffset extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makePlayButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+bHeight) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeSelectButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*2)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeGroupButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*3)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeDeleteButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*4)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeInspectButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*5)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeRedrawButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*6)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeEraseButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*7)) extent: bWidth@bHeight) 
				borderWidth:1.!

makeAmpGaugeViewOn: eventListView
	"create and return an initialized Amplitude bead gauge view"

	| ampGaugeView |
	ampGaugeView  _ BeadGauge compositeOn: eventListView model
			aspect: #ampl 
			change: #ampl: 
			percent: 0 
			range: (1 to: 127 by: 24). 
	(ampGaugeView subViews at: 1) 
			labels: #('ff' 'f' 'mf' 'mp' 'p' 'pp'); 
			box: ((0.5@0.03) corner: 0.94@0.97); 
			orientation: #vertical.
	^ampGaugeView!

makeButtonRowOn: eventListView within: topView rect: offRect
	"create the entire row of buttons and add them in"

	| bWidth bHeight bXOffset bYOffset |
	bXOffset _ offRect origin x.
	bYOffset _ offRect origin y.
	bWidth _ offRect extent x.
	bHeight _ (offRect extent y) / 6.
	topView
		addSubView: (self makeScrollButtonOn: eventListView) 
				in: (bXOffset@bYOffset extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makePlayButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+bHeight) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeSelectButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*2)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeGroupButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*3)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeDeleteButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*4)) extent: bWidth@bHeight) 
				borderWidth:1;
		addSubView: (self makeInspectButtonOn: eventListView) 
				in: (bXOffset@(bYOffset+(bHeight*5)) extent: bWidth@bHeight) 
				borderWidth:1.!

makeDeleteButtonOn: eventListView
	"create and return an initialized Delete Button"

	| deleteView |
	deleteView _ SwitchView new model: (Button new 
			onAction: [eventListView controller delete]).
	deleteView label: ((Cursor bull) deepCopy offset: 0@0); 
			highlightForm: Form white.
	deleteView controller selector: #turnOn.
	^deleteView!

makeEraseButtonOn: eventListView
	"create and return an initialized Inspect Button"

	| inspectView |
	inspectView _ SwitchView new model: (Button new 
			onAction: [eventListView controller erase]).
	inspectView label: (DisplayText text: (Text string: 'x' emphasis: 2)
								textStyle: (TextStyle styleNamed: #large)); 
			highlightForm: Form white.
	inspectView controller selector: #turnOn.
	^inspectView!

makeEventListView
	"create and return an initialized EventListView"

	| eventListEditor eventListEditorController eventListView |
	eventListEditor _ EventListEditor new 
		list: EventListEditor defaultEL.
	eventListEditorController _ EventListEditorController new 
		model: eventListEditor.
	eventListView _ self new initialize; 
		controller: eventListEditorController; 
		model: eventListEditor.
	^eventListView!

makeEventListViewOn: theEventList
	"create and return an initialized EventListView"

	| eventListEditor eventListEditorController eventListView |
	eventListEditor _ EventListEditor new 
		list: theEventList.
	eventListEditorController _ EventListEditorController new 
		model: eventListEditor.
	eventListView _ self new initialize.
	eventListView controller: eventListEditorController; 
		model: eventListEditor.
	^eventListView!

makeGroupButtonOn: eventListView
	"create and return an initialized Group Button"

	| groupView |
	groupView _ SwitchView new model: (Button new 
			onAction: [eventListView controller group]).
	groupView label: ((Cursor circle) deepCopy offset: 0@0); 
			highlightForm: Form white.
	groupView controller selector: #turnOn.
	^groupView!

makeInspectButtonOn: eventListView
	"create and return an initialized Inspect Button"

	| inspectView |
	inspectView _ SwitchView new model: (Button new 
			onAction: [eventListView controller inspect]).
	inspectView label: (Form constantNamed: #signe2); 
			highlightForm: Form white.
	inspectView controller selector: #turnOn.
	^inspectView!

makeISViewOn: eventListView
	"create and return an initialized instrument selector view"

	^InstrumentSelector new 
			borderWidth: 2; 
			insideColor: Form white; 
			eLView: eventListView!

makeMenuViewOn: eventListView
	"create and return an initialized MenuButtonView"

	| menuView |
	menuView _ SwitchView new model: (Button new 
			onAction: [eventListView controller yellowButtonActivity]).
	menuView label: (Form constantNamed: #menu); highlightForm: Form white.
	menuView controller selector: #turnOn.
	^menuView!

makePlayButtonOn: eventListView
	"create and return an initialized Play Button"

	| playView |
	playView _ SwitchView new model: (Button new 
			onAction: [eventListView controller play]).
	playView label: ((Form constantNamed: #eighthNote) copy offset: 0@0); 
			highlightForm: Form white.
	playView controller selector: #turnOn.
	^playView!

makeRedrawButtonOn: eventListView
	"create and return an initialized Group Button"

	| groupView |
	groupView _ SwitchView new model: (Button new 
			onAction: [eventListView redraw]).
	groupView label: ((Cursor thumbsUp) deepCopy offset: 0@0); 
			highlightForm: Form white.
	groupView controller selector: #turnOn.
	^groupView!

makeScrollButtonOn: eventListView
	"create and return an initialized ScrollButtonView"

	| ScrollView |
	ScrollView _ SwitchView new 
		model: (Button new onAction: [eventListView scrollContinuously]).
	ScrollView label: (Cursor scrollButton offset: 0@0); 
		highlightForm: Form lightGray.
	ScrollView controller selector: #turnOn.
	^ScrollView!

makeSelectButtonOn: eventListView
	"create and return an initialized Select Button"

	| selectView |
	selectView _ SwitchView new model: (Button new 
			onAction: [eventListView controller select]).
	selectView label: ((Cursor crossHair) deepCopy offset: 0@0); 
			highlightForm: Form white.
	selectView controller selector: #turnOn.
	^selectView!

makeSplineEdViewOn: eventListView
	"create and return an initialized spline function editor view"

	| splineEdV |
	splineEdV _  SplineEditor new
		insideColor: Form white;
		eLView: eventListView.
	^splineEdV! !

HSEventListView initialize!


Object subclass: #EventListEditor
	instanceVariableNames: 'list ostinato selection ampl '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-ListEditors'!
EventListEditor comment:
'Instances of EventListEditor are used as the models of EventListViews.
They hold onto EventLists and send messages to them.

Instance variables:
	list 			<EventList> my EventList (subject)
	ostinato 	<Ostinato> my loop process (if i''m looping)
	selection 	<EventList> The "selected" notes
	ampl 		<Number or Amplitude> My default amplitude
'!


!EventListEditor methodsFor: 'initialize-release'!

initialize
	"set up some default parameters"

	self ampl: 60! !

!EventListEditor methodsFor: 'accessing'!

ampl
	"answer the default amplitude of new events"

	^ampl!

ampl: newValue
	"set the default amplitude of new events"

	ampl _ newValue asInteger.		"coerce to integer by default"
	self changed: #ampl!

decrementampl
	"for use by the amplitude buttons"

	self ampl: (self ampl - 1)!

defaultEventClass
	"return the list's default class"

	^list defaultEventClass!

incrementampl
	"for use by the amplitude buttons"

	self ampl: (self ampl + 1)!

list
	"return my subject eventList"

	^list!

list: anEL
	"take the given EventList as my model"

	list _ anEL!

selection
	"return the collection of selected events"

	selection == nil ifFalse:
		[ ^selection ].
	^OrderedCollection new! !

!EventListEditor methodsFor: 'list access'!

add: anAss
	"plug in defaults that may be defined here"

	"assume we have a Magnitude as amplitude slider"
	self ampl == nil ifFalse:
		[ anAss value ampl: (Amplitude value: self ampl) ].
	list add: anAss.
	self changed: #added.!

eraseList

	list empty.
	self changed: #deleted.!

inspectList

	list inspect.
	self changed!

play
	"play the editor's eventList"

"	Sensor leftShiftDown 
		ifTrue: [self startStop]
		ifFalse: [list play]
"
	list play!

remove: aNote
	"remove the given note from my list"

"	Transcript show: 'remove: ', aNote printString; cr."
	list remove: aNote.
	self changed: #deleted.!

startStop
	"if not playing, create ostinato and start; if playing, stop and release."

	ostinato == nil
		ifTrue: [ostinato _ Ostinato onList: list.
				[ostinato play]]
		ifFalse: [ostinato stop.
				ostinato _ nil]! !

!EventListEditor methodsFor: 'links and groups'!

link: anEvent with: anOtherEvent
	"set up a link between the two events or lists"

	^self!

select: anA
	"add the given association to the list of selected"

	selection == nil ifTrue: [
		selection _ EventList newNamed: EventList nextName ].
	selection add: anA! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EventListEditor class
	instanceVariableNames: ''!


!EventListEditor class methodsFor: 'instance creation'!

new
	"return an initialized EventListEditor"

	^super new initialize! !

!EventListEditor class methodsFor: 'private'!

defaultEL
	"return an empty anonymous EventList."

	^EventList newNamed: #temp! !

!EventListEditor class methodsFor: 'event list creation'!

defaultEventList
	"return an empty anonymous EventList."

	^EventList newNamed: #temp! !

HSEventListView subclass: #PianoView
	instanceVariableNames: 'duration steps sharps '
	classVariableNames: 'LedgerLine '
	poolDictionaries: ''
	category: 'Music-ListEditors'!
PianoView comment:
'A PianoView is a cheap common-practise music notation-stryle of EventListEditorView.
It refines the layout and placement routines of HSEventListView for CMN-style positioning.
This is still pretty primitive.  It uses the event list''s tempoScale as a hint 
of what a whole note is.  If there is none, it assumes whole = 1 Sec.
For example, if the model has durations like 200, 400 and 800 ms and the 
tempoScale is 800, it will assume that this means whole note and 200 will be a 
1/4 note.
It does not do rests or ties yet.

Instance variable:
	duration 	<Duration> my current default duration value
	steps		<Array of SmallIntegers> the offset tqable for mapping Midi pitches sto 
y-offsets in the form
	sharps		<Array of SmallIntegers> The list of which notes require accidentals
'!


!PianoView methodsFor: 'initialize-release'!

initialize
	"set up the default variables for a 2-staff CMN editor"

	super initialize.
	self defaultParameters2.
	self initForms2.
	self CMNstaff2.			"default 2-system staves"
	cachedForm _ form deepCopy.
	self initBarLines.
	icon _ Form constantNamed: #quarterNote.
	instrument _ 1.
	dOffset _ 0@0.
	^self! !

!PianoView methodsFor: 'accessing'!

instrument: aCursor number: aNumber
	"set the instrument to be the given one (used as default duration here)"

	icon _ aCursor.
	instrument _ aNumber.
	duration _ 1/4! !

!PianoView methodsFor: 'interaction'!

eventAt: aPoint 
	"take a point and put up the right Event icon there.  
	Return a NoteEvent to the controller who passes it on to the model. "

	| origin displayPoint roundedPoint nDuration association relativePoint |
	origin _ insetDisplayBox origin.
										"point relative to whole form"
	relativePoint _ self relativeCoordinatesFor: (aPoint - origin).
"	Transcript show: ' rP: ', relativePoint printString."
										"rounded display point"
	displayPoint _  (self displayCoordinatesFor: relativePoint) + origin.	
										"show note position"
	icon displayOn: Display at: displayPoint
		clippingBox: insetDisplayBox
		rule: Form paint 
		mask: Form black.
	(Sensor redButtonPressed) 			"now track note duration"
		ifTrue: [ nDuration _ self trackEvent: displayPoint inset: insetDisplayBox ]
		ifFalse: [ nDuration _ 0. ].
	nDuration _ (MusicConstants at: #StdNoteIcons) keyAtValue: icon ifAbsent:
			[(MusicConstants at: #StdRestIcons) keyAtValue: icon ifAbsent: [nil]].
	(nDuration isKindOf: Symbol)
			ifTrue: [nDuration _ Number readFrom: (ReadStream on: nDuration asString)].
	roundedPoint _ self roundToNote: relativePoint.
						"generate a time -> event association for returning to the controller."
	association _ (roundedPoint x) =>
			(model defaultEventClass 
				dur: nDuration
				pitch: roundedPoint y 
				voice: instrument).
	Transcript show: '  ',  association printString.
									"put the note on the form."
"	self display: association selected: false.	  --not necessary if we believe 
the above scaling works
	self displayView.
"	^association!

trackEvent: pt inset: box
	"track an active event's length while active"

	| newPt oldPt x  xmin     |
	xmin _ pt x.
	x _ 0.
					"track the cursor's x-coordinate as long as the redButton's pressed"
	[Sensor redButtonPressed] whileTrue: 
			[newPt _ Sensor cursorPoint.
			newPt ~= oldPt ifTrue: 
			[x _ (newPt x max: xmin) roundTo: timeGrain.
			oldPt _ newPt]].
	^x - pt x! !

!PianoView methodsFor: 'transformations'!

formXFromTime: aTime
	"take a note's time and return a display point x coordinate relative to the form."

	^(aTime - timeOffset / timeScale) asInteger + clefForm width + 8!

formYFromPitch: aPitch
	"take a note's pitch and return a display point y coordinate relative to the form."

	| pch |
	pch _ aPitch asMidi - pitchOffset.
	((pch >= 0) and: [pch < steps size])
		ifTrue: [^steps at: (pch + 1)].
	pch < 0 ifTrue: [^152].
	pch >= steps size ifTrue: [^8]!

roundToNote: aPoint
	"take a rounded view point and justify it to my eventListEditor's note sizes."

	| time pitch |
	time _ aPoint x roundTo: timeGrain.
	pitch _ (aPoint y // pitchScale + pitchOffset) rounded.
	pitch > 36 ifTrue: [ pitch _ pitch + 2 ].
	^((time * timeScale + timeOffset) rounded) @ pitch! !

!PianoView methodsFor: 'displaying'!

display: anAss selected: aBool
	"Display an Event in an EventListEditorView--does the fancy CMN positioning, 
and draws accidentals and ledger lines"

	| anE displayPoint myBox dur pch tempoScale |
	anE _ anAss value.
	myBox _ form boundingBox.
	pch _ anE pitch asMidi.
	anE duration value = 0 ifTrue: [^nil].
	tempoScale _ (model list at: #tempoScale).
	tempoScale == nil 
		ifFalse: [dur _ 1 / (tempoScale / anE duration asMS)]
		ifTrue: [dur _ anE duration asFraction reduced].
	displayPoint _ (self formXFromTime: anAss key) @ 
				(self formYFromPitch: anE pitch).
"	Transcript show: dur printString; tab; show: displayPoint printString; cr."
	(sharps includes: (pch \\ 12)) ifTrue:
		[(Form constantNamed: #sharp)
			displayOn: form 
			at: displayPoint - (14@8)
			clippingBox: myBox 
			rule: Form paint 
			mask: Form black].
	((MusicConstants at: #StdNoteIcons) at: (dur asSymbol) ifAbsent: 
		["Transcript show: dur printString; tab."
		(MusicConstants at: #StdNoteIcons) at: ('1' asSymbol)]) 
			displayOn: form 
			at: displayPoint
			clippingBox: myBox 
			rule: Form paint 
			mask: Form black.
					"Now display ledger lines"
	pch < (pitchOffset + 5) ifTrue:
		[LedgerLine displayOn: form
			at: (displayPoint x - 8) @ 136
			clippingBox: myBox 
			rule: Form paint 
			mask: Form black.
		pch < (pitchOffset + 2) ifTrue:
			[LedgerLine displayOn: form
				at: (displayPoint x - 8) @ 144
				clippingBox: myBox 
				rule: Form paint 
				mask: Form black]].
	pch > (pitchOffset + 44) ifTrue:
		[LedgerLine displayOn: form
			at: (displayPoint x - 8) @ 24
			clippingBox: myBox 
			rule: Form paint 
			mask: Form black.
		pch > (pitchOffset + 47) ifTrue:
			[LedgerLine displayOn: form
				at: (displayPoint x - 8) @ 16
				clippingBox: myBox 
				rule: Form paint 
				mask: Form black]].
	(pch == (pitchOffset + 24) or: [pch == (pitchOffset + 25)]) ifTrue:
		[LedgerLine displayOn: form
			at: (displayPoint x - 8) @ 80
			clippingBox: myBox 
			rule: Form paint 
			mask: Form black]! !

!PianoView methodsFor: 'staves'!

CMNstaff2
	"display a 2-system CMN staff onto a form"

	| box top1 left1 right1 aNum blt |
							" set up variables "
	box _ form boundingBox.
	top1 _ box top.
	left1 _ box left + 1.
	right1 _ box right - 1.
	blt _ BitBlt destForm: form sourceForm: (Form dotOfSize: 1) 
			halftoneForm: Form black
			combinationRule: Form over 
			destOrigin: 0@0 sourceOrigin: 0@0
			extent: (right1-left1)@1 
			clipRect: box.

	aNum _ top1 + 32.
"	Transcript show: '[ ', aNum printString, ' < ', bottom1 printString, ']...'; cr. "
										" loop from top1 to bottom1 drawing lines "
	5 timesRepeat: 
		[ blt drawFrom: left1@aNum to: right1@aNum.
		aNum _ aNum + (pitchScale // 2) ].
	aNum _ aNum + (pitchScale * (3 / 2)).
	5 timesRepeat:
		[ blt drawFrom: left1@aNum to: right1@aNum.
		aNum _ aNum + (pitchScale // 2) ].
	(Form constantNamed: #trebClef) 
		displayOn: form at: 0@32 rule: Form paint.
	(Form constantNamed: #bassClef) 
		displayOn: form at: 0@93 rule: Form paint!

initBarLines
	"draw them either every second or at the tempo scale"

	(model == nil or: [(model list at: #tempoScale) == nil])
		ifTrue: [self class barLines: form scale: 1000 offset: 40]
		ifFalse: [self class barLines: form 
			scale: ((model list at: #tempoScale) / timeScale) truncated 
			offset: 28].! !

!PianoView methodsFor: 'private'!

defaultParameters2
	"set up the default layout parameters for two-staff piano sheets"

	| base stepSizes count |
	base _ 8.				"basic step size in pixels"
	timeOffset _ 0.			"view offset"
	timeScale _ 10.			"ms/pixel"
	timeGrain _ 25.			"min event offset"
	pitchOffset _ 36.		"lowest note"
	pitchScale _ base * 2.	"well tempered"
	pitchGrain _ base / 2.	"spacing between accidentals"
	duration _ 1/4.

	steps _ Array new: 49.		"array of note offsets"
	count _ 1.
	stepSizes _ #(0 0 1 1 2 3 3 4 4 5 5 6). "half-step's distances from c"
	0 to: 1 do: [:oct |			"lower 2 octaves"
		stepSizes do: [:step |
			steps at: count 
				put: (144 - (oct * pitchGrain * 7 + (step * pitchGrain))).
			count _ count + 1]].
	0 to: 1 do: [:oct |			"upper 2 octaves"
		stepSizes do: [:step |
			steps at: count 
				put: (72 - (oct * pitchGrain * 7 + (step * pitchGrain))).
			count _ count + 1]].
	steps at: 25 put: 80;		"Middle c"
		 at: 26 put: 80;		"Middle c#"
		 at: 49 put: 16.			"c two octaves above middle c"
	sharps _ #(1 3 6 8 10).	"which half-steps have accidentals"!

initForm
	"set up my graphical form for scrolling around on."

	cachedForm == nil 
		ifTrue: [ form _ self class staff]
		ifFalse: [ form _ cachedForm deepCopy ].
	self initBarLines!

initForms2
	"set up my graphical forms for scrolling around on."

	| formSize clefSize |
	formSize _ 2400@170.
	form _ Form extent: formSize.
	form white.
	clefSize _ 32@170.
	clefForm _ Form extent: clefSize.
	clefForm white.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PianoView class
	instanceVariableNames: ''!


!PianoView class methodsFor: 'class initialization'!

barLines: form scale: theScale offset: theO
	"Put barlines every other second onto the form."

	| box t1 b1 lf1 rt1 blt count |
	box _ form boundingBox.
	t1 _ box top + 1.
	b1 _ box bottom - 1.
	lf1 _ box left.
	rt1 _ box right.
	blt _ BitBlt destForm: form 
			sourceForm: (Form dotOfSize: 1) 
			halftoneForm: Form black
			combinationRule: Form over 
			destOrigin: 0@0 
			sourceOrigin: 0@0
			extent: (b1-t1)@1 
			clipRect: box.
	count _ lf1 + theO + theScale.
	[count < rt1] whileTrue: 
		[blt drawFrom: count@t1 to: count@b1.
		   count _ count + theScale].!

initialize
	"PianoView initialize"

	Staff _ HSEventListView staff.
	Clef _ HSEventListView clef.
	LedgerLine _ (Form extent: 16@1) black! !

!PianoView class methodsFor: 'instance creation'!

open2StaffAndPlayOn: anEL
	"open the pianoView, play and scroll and then go away"
	"PianoView open2StaffOnd: (EventList named : #Allemande)"

	| eventList topView eventListEditor eventListView eventListEditorController start |
	eventList _ anEL.				"given event list"
	eventListEditor _ EventListEditor new.
	eventListEditor list: eventList.	"set me up"

	topView _ StandardSystemView new
		label: 'CMN Event List Editor';
		minimumSize: 900@200; 
		maximumSize: 1024@200;
		borderWidth: 2;
		insideColor: Form gray.

	eventListEditorController _ EventListEditorController new
		model: eventListEditor.
	eventListView _ self new initialize;
		controller: eventListEditorController;
		model: eventListEditor.
	self make8ButtonRowOn: eventListView 
			within: topView 
			rect: (0.01@0.02 extent: 0.03@0.96).
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
			in: (0.045@0.02 extent: 0.06@0.96) 
			borderWidth: 0;
		addSubView: eventListView 
			in: (0.12@0.07 extent: 0.77@0.8) 
			borderWidth: 1;
		addSubView: (self addHorizontalScrollBarOn: eventListView) 
			in: (0.12@0.88 extent: 0.77@0.07) 
			borderWidth: 0;
		addSubView: (NoteSelector new insideColor: Form white; eLView: eventListView) 
			in: (0.9@0.02 extent: 0.08@0.96) 
			borderWidth:1.
	topView controller openNoTerminateDisplayAt: (500@300).
	eventListView redraw.
		start _ Time millisecondClockValue.
"		eventListEditorController model play.
"		[eventListEditorController model play] forkAt: 5.
		eventListEditorController followFrom: start.
	topView controller close.
	Processor terminateActive!

open2StaffOn: anEL
	"Open a view for a new eventListEditor without scrollBars"
	"PianoView open2StaffOnd: (EventListEditor defaultEL)"

	| eventList topView eventListEditor eventListView eventListEditorController |
	eventList _ anEL.				"given event list"
	eventListEditor _ EventListEditor new.
	eventListEditor list: eventList.	"set me up"

	topView _ StandardSystemView new
		label: 'CMN Event List Editor';
		minimumSize: 800@200; 
		maximumSize: 1024@200;
		borderWidth: 2;
		insideColor: Form gray.

	eventListEditorController _ EventListEditorController new
		model: eventListEditor.
	eventListView _ self new initialize;
		controller: eventListEditorController;
		model: eventListEditor.
	self make8ButtonRowOn: eventListView 
			within: topView 
			rect: (0.01@0.02 extent: 0.03@0.96).
	topView addSubView: (self makeAmpGaugeViewOn: eventListView) 
			in: (0.045@0.02 extent: 0.06@0.96) 
			borderWidth: 0;
		addSubView: eventListView 
			in: (0.12@0.07 extent: 0.77@0.8) 
			borderWidth: 1;
		addSubView: (self addHorizontalScrollBarOn: eventListView) 
			in: (0.12@0.88 extent: 0.77@0.07) 
			borderWidth: 0;
		addSubView: (NoteSelector new insideColor: Form white; eLView: eventListView) 
			in: (0.9@0.02 extent: 0.08@0.96) 
			borderWidth:1.
	topView controller open.! !

PianoView initialize!


PianoView subclass: #StringQView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-ListEditors'!
StringQView comment:
'StringQView is a refinement of PianoView that does layout and interaction on 
4-staves in string quartet notation (with a bass cleff, a c-cleff and two 
treble clefs).
It uses the steps and sharps arrays as in PianoView.'!


!StringQView methodsFor: 'initialize-release'!

initialize4
	"set up the default variables for a 4-staff CMN editor"

	super initialize.
	self defaultParameters4.
	self initForms4.
	self CMNstaff4.			"default 2-system staves"
	self class barLines: form scale: 20.
	icon _ Form constantNamed: #quarterNote.
	instrument _ 1.
	dOffset _ 0@-4.! !

!StringQView methodsFor: 'staves'!

CMNstaff4
	"display a 4-system CMN staff onto a form"

	| box top1 bottom1 left1 right1 aNum blt |
										" set up variables "
	box _ form boundingBox ." self insetDisplayBox."
	top1 _ box top.
	bottom1 _ box bottom.
	left1 _ box left + 1.
	right1 _ box right - 1.
	blt _ BitBlt destForm: form sourceForm: (Form dotOfSize: 1) 
			halftoneForm: Form black
			combinationRule: Form over 
			destOrigin: 0@0 sourceOrigin: 0@0
			extent: (right1-left1)@1 clipRect: Display boundingBox.
	aNum _ top1 + 24.
"	Transcript show: '[ ', aNum printString, ' < ', bottom1 printString, ']...'; 
cr. "
										" loop from top1 to bottom1 drawing lines "
	4 timesRepeat:
		[ 5 timesRepeat:
			[ blt drawFrom: left1@aNum to: right1@aNum.
			aNum _ aNum + (pitchScale // 2) ].
		aNum _ aNum + (pitchScale * 3/2) ].
	(Form constantNamed: #trebClef) displayOn: form at: 0@19 rule: Form paint.
	(Form constantNamed: #trebClef) displayOn: form at: 0@66 rule: Form paint.
	(Form constantNamed: #altoClef) displayOn: form at: 2@116 rule: Form paint.
	(Form constantNamed: #bassClef) displayOn: form at: 0@163 rule: Form paint.! !

!StringQView methodsFor: 'private'!

defaultParameters4

	| base |
	base _ 6.				"basic step size in pixels"
	timeOffset _ 0.			"view offset"
	timeScale _ 20.			"ms/pixel"
	timeGrain _ 20.			"min event offset"
	pitchOffset _ 24.		"lowest note"
	pitchScale _ base * 2.	"well tempered"
	pitchGrain _ base.		"spacing between accidentals"
	duration _ 1/4.!

initForms4
	"set up my graphical forms for scrolling around on."

	|formSize clefSize |
	formSize _ 1024@220.
	form _ Form extent: formSize.
	form fill: (0@0 extent: formSize) rule: Form over mask: Form white.
	clefSize _ 32@220.
	clefForm _ Form extent: clefSize.
	clefForm fill: (0@0 extent: clefSize) rule: Form over mask: Form white.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

StringQView class
	instanceVariableNames: ''!


!StringQView class methodsFor: 'class initialization'!

initialize
	"StringQView initialize"

	Staff _ HSEventListView staff.
	Clef _ HSEventListView clef.
	LedgerLine _ (Form extent: 16@1) black! !

!StringQView class methodsFor: 'instance creation'!

open4StaffOn: anEL
	"Open a view for a new eventListEditor without scrollBars"
	"StringQView open4StaffOn: (EventListEditor defaultEL)"

	| eventList topView eventListEditor eventListView eventListEditorController noteSelV|

	eventList _ anEL.		"given event list"
	eventListEditor _ EventListEditor new.
	eventListEditor list: eventList.	"set me up"

	eventListEditorController _ EventListEditorController new
		model: eventListEditor.
	eventListView _ self new initialize4;
		controller: eventListEditorController;
		model: eventListEditor.
	topView _ StandardSystemView new
		label: 'Simple CMN Event List Editor';
		minimumSize: 540@220; maximumSize: 1024@220;
		borderWidth: 2;
		insideColor: Form white.
	noteSelV _ NoteSelector new
"		borderWidth: 2;"
		insideColor: Form white;
		eLView: eventListView.
	topView addSubView: eventListView in: (0@0 extent: 0.95@1) borderWidth: 1.
	topView addSubView: noteSelV in: (0.95@0 extent: 0.05@1) borderWidth:1.
	topView controller open.!

open4StaffOnb: anEL
	"Open a view for a new eventListEditor without scrollBars"
	"StringQView open4StaffOnb: (EventListEditor defaultEventList)"

	| eventList topView eventListEditor eventListView eventListEditorController 
noteSelV splineEdV ampV |

	eventList _ anEL.		"given event list"
	eventListEditor _ EventListEditor new.
	eventListEditor list: eventList.	"set me up"

	eventListEditorController _ EventListEditorController new
		model: eventListEditor.
	eventListView _ self new initialize4;
		controller: eventListEditorController;
		model: eventListEditor.
	noteSelV _ NoteSelector new
"		borderWidth: 2;"
		insideColor: Form white;
		eLView: eventListView.
	splineEdV _  SplineEditor new
		insideColor: Form white";
		eLView: eventListView".
	ampV  _ BarGauge on: eventListEditor
		aspect: #ampl change: #ampl: range: (0 to: 128 by: 32) initialValue: 80.
	ampV labels: true; box: ((0.5@0.03) corner: 0.94@0.97); orientation: #vertical.

	topView _ StandardSystemView new
		label: 'Simple CMN Event List Editor';
		minimumSize: 590@280; maximumSize: 1024@280;
		borderWidth: 2;
		insideColor: Form veryLightGray.

	topView addSubView: splineEdV in: (0.14@0.02 extent: 0.75@0.2) borderWidth:1.
	topView addSubView: ampV in: (0.02@0.24 extent: 0.1@0.74) borderWidth:1.
	topView addSubView: noteSelV in: (0.91@0.24 extent: 0.07@0.74) borderWidth:1.
	topView addSubView: eventListView in: (0.14@0.24 extent: 0.75@0.74) 
borderWidth: 1.
	topView controller open.! !

StringQView initialize!
