View subclass: #PathEditor
	instanceVariableNames: 'eLView path '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-UI-Support'!


!PathEditor methodsFor: 'controller access'!

defaultControllerClass
	^PathEditorController! !

Controller subclass: #PathEditorController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-UI-Support'!


!PathEditorController methodsFor: 'control defaults'!

controlActivity
	"edit the points of the current path until the idiot presses the yellow button"

	| newPath |
	super controlActivity.
	sensor redButtonPressed ifTrue: 
		["self edit: view path points"
"		Transcript show: view path points printString; cr."
		newPath _ (view path scaleBy: view insetDisplayBox extent) 
						translateBy: view insetDisplayBox origin.
"		Transcript show: newPath points printString; cr."
		self edit: newPath points
			at: 0@0
			until: [Sensor yellowButtonPressed].
"		Transcript show: newPath points printString; cr."
		newPath _ (newPath translateBy: ((0@0) - view insetDisplayBox origin))
					 scaleFloatingBy: ((1.0@1.0) / view insetDisplayBox extent).
"		Transcript show: newPath points printString; cr."
		view path: newPath.
		view path computeCurve.
		Display white: view insetDisplayBox.
		view display
	]! !

!PathEditorController methodsFor: 'path editing'!

drag: obj at: point until: terminationBlock
	"move the given point while the given block is true"

	| previousPoint cursorPoint |
	obj displayOn: Display at: point rule: Form erase.
	Sensor cursorPoint: point .
	previousPoint  _ sensor cursorPoint.
	obj displayOn: Display at: previousPoint rule: Form reverse.
	[terminationBlock value ] whileFalse: [
		cursorPoint _ Sensor cursorPoint.
		(cursorPoint ~= previousPoint) ifTrue: [
			obj displayOn: Display
				at: previousPoint
				rule: Form reverse.
			obj displayOn: Display
				at: cursorPoint
				rule: Form reverse.
			previousPoint  _ cursorPoint.
			].
		].
	obj displayOn: Display at: previousPoint rule: Form reverse.
	obj displayOn: Display at: cursorPoint rule: Form under.
	^cursorPoint!

edit: pointCollection at: origin until: terminationBlock
	"edit the path's collection of points"

	| box dot  dPoint ePoint i hitRectangle |
	box _ view insetDisplayBox.
	dot _ Form dotOfSize: 8.
	pointCollection do: [ :item |
		dPoint _ item "* box extent + box origin".
 		dot displayOn: Display at: dPoint rule: Form reverse].
	[true] whileTrue: 
		[[Sensor anyButtonPressed not] whileTrue: [].
		(terminationBlock value) ifTrue: [^pointCollection].
		ePoint _ Sensor cursorPoint.
		i _ 0.
		(1 to: pointCollection size) do: [ :p |
			hitRectangle _ (pointCollection at: p) + origin + dot offset extent: 10@10.
			(hitRectangle containsPoint: ePoint) ifTrue: [i _ p]].
		i > 0 ifTrue: [ 
			ePoint _ self drag: dot
					at: (pointCollection at: i) + origin
					until: [Sensor redButtonPressed not ].
			pointCollection at: i put: (ePoint - origin)]]! !

DictionaryInspector subclass: #EventListDictionaryInspector
	instanceVariableNames: ''
	classVariableNames: 'EvListDictListMenu '
	poolDictionaries: ''
	category: 'Music-UI-Support'!


!EventListDictionaryInspector methodsFor: 'field list'!

fieldMenu
	"EventListDictionaryInspector flushMenus" 

	field == nil ifTrue:
		[^ActionMenu
			labels: 'inspect dictionary\update list' withCRs
			lines: nil
			selectors: #(inspectDict updateList)].
	EvListDictListMenu == nil ifTrue:
		[EvListDictListMenu _ ActionMenu
			labels: 'inspect\references\play\edit\rename\remove' withCRs
			lines: #(2 4)
			selectors: #(inspectField browseReferences
					 play edit renameField removeField)].
	^EvListDictListMenu! !

!EventListDictionaryInspector methodsFor: 'menu items'!

edit
	"edit the selected list"

	^(object at: field) edit!

inspectDict
	EventLists inspect!

play
	"play the selected list"

	^(object at: field) play!

playOnceAndScroll
	"play the selected list"

	^(object at: field) playOnceAndScroll!

renameField 
	"rename the selected item"

	| name |
	name _ FillInTheBlank request: 'new name:'
			initialAnswer: field.
	((name = '') or: [ name = field ]) ifTrue: [ ^nil ].
	object at: name asSymbol put: (object at: field).
	object removeKey: field.
	self changed: #field!

updateList
	"update the list"

	self changed: #field! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EventListDictionaryInspector class
	instanceVariableNames: ''!


!EventListDictionaryInspector class methodsFor: 'initialization'!

flushMenus
	EvListDictListMenu _ nil! !

Inspector subclass: #EventInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-UI-Support'!


!EventInspector methodsFor: 'field list'!

fieldIndex
	"Answer the offset corresponding to the currently selected field."

	^object class allInstVarNames indexOf: field!

fieldList
	"Answer an Array consisting of 'self' and the instance variable names of the inspected object."

	^(Array with: 'self'), object class allInstVarNames,
		(object properties == nil 
			ifFalse: [object properties keys asSortedCollection]
			ifTrue: [nil])!

fieldMenu

	field == nil ifTrue:
		[ ^ActionMenu 
			labels: 'add property' withCRs
			selectors: #(addProperty) ]
		ifFalse: [ ^ActionMenu
			labels: 'inspect\remove property' withCRs
			selectors: #(inspectField removeProperty)].!

fieldValue

	field = 'self' ifTrue: [^object].
"	self fieldIndex > object class instSize"
	self fieldIndex = 0
		ifTrue: [^object properties at: field]
		ifFalse: [^object instVarAt: self fieldIndex]! !

!EventInspector methodsFor: 'menu commands'!

addProperty
		
	| key value |
	key _ FillInTheBlank request: 'property key?'
			initialAnswer: ''.
	key = '' ifTrue: [ ^nil ].
	value _ FillInTheBlank request: 'property value?'
			initialAnswer: ''.
	value = '' ifTrue: [ ^nil ].
	Sensor leftShiftDown ifTrue:
		[value _ self evaluateText: value from: self 
				ifFail: [ self error: 'cannot evaluate text...']].
	object properties at: key asSymbol put: value.
	self changed: #field!

removeProperty
	(BinaryChoice message: 'remove property ', field printString)
		ifTrue: [ object properties removeKey: field asSymbol.
			self changed: #field ]! !

!EventInspector methodsFor: 'doIt/accept/explain'!

acceptText: aText from: aController
	| val |
	field == nil ifTrue: [^false].
	val _ self evaluateText: aText string from: aController ifFail: [^ false].
	self fieldIndex = 0
		ifFalse: [object instVarAt: self fieldIndex put: val]
		ifTrue: [object properties at: field put: val].
	self changed: #text.
	^ true! !

View subclass: #InstrumentSelector
	instanceVariableNames: 'eLView '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-UI-Support'!


!InstrumentSelector methodsFor: 'accessing'!

eLView
	^eLView!

eLView: anEventListView
	eLView _ anEventListView! !

!InstrumentSelector methodsFor: 'displaying'!

displayView
	
	| number point icons |
	icons _ MusicConstants at: #NoteIcons.
	number _ icons size.
	point _ self insetDisplayBox origin + (7@12).		"magic numbers here"
	1 to: number do: [ :aNum |
		(icons at: aNum) displayOn: Display at: point 
			clippingBox: self insetDisplayBox rule: Form reverse mask: Form black.
		point _ point + (0@24)].! !

!InstrumentSelector methodsFor: 'controller access'!

defaultControllerClass
	^InstrumentSelectorController! !

!InstrumentSelector methodsFor: 'selecting'!

pickIcon: num

	| box count |
	count _ num // 24 + 1.
	box _ (self insetDisplayBox origin + (0@num) 
		extent: insetDisplayBox extent x @ 24).
	Display reverse: box.
	(Delay forMilliseconds: 300) wait.
	Display reverse: box.
	eLView instrument: ((MusicConstants at: #NoteIcons) at: count) 
		number: count! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

InstrumentSelector class
	instanceVariableNames: ''!


!InstrumentSelector class methodsFor: 'class initialization'!

initialize
	"InstrumentSelector initialize."

	self initializeIcons.
	self initializeHLIcons.!

initializeHLIcons
	"InstrumentSelector initialize."

	| array2 |
	array2 _ Array new: 10.
	array2 at: 1 put: (Cursor extent: 16@16
		fromArray: #( 0 0 0 4104 2064 960 1632 3504 3504 1632 960 2064 4104 0 0 0) offset: -8@-8).
	array2 at: 2 put: (Cursor extent: 16@16
		fromArray: #( 0 0 0 4104 2448 960 2016 4080 4080 2016 960 2448 4104 0 0 0) offset: -8@-8).
	array2 at: 3 put: (Cursor extent: 16@16
		fromArray: #( 0 0 8192 4616 784 1920 1984 4064 4080 8184 0 2064 4104 0 0 0) offset: -7@-7).
	array2 at: 4 put: (Cursor extent: 16@16
		fromArray: #( 0 4 8648 5104 2032 4088 8184 8184 8176 4064 4032 5008 8200 0 0 0) offset: -7@-7).
	array2 at: 5 put: (Cursor extent: 16@16
		fromArray: #( 0 0 0 9156 6120 3120 6168 12300 12300 6168 3120 6120 9156 0 0 0) offset: -8@-8).
	array2 at: 6 put: (Cursor extent: 16@16
		fromArray: #( 0 0 0 0 2336 1344 256 8176 256 1344 2336 0 0 0 0 0) offset: -7@-7).
	array2 at: 7 put: (Cursor extent: 16@16
		fromArray: #( 0 0 0 4368 2336 896 1984 16376 1984 896 2336 4368 0 0 0 0) offset: -7@-7).
	array2 at: 8 put: (Cursor extent: 16@16
		fromArray: #( 0 0 256 256 2336 1088 0 30780 0 1088 2336 256 256 0 0 0) offset: -7@-7).
	array2 at: 9 put: (Cursor extent: 16@16
		fromArray: #( 0 0 0 4100 2184 864 1040 2056 1040 864 2184 4100 0 0 0 0) offset: -8@-9).
	array2 at: 10 put: (Cursor extent: 16@16
		fromArray: #( 0 0 0 2064 1056 384 960 960 384 1056 2064 0 0 0 0 0) offset: -8@-8).
	(Smalltalk at: #MusicConstants) at: #HighlightIcons put: array2!

initializeIcons
	"InstrumentSelector initialize."

	| array1 array2 |
	array1 _ Array new: 10.
	array1 at: 1 put: Cursor note.
	array1 at: 2 put: Cursor dot.
	array1 at: 3 put: Cursor triangle.
	array1 at: 4 put: Cursor bDot.
	array1 at: 5 put: Cursor hole.
	array1 at: 6 put: Cursor z1.
	array1 at: 7 put: Cursor z2.
	array1 at: 8 put: Cursor spec.
	array1 at: 9 put: Cursor circle.
	array1 at: 10 put: Cursor spot.
	(Smalltalk at: #MusicConstants) at: #NoteIcons put: array1.! !

InstrumentSelector initialize!


Controller subclass: #InstrumentSelectorController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-UI-Support'!


!InstrumentSelectorController methodsFor: 'control defaults'!

controlActivity 
	"Wait for a button press and pick a new icon."

	| y icon |
"	super controlActivity."
	(sensor anyButtonPressed) ifTrue: [
		y _ (sensor cursorPoint - view insetDisplayBox origin) y. 
		view pickIcon: y.
		Sensor waitNoButton.
	]!

isControlActive
	"I'm active even if Blue is pressed (unless control, etc.)"

	^(view containsPoint: sensor cursorPoint)! !

PathEditor subclass: #SplineEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-UI-Support'!


!SplineEditor methodsFor: 'displaying'!

display

	insetDisplayBox == nil ifFalse: [
	  path == nil ifTrue: [
"		path _ Spline new form: (Form dotOfSize: 2).
		path add: (insetDisplayBox origin + (0 @ (insetDisplayBox extent y / 2)));
			add: (insetDisplayBox center + (10@10));
			add: (insetDisplayBox corner - (0 @ (insetDisplayBox extent y / 2)));
			computeCurve;
			display
"
self halt
	]].
	super display!

displayView
	"display the points"

	| path2 | 
	  path == nil ifTrue: [
			path _ Spline new form: (Form dotOfSize: 2).
			path add: (0 @0.5);
				add: (0.2@0.4);
				add: (0.7@0.6);
				add: (1 @0.5) ].
	path2 _ (path scaleBy: insetDisplayBox extent) translateBy: insetDisplayBox origin.
	path2 computeCurve.
	path2 display! !

!SplineEditor methodsFor: 'controller access'!

defaultControllerClass
	^PathEditorController! !

!SplineEditor methodsFor: 'accessing'!

eLView

	"Return the instance variable 'eLView'."

	^ eLView!

eLView: theELView

	"Accept the argument, 'theELView', as the new instance variable 'eLView'."

	eLView _ theELView.!

path

	"Return the instance variable 'path'."

	^ path!

path: thePath

	"Accept the argument, 'thePath', as the new instance variable 'path'."

	path _ thePath.! !

InspectorView subclass: #EventInspectorView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-UI-Support'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EventInspectorView class
	instanceVariableNames: ''!


!EventInspectorView class methodsFor: 'private'!

addListButtonsTo: aView in: aRectangle for: model
	"listView buttons are: inspect"

	| number size offset button |
	number _ 2.
	offset _ aRectangle width / number.
	size _ aRectangle origin extent: (offset @ aRectangle height).
	button _ self makeActionButtonFor: (Form constantNamed: #inspect)  selector: #inspectField on: model.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: 'add prop' asDisplayText selector: #addProperty on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.!

buildScheduledViewWithButtonsOn: anInspector
	"build a newStyle inspector view on the argument"
 
	| topView |
	topView _ super buildScheduledViewWithButtonsOn: anInspector.
	topView label: 'Event Inspector'.
"	topView cacheRefresh: false."
	^ topView! !

EventInspectorView subclass: #EventListDictionaryInspectorView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-UI-Support'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EventListDictionaryInspectorView class
	instanceVariableNames: ''!


!EventListDictionaryInspectorView class methodsFor: 'private'!

addListButtonsTo: aView in: aRectangle for: model
	"listView buttons are: inspect play edit"

	| number size offset button form |
	number _ 4.
	offset _ aRectangle width / number.
	size _ aRectangle origin extent: (offset @ aRectangle height).
	button _ self makeActionButtonFor: (Form constantNamed: #inspect) 
		selector: #inspectField on: model.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: (((Form constantNamed: #eighthNote) deepCopy) offset: 0@0)
		selector: #play on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.
	button _ self makeActionButtonFor: ((((Icon constantNamed: #score) form figure) deepCopy) offset: 0@2)
		selector: #edit on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.
	form _ ((Form constantNamed: #eighthNote) deepCopy) offset: 0@0.
	form reverse.
	button _ self makeActionButtonFor: form
		selector: #playOnceAndScroll on: model.
	size _ size moveBy: offset@0.
	aView addSubView: button in: size borderWidth: 1.!

buildScheduledViewOn: anInspector
	"build a newStyle inspector view on the argument"
 
	| topView |
	topView _ super buildScheduledViewWithButtonsOn: anInspector.
	topView label: 'Event List Dictionary'.
"	topView cacheRefresh: false."
	^ topView!

buildScheduledViewWithButtonsOn: anInspector
	"build a newStyle inspector view on the argument"
 
	| topView |
	topView _ super buildScheduledViewWithButtonsOn: anInspector.
	topView label: 'Event List Dictionary'; minimumSize: 600 @ 280.

"	topView cacheRefresh: false."
	^ topView! !

InstrumentSelector subclass: #NoteSelector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music-UI-Support'!


!NoteSelector methodsFor: 'displaying'!

displayView
	
	| number point icons |
	icons _ Array new: 6.
	number _ 1.
	#('1' '1/2' '1/4' '1/8' '1/16' '1/32') do: 
		[:dur |	
		icons at: number 
			put: ((MusicConstants at: #StdNoteIcons) at: dur asSymbol).
		number _ number + 1].
	number _ icons size.
	point _ self insetDisplayBox origin + (8@12).
	1 to: number do: [ :aNum |
		(icons at: aNum) displayOn: Display at: point 
				clippingBox: self insetDisplayBox rule: Form reverse mask: Form black.
		point _ point + (0@32)].
	point _ self insetDisplayBox origin + (28@8).
	number _ 1.
	#('1/1' '1/2' '1/4' '1/8' '1/16' '1/32') do: 
		[:dur |	
		icons at: number 
			put: ((MusicConstants at: #StdRestIcons) at: dur asSymbol).
		number _ number + 1].
	number _ icons size.
	1 to: number do: [ :aNum |
		(icons at: aNum) displayOn: Display at: point 
				clippingBox: self insetDisplayBox rule: Form reverse mask: Form black.
		point _ point + (0@32)].! !

!NoteSelector methodsFor: 'selecting'!

pickIcon: num

	| list box count |
	list _ #('1/1' '1/2' '1/4' '1/8' '1/16' '1/32'). 
	count _ num // 32 + 1.
	box _ (self insetDisplayBox origin + (0@(count - 1 * 32)) 
		extent: insetDisplayBox extent x @ 32).
	Display reverse: box.
	(Delay forMilliseconds: 300) wait.
	Display reverse: box.
	count < 8 
		ifTrue: [eLView instrument: 
				((MusicConstants at: #StdNoteIcons) at: ((list at: count) asSymbol))
			number: count]
		ifFalse: [eLView instrument: 
				((MusicConstants at: #StdRestIcons) at: ((list at: count) asSymbol)) 
			number: count]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NoteSelector class
	instanceVariableNames: ''!


!NoteSelector class methodsFor: 'class initialization'!

initialize
	"NoteSelector initialize."

	| dict |
	dict _ Dictionary new: 6.
	dict at: '1' asSymbol put: ((Form constantNamed: #wholeNote) offset: -6@-8).
	dict at: '1/2' asSymbol put: ((Form constantNamed: #halfNote) offset: -6@-15).
	dict at: '1/4' asSymbol put: ((Form constantNamed: #quarterNote) offset: -6@-16).
	dict at: '1/8' asSymbol put: ((Form constantNamed: #eighthNote) offset: -4@-16).
	dict at: '1/16' asSymbol put: ((Form constantNamed: #sixteenthNote) offset: -4@-16).
	dict at: '1/32' asSymbol put: ((Form constantNamed: #thirtySecondNote) offset: -4@-16).
	(Smalltalk at: #MusicConstants) at: #StdNoteIcons put: dict.

	dict _ Dictionary new: 6.
	dict at: '1/1' asSymbol put: ((Form constantNamed: #wholeRest) offset: -7@-9).
	dict at: '1/2' asSymbol put: ((Form constantNamed: #halfRest) offset: -6@-9).
	dict at: '1/4' asSymbol put: ((Form constantNamed: #quarterRest) offset: -6@-9).
	dict at: '1/8' asSymbol put: ((Form constantNamed: #eighthRest) offset: -7@-8).
	dict at: '1/16' asSymbol put: ((Form constantNamed: #sixteenthRest) offset: -7@-8).
	dict at: '1/32' asSymbol put: ((Form constantNamed: #thirtySecondRest) offset: -7@-8).
	(Smalltalk at: #MusicConstants) at: #StdRestIcons put: dict.! !

NoteSelector initialize!
